#ifndef _SILKWORM_40X0_MISC_H
#define _SILKWORM_40X0_MISC_H

#include <linux/types.h>

struct silkworm_pcie_shared_mem {
    // synchronize the PCI scan and init between active and standby
    // We store the synch flag between active and standby in fsl,failover-shared at
    // offset of 1KB. The first 9 bytes of this 4K memory is used by FMAN.

    volatile bool config_done;   // config of RC done
    volatile bool scan_done;     // PCI device scan done
    volatile unsigned long pci_lock; // PCI serialization lock between active/standby GOS
    volatile unsigned long i2c_lock; // I2C serialization lock between active/standby GOS
};

typedef enum {
    STATUS_UNKNOWN, 
    STATUS_ACTIVE, 
    STATUS_STANDBY, 
    STATUS_NA 
} silkworm_partition_status; 



// return true if LC is in active/standby mode
// return false if only one partition is present on LC.
bool silkworm_is_ha_config(void);

// return true if PB needs dual partition and shared plat chunk needs to be present
// return false if only one partition is present on PB.
bool silkworm_is_plat_wmem_config(void);

// return STATUS_NA if active/standby status is not relevant, e.g. on MM or not
// running with HV.
// return STATUS_ACTIVE  for ACTIVE
// return STATUS_STANDBY for STANDBY
// returns the information from the device tree held with HV.
silkworm_partition_status silkworm_get_partition_status(void);

// handle = from the device tree ... -1 implies self
// return STATUS_NA if active/standby status is not relevant, e.g. on MM or not
// running with HV.
// return STATUS_ACTIVE  for ACTIVE
// return STATUS_STANDBY for STANDBY
// returns the information from the device tree held with HV.
silkworm_partition_status silkworm_get_partition_status_by_handle(int handle);

// return the partition id of self.
// should be used by code that need this infromation for any purpose.
// -1 on error e.g. use in case w/o HV.
// partition id is read from /chosen/label which is of the following format
// p<id>-linux.
int silkworm_get_partition_id(void);

// set the current status of partition as active/standby
// status = STATUS_ACTIVE or STATUS_STANDBY
// who = 0 implies self, who = 1 implies the other partition as applicable.
// The function sets the status of the partition in the device tree with HV.
// in case of self, it also sets the status in the GOS local device tree.
// should be called after a partition has gone active from standby status for
// self and the other partition is stopped.
int silkworm_set_partition_status(silkworm_partition_status status, int who);


int silkworm_claim_localbus(void);
int silkworm_lc_get_cpldirq(void);
int silkworm_lc_get_phyirq(void);

#endif // _SILKWORM_40X0_MISC_H
