/*
 * Brocade platform support for all the Silkworm products.
 * Based on: arch/ppc/platforms/pmac_features.c - Copyright 2002 MontaVista SW
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 or the License, or (at your
 * option) any later version.
 */

#include <linux/init.h>
#include <linux/types.h>
#include <linux/errno.h>
#include <stdarg.h>

#include <brocade/cpld.h>
#include <brocade/platform.h>
#include <brocade/silkworm.h>
#include <brocade/silkworm_features.h>

extern sw_bd silkworm;

/* Forward declaration */
static int silkworm_get_memhole_physaddr(int param, int value);

static silkworm_feature_tbe_t	silkworm_features[] = {
	{ SILKWORM_FTR_GET_MEM_HOLE,	silkworm_get_memhole_physaddr },
	{ 0, NULL }
};

silkworm_mb_def_t silkworm_mb = {
	 "All boards", "Silkworm", NULL, silkworm_features
};

/*
 * Function: silkworm_get_mem_hole
 * Input :
 * 		param : Pointer to struct with phys_addr (phys_addr_t and size)
 * 
 * Returns:
 *		sets the value in the data structure passed
 *
 * Errors:
 *		returns -ENODEV if current platform does not support it.	
 */
static int
silkworm_get_memhole_physaddr(int param, int value)
{
	silkworm_mem_hole_t *memh_p = (silkworm_mem_hole_t *)param;

	if (!memh_p)
		return -EINVAL;

	switch (silkworm.board) {
	case SWBD21:
	case SWBD42:
	case SWBD46:
			memh_p->offset = (unsigned int)(MET_CONDOR_DESC_BASE_ADDR);
			memh_p->size = (unsigned int)(MET_CONDOR_DESC_SIZE);
			break;
	case SWBD62: /* neptune CP */
	case SWBD77: /* pluto CP */
	case SWBD141:
	case SWBD142:
			memh_p->offset = (unsigned int)(MAINCPU_RESERVED_MEM_BASE_ADDR);
			memh_p->size = (unsigned int)(MAINCPU_RESERVED_MEM_SIZE);
			break;
	default:
		return -ENODEV;
	}  

	return (0);
}

long
ppc44x_do_feature_call(unsigned int selector, ...)
{
	int i;
	unsigned int param, value;
	feature_call func = NULL;
	va_list args;

	if (silkworm_mb.features) {
		for (i=0; silkworm_mb.features[i].function; i++) {
			if (silkworm_mb.features[i].selector == selector) {
				func = silkworm_mb.features[i].function;
			}
		}
	}

	if (!func)
		return -ENODEV;
	
	va_start(args, selector);
	param = va_arg(args, int);
	value = va_arg(args, int);
	va_end(args);

	return func(param, value);
}
