/*
 * FSL SoC setup code
 *
 * Maintained by Kumar Gala (see MAINTAINERS for contact information)
 *
 * 2006 (c) MontaVista Software, Inc.
 * Vitaly Bordug <vbordug@ru.mvista.com>
 *
 * Copyright 2010-2011 Freescale Semiconductor, Inc.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */

#include <linux/stddef.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/errno.h>
#include <linux/major.h>
#include <linux/delay.h>
#include <linux/irq.h>
#include <linux/module.h>
#include <linux/device.h>
#include <linux/platform_device.h>
#include <linux/of.h>
#include <linux/of_platform.h>
#include <linux/phy.h>
#include <linux/phy_fixed.h>
#include <linux/spi/spi.h>
#include <linux/fsl_devices.h>
#include <linux/fs_enet_pd.h>
#include <linux/fs_uart_pd.h>
#ifdef CONFIG_FSL_HYPERVISOR
#include <linux/fsl_hypervisor.h>
#endif
#include <asm/system.h>
#include <asm/atomic.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/time.h>
#include <asm/prom.h>
#include <asm/machdep.h>
#include <sysdev/fsl_soc.h>
#include <mm/mmu_decl.h>
#include <asm/cpm2.h>
#include <asm/mpc85xx.h>

#ifdef CONFIG_FSL_HYPERVISOR
/*
 * Platforms that use HV cannot get the CCSRBAR value
 * from the device tree. In that case, default to the
 * address that is known bo be used by those platforms.
 */
#define CCSRBAR  0xffe000000ull
#endif

extern void init_fcc_ioports(struct fs_platform_info*);
extern void init_fec_ioports(struct fs_platform_info*);
extern void init_smc_ioports(struct fs_uart_platform_info*);
static phys_addr_t immrbase = -1;
/*
static int pamu_enable_liodn_dummy(int liodn_index)
{
	return 0;
}
*/
/**
 * pamu_enable_liodn
 * @of_node: Device tree node containing LIODN(s)
 * @liodn_index: Zero-based index into the array of LIODNs provided,
 *		 or -1 for all LIODNs in the node.
 *
 * Returns zero on success, negative on error.
 */
//	int (*pamu_enable_liodn)(int liodn_index) = pamu_enable_liodn_dummy;

phys_addr_t get_immrbase(void)
{
	struct device_node *soc;

	if (immrbase != -1)
		return immrbase;

	soc = of_find_node_by_type(NULL, "soc");
	if (soc) {
		int size;
		u32 naddr;
		const u32 *prop = of_get_property(soc, "#address-cells", &size);

		if (prop && size == 4)
			naddr = *prop;
		else
			naddr = 2;

		prop = of_get_property(soc, "ranges", &size);
		if (prop)
			immrbase = of_translate_address(soc, prop + naddr);

		of_node_put(soc);
	}
#ifdef CONFIG_FSL_HYPERVISOR
	  else if (has_fsl_hypervisor()){
		immrbase = (phys_addr_t) CCSRBAR;
	}
#endif

	return immrbase;
}

EXPORT_SYMBOL(get_immrbase);

static u32 sysfreq = -1;

u32 fsl_get_sys_freq(void)
{
	struct device_node *soc;
	const u32 *prop;
	int size;

	if (sysfreq != -1)
		return sysfreq;

	soc = of_find_node_by_type(NULL, "soc");
	if (!soc)
		return -1;

	prop = of_get_property(soc, "clock-frequency", &size);
	if (!prop || size != sizeof(*prop) || *prop == 0)
		prop = of_get_property(soc, "bus-frequency", &size);

	if (prop && size == sizeof(*prop))
		sysfreq = *prop;

	of_node_put(soc);
	return sysfreq;
}
EXPORT_SYMBOL(fsl_get_sys_freq);

#if defined(CONFIG_CPM2) || defined(CONFIG_QUICC_ENGINE) || defined(CONFIG_8xx)

static u32 brgfreq = -1;

u32 get_brgfreq(void)
{
	struct device_node *node;
	const unsigned int *prop;
	int size;

	if (brgfreq != -1)
		return brgfreq;

	node = of_find_compatible_node(NULL, NULL, "fsl,cpm-brg");
	if (node) {
		prop = of_get_property(node, "clock-frequency", &size);
		if (prop && size == 4)
			brgfreq = *prop;

		of_node_put(node);
		return brgfreq;
	}

	/* Legacy device binding -- will go away when no users are left. */
	node = of_find_node_by_type(NULL, "cpm");
	if (!node)
		node = of_find_compatible_node(NULL, NULL, "fsl,qe");
	if (!node)
		node = of_find_node_by_type(NULL, "qe");

	if (node) {
		prop = of_get_property(node, "brg-frequency", &size);
		if (prop && size == 4)
			brgfreq = *prop;

		if (brgfreq == -1 || brgfreq == 0) {
			prop = of_get_property(node, "bus-frequency", &size);
			if (prop && size == 4)
				brgfreq = *prop / 2;
		}
		of_node_put(node);
	}

	return brgfreq;
}

EXPORT_SYMBOL(get_brgfreq);

static u32 fs_baudrate = -1;

u32 get_baudrate(void)
{
	struct device_node *node;

	if (fs_baudrate != -1)
		return fs_baudrate;

	node = of_find_node_by_type(NULL, "serial");
	if (node) {
		int size;
		const unsigned int *prop = of_get_property(node,
				"current-speed", &size);

		if (prop)
			fs_baudrate = *prop;
		of_node_put(node);
	}

	return fs_baudrate;
}

EXPORT_SYMBOL(get_baudrate);
#endif /* CONFIG_CPM2 */

#ifdef CONFIG_FIXED_PHY
#if defined(CONFIG_SILKWORM) && !defined(CONFIG_SILKWORM_LC) && !defined(CONFIG_SILKWORM_MM) && !defined(CONFIG_SILKWORM_PB) && !defined(CONFIG_SILKWORM_EMBEDDED)
struct fixed_phy_status *of_add_fixed_phys(u32 *fixed_link)
{
	struct fixed_phy_status *status;

	status = (struct fixed_phy_status *) kmalloc(
			sizeof(struct fixed_phy_status), GFP_KERNEL);

	if (status) {
		status->link =  1;  /* standard fixed_link[0] is phyid */
		status->duplex = fixed_link[1];
		status->speed = fixed_link[2];
		status->pause = fixed_link[3];
		status->asym_pause = fixed_link[4];
	}
	return status;
}
#else
static int __init of_add_fixed_phys(void)
{
	int ret;
	struct device_node *np;
	u32 *fixed_link;
	struct fixed_phy_status status = {};

	for_each_node_by_name(np, "ethernet") {
		fixed_link  = (u32 *)of_get_property(np, "fixed-link", NULL);
		if (!fixed_link)
			continue;

		status.link = 1;
		status.duplex = fixed_link[1];
		status.speed = fixed_link[2];
		status.pause = fixed_link[3];
		status.asym_pause = fixed_link[4];

		ret = fixed_phy_add(PHY_POLL, fixed_link[0], &status);
		if (ret) {
			of_node_put(np);
			return ret;
		}
	}

	return 0;
}
arch_initcall(of_add_fixed_phys);
#endif /* CONFIG_SILKWORM */
#endif /* CONFIG_FIXED_PHY */

static enum fsl_usb2_phy_modes determine_usb_phy(const char *phy_type)
{
	if (!phy_type)
		return FSL_USB2_PHY_NONE;
	if (!strcasecmp(phy_type, "ulpi"))
		return FSL_USB2_PHY_ULPI;
	if (!strcasecmp(phy_type, "utmi"))
		return FSL_USB2_PHY_UTMI;
	if (!strcasecmp(phy_type, "utmi_wide"))
		return FSL_USB2_PHY_UTMI_WIDE;
	if (!strcasecmp(phy_type, "serial"))
		return FSL_USB2_PHY_SERIAL;

	return FSL_USB2_PHY_NONE;
}

static bool has_erratum_a005275(struct device_node *node)
{
	unsigned int svr = mfspr(SPRN_SVR);
	bool flag = false;
	/* Deal with USB Erratum USB A-005275
	 * Packet corruption in HS mode, default to
	 * FS mode for the following
	 * P3041 and P2041 rev 1.0 and 1.1
	 * P5020 and P5010 rev 1.0 and 2.0
	 * P5040 and P1010 rev 1.0
	 */
	if ((SVR_SOC_VER(svr) == SVR_P2041) ||
			(SVR_SOC_VER(svr) == SVR_P3041))
		flag = (SVR_REV(svr) == 0x10) || (SVR_REV(svr) == 0x11);
	else if ((SVR_SOC_VER(svr) == SVR_P5010) ||
			(SVR_SOC_VER(svr) == SVR_P5020))
		flag = (SVR_REV(svr) == 0x10) || (SVR_REV(svr) == 0x20);
	else if ((SVR_SOC_VER(svr) == SVR_P5040) ||
			(SVR_SOC_VER(svr) == SVR_P1010))
		flag = (SVR_REV(svr) == 0x10);

	printk(KERN_INFO "FSL-SOC USB %s %s\n", __FUNCTION__, flag ? "true" : "false");
	return flag;
}

static bool has_erratum_a005697(void)
{
	unsigned int svr = mfspr(SPRN_SVR);
	bool flag = false;

	switch (SVR_SOC_VER(svr)) {
	case SVR_T1022:
	case SVR_T1040:
		if ((SVR_REV(svr) == 0x10) || (SVR_REV(svr) == 0x11))
			flag = true;
		break;
	case SVR_P5040:
	case SVR_P5021:
		if ((SVR_REV(svr) == 0x10) || (SVR_REV(svr) == 0x20) ||
				(SVR_REV(svr) == 0x21))
			flag = true;
		break;
	case SVR_T4240:
	case SVR_T4160:
	case SVR_P5020:
	case SVR_P5010:
		if ((SVR_REV(svr) == 0x10) || (SVR_REV(svr) == 0x20))
			flag = true;
		break;
	case SVR_P2041:
	case SVR_P3041:
		if ((SVR_REV(svr) == 0x10) || (SVR_REV(svr) == 0x11) ||
				(SVR_REV(svr) == 0x20))
			flag = true;
		break;
	case SVR_P4080:
		if ((SVR_REV(svr) == 0x10) || (SVR_REV(svr) == 0x20) ||
				(SVR_REV(svr) == 0x30))
			flag = true;
		break;
	}

	printk(KERN_INFO "FSL-SOC USB %s %s\n", __FUNCTION__, flag ? "true" : "false");
	return flag;
}

static bool has_erratum_a007792(int controller_ver)
{
	unsigned int svr = mfspr(SPRN_SVR);
	bool flag = false;

	switch (SVR_SOC_VER(svr)) {
		case SVR_T4240:
		case SVR_T4160:
			if (SVR_REV(svr) == 0x20)
				flag = true;
			break;
		case SVR_T1023:
		case SVR_T1024:
			if (SVR_REV(svr) == 0x10)
				flag = true;
			break;
		case SVR_T1020:
		case SVR_T1022:
		case SVR_T1040:
		case SVR_T1042:
		case SVR_T2080:
		case SVR_T2081:
			if ((SVR_REV(svr) == 0x10) || (SVR_REV(svr) == 0x11))
				flag = true;
			break;
	}

	printk(KERN_INFO "FSL-SOC USB %s %s\n", __FUNCTION__, flag ? "true" : "false");
	return flag;
}

static int usb_get_ver_info(void)
{
	int ver = -1;
	unsigned int svr = mfspr(SPRN_SVR);

	switch (SVR_SOC_VER(svr)) {
		case SVR_P2041:
		case SVR_P3041:
		case SVR_P4080:
		case SVR_P5020:
		case SVR_P5040:
			ver = FSL_USB_VER_1_6;
			break;
		case SVR_T1020:
		case SVR_T1021:
		case SVR_T1022:
		case SVR_T1023:
		case SVR_T1024:
		case SVR_T1040:
		case SVR_T1042:
		case SVR_T2080:
		case SVR_T2081:
		case SVR_T4240:
			ver = FSL_USB_VER_2_5;
			break;
		default:
			ver = FSL_USB_VER_OLD;
			break;
	}

	return ver;
}

static int __init fsl_usb_of_init(void)
{
	struct device_node *np;
	unsigned int i = 0;
	struct platform_device *usb_dev_mph = NULL, *usb_dev_dr_host = NULL,
		*usb_dev_dr_client = NULL;
	int ret;

	for_each_compatible_node(np, NULL, "fsl-usb2-mph") {
		struct resource r[2];
		struct fsl_usb2_platform_data usb_data;
		const unsigned char *prop = NULL;

		memset(&r, 0, sizeof(r));
		memset(&usb_data, 0, sizeof(usb_data));

		ret = of_address_to_resource(np, 0, &r[0]);
		if (ret)
			goto err;

		of_irq_to_resource(np, 0, &r[1]);

		usb_dev_mph =
		    platform_device_register_simple("fsl-ehci", i, r, 2);
		if (IS_ERR(usb_dev_mph)) {
			ret = PTR_ERR(usb_dev_mph);
			goto err;
		}

		usb_dev_mph->dev.coherent_dma_mask = 0xffffffffUL;
		usb_dev_mph->dev.dma_mask = &usb_dev_mph->dev.coherent_dma_mask;

		usb_dev_mph->dev.archdata.of_node = np;
		of_node_get(np);

		usb_data.have_sysif_regs = 1;
		usb_data.operating_mode = FSL_USB2_MPH_HOST;
		usb_data.controller_ver = usb_get_ver_info();

		/* Activate workaround for USB erratum-A005275 if
		 * fsl,no-erratum-a005275 property not defined for
		 * affected socs
		 */
		if (!of_get_property(np, "fsl,no-erratum-a005275", NULL) &&
				has_erratum_a005275(np))
			usb_data.has_fsl_erratum_a005275 = 1;
		else
			usb_data.has_fsl_erratum_a005275 = 0;

		if (has_erratum_a005697())
			usb_data.has_fsl_erratum_a005697 = 1;
		else
			usb_data.has_fsl_erratum_a005697 = 0;

		if (has_erratum_a007792(usb_data.controller_ver))
			usb_data.has_fsl_erratum_a007792 = 1;
		else
			usb_data.has_fsl_erratum_a007792 = 0;

		if (usb_data.controller_ver < 0) {
			dev_warn(&usb_dev_mph->dev, "Could not get controller version\n");
			ret = -ENODEV;
			goto err;
		}

		prop = of_get_property(np, "port0", NULL);
		if (prop)
			usb_data.port_enables |= FSL_USB2_PORT0_ENABLED;

		prop = of_get_property(np, "port1", NULL);
		if (prop)
			usb_data.port_enables |= FSL_USB2_PORT1_ENABLED;

		prop = of_get_property(np, "phy_type", NULL);
		usb_data.phy_mode = determine_usb_phy(prop);

#if defined(CONFIG_SILKWORM) && (defined(CONFIG_USB_STORAGE) || defined(CONFIG_USB_STORAGE_MODULE))
		{
		int *index;
		index = (int *)of_get_property(np, "sd_index", NULL);
		if (index)
			usb_data.sd_index = *index;
		else
			usb_data.sd_index = -1;
		}
#endif

		ret =
		    platform_device_add_data(usb_dev_mph, &usb_data,
					     sizeof(struct
						    fsl_usb2_platform_data));
		if (ret)
			goto unreg_mph;
		i++;
	}

	for_each_compatible_node(np, NULL, "fsl-usb2-dr") {
		struct resource r[2];
		struct fsl_usb2_platform_data usb_data;
		const unsigned char *prop = NULL;
#ifndef CONFIG_SILKWORM_EXT_USB_FAILOVER
		/*
		 * Ideally the device tree parsing for usb/ehci should never be
		 * here and also not be __arch_init since registration of the
		 * platform device can happen when the device is actually
		 * probed (which in case of ehci lkm be much later). 
		 * Perhaps this can be done in next release
		 *
		 * Temporarily however allow the standby gos to also register
		 * the driver and set up platform device so that we dont have to
		 * do this in failover path and so that dynamic loading
		 * works
		 */

		if (!of_device_is_available(np))
			continue;
#endif
		memset(&r, 0, sizeof(r));
		memset(&usb_data, 0, sizeof(usb_data));

		ret = of_address_to_resource(np, 0, &r[0]);
		if (ret)
			goto unreg_mph;

		of_irq_to_resource(np, 0, &r[1]);

		prop = of_get_property(np, "dr_mode", NULL);

		usb_data.have_sysif_regs = 1;
		usb_data.controller_ver = usb_get_ver_info();

		/* Activate workaround for USB erratum-A005275 if
		 * fsl,no-erratum-a005275 property not defined for
		 * affected socs
		 */
		if (!of_get_property(np, "fsl,no-erratum-a005275", NULL) &&
				has_erratum_a005275(np))
			usb_data.has_fsl_erratum_a005275 = 1;
		else
			usb_data.has_fsl_erratum_a005275 = 0;

		if (has_erratum_a005697())
			usb_data.has_fsl_erratum_a005697 = 1;
		else
			usb_data.has_fsl_erratum_a005697 = 0;

		if (has_erratum_a007792(usb_data.controller_ver))
			usb_data.has_fsl_erratum_a007792 = 1;
		else
			usb_data.has_fsl_erratum_a007792 = 0;

		if (usb_data.controller_ver < 0) {
			dev_warn(&usb_dev_mph->dev, "Could not get controller version\n");
			ret = -ENODEV;
			goto err;
		}

		if (!prop || !strcmp(prop, "host")) {
			usb_data.operating_mode = FSL_USB2_DR_HOST;
			usb_dev_dr_host = platform_device_register_simple(
					"fsl-ehci", i, r, 2);
			if (IS_ERR(usb_dev_dr_host)) {
				ret = PTR_ERR(usb_dev_dr_host);
				goto err;
			}

#if defined(CONFIG_SILKWORM) && (defined(CONFIG_USB_STORAGE) || defined(CONFIG_USB_STORAGE_MODULE))
			{
			int *index;
			index = (int *)of_get_property(np, "sd_index", NULL);
			if (index)
				usb_data.sd_index = *index;
			else
				usb_data.sd_index = -1;
			}
#endif

		} else if (prop && !strcmp(prop, "peripheral")) {
			usb_data.operating_mode = FSL_USB2_DR_DEVICE;
			usb_dev_dr_client = platform_device_register_simple(
					"fsl-usb2-udc", i, r, 2);
			if (IS_ERR(usb_dev_dr_client)) {
				ret = PTR_ERR(usb_dev_dr_client);
				goto err;
			}
		} else if (prop && !strcmp(prop, "otg")) {
			usb_data.operating_mode = FSL_USB2_DR_OTG;
			usb_dev_dr_host = platform_device_register_simple(
					"fsl-ehci", i, r, 2);
			if (IS_ERR(usb_dev_dr_host)) {
				ret = PTR_ERR(usb_dev_dr_host);
				goto err;
			}
			usb_dev_dr_client = platform_device_register_simple(
					"fsl-usb2-udc", i, r, 2);
			if (IS_ERR(usb_dev_dr_client)) {
				ret = PTR_ERR(usb_dev_dr_client);
				goto err;
			}
		} else {
			ret = -EINVAL;
			goto err;
		}

		prop = of_get_property(np, "phy_type", NULL);
		usb_data.phy_mode = determine_usb_phy(prop);

		if (usb_dev_dr_host) {
			usb_dev_dr_host->dev.coherent_dma_mask = 0xffffffffUL;
			usb_dev_dr_host->dev.dma_mask = &usb_dev_dr_host->
				dev.coherent_dma_mask;
			if ((ret = platform_device_add_data(usb_dev_dr_host,
						&usb_data, sizeof(struct
						fsl_usb2_platform_data))))
				goto unreg_dr;

			usb_dev_dr_host->dev.archdata.of_node = np;
			of_node_get(np);
		}
		if (usb_dev_dr_client) {
			usb_dev_dr_client->dev.coherent_dma_mask = 0xffffffffUL;
			usb_dev_dr_client->dev.dma_mask = &usb_dev_dr_client->
				dev.coherent_dma_mask;
			if ((ret = platform_device_add_data(usb_dev_dr_client,
						&usb_data, sizeof(struct
						fsl_usb2_platform_data))))
				goto unreg_dr;

			usb_dev_dr_client->dev.archdata.of_node = np;
			of_node_get(np);
		}
		i++;
	}
	return 0;

unreg_dr:
	if (usb_dev_dr_host)
		platform_device_unregister(usb_dev_dr_host);
	if (usb_dev_dr_client)
		platform_device_unregister(usb_dev_dr_client);
unreg_mph:
	if (usb_dev_mph)
		platform_device_unregister(usb_dev_mph);
err:
	return ret;
}

device_initcall(fsl_usb_of_init);

#if defined(CONFIG_FSL_SOC_BOOKE) || defined(CONFIG_PPC_86xx)
static __be32 __iomem *rstcr;

static int __init setup_rstcr(void)
{
	struct device_node *np;

	for_each_node_by_name(np, "global-utilities") {
		if ((of_get_property(np, "fsl,has-rstcr", NULL))) {
			rstcr = of_iomap(np, 0) + 0xb0;
			if (!rstcr)
				printk (KERN_ERR "Error: reset control "
						"register not mapped!\n");
			break;
		}
	}

	if (!rstcr && ppc_md.restart == fsl_rstcr_restart)
		printk(KERN_ERR "No RSTCR register, warm reboot won't work\n");

	if (np)
		of_node_put(np);

	return 0;
}

arch_initcall(setup_rstcr);

void fsl_rstcr_restart(char *cmd)
{
	local_irq_disable();
	if (rstcr)
		/* set reset control register */
		out_be32(rstcr, 0x2);	/* HRESET_REQ */

	while (1) ;
}
#endif

#if defined(CONFIG_FB_FSL_DIU) || defined(CONFIG_FB_FSL_DIU_MODULE)
struct platform_diu_data_ops diu_ops;
EXPORT_SYMBOL(diu_ops);
#endif
