/*
 *
 *    Copyright (c) 1996-2008 Brocade Communications Systems, Inc.
 *    All rights reserved.
 *
 *    Description:
 *      Silkworm Warm Memory (shared & Non-shared) chunk Allocator Module.
 *
 */

#ifdef CONFIG_SILKWORM_WMAP

#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/string.h>
#include <linux/proc_fs.h>
#include <include/asm/uaccess.h>
#include <linux/lmb.h>
#include <linux/wmap.h>
#include <linux/stacktrace.h>
#include <asm/silkworm_40x0_misc.h>

void WMAP_init(void);
unsigned int WMAP_get_unshared_chunk_range(unsigned int chunkID, void **start_addr, int *chunk_size);
unsigned int WMAP_get_unshared_seg_size(void );

static int WMAP_initialized =0;
/* Warm memory starts at the end of low memory & grows up */
static int wmem_start, unshared_start_paddr, unshared_start_vaddr;
static int wmap_rw=0, wmap_rw_vaddr, wmap_rw_size;
static int WMAP_validate_magic(int magic_start);
static int WMAP_set_magic(int magic_start);
static int WMAP_config_read(char *page, char **start, off_t off, int count, int *eof, void *data);
static int WMAP_chunk_read(char *page, char **start, off_t off, int count, int *eof, void *data);
static int WMAP_chunk_write(struct file * file, const char *buffer, unsigned long count, void *data);
static int WMAP_lmb_res_read(char *page, char **start, off_t off, int count, int *eof, void *data);
static int WMAP_check_magic_codes(char *page, char **start, off_t off, int count, int *eof, void *data);
static int WMAP_set_magic_codes(struct file * file, const char *buffer, unsigned long count, void *data);

#define PROC_WMAP_DIR_NAME "wmap"
#define PROC_WMAP_CONFIG_DATA "wmap_chunks"
#define PROC_WMAP_UNSHARED_TEST_MOD_FILE "unshared_test_chunk"
#define PROC_WMAP_LMB_FILE "lmb_res_callers"
#define PROC_WMAP_CHUNK_RW_FILE "wmap_rw"
#define PROC_WMAP_MAGIC_FILE "wmap_magic"
#define CHUNK_MAGIC_SIZE 128
#define CHUNK_MAGIC_NUM 0x73719BA9
#define CHUNK_NAME_ARR_SIZE 200

/* To add new modules for warm memory, 
 * Add a new element (first 3 fields & rest to 0) to the array below. Also, add chunk ID in the enum array in wmap.h.
 */
typedef struct WMAP_chunk_info {
    char chunk_name[CHUNK_NAME_ARR_SIZE];
    int chunk_id;
    int usr_chunk_size;

    int allocated;
    int magic_verify1;
    int magic_verify2;
    int top_magic_start;
    int top_magic_end;
    int usr_chunk_start;
    int usr_chunk_end;
    int bot_magic_start;
    int bot_magic_end;
    int tot_chunk_size;
} WMAP_chunk_info_t;

WMAP_chunk_info_t unshared_chunks[MAX_UNSHARED_CHUNK_ID] = {
    {"Panic Dump", WMAP_PANIC_DUMP, 10*BYTES_PER_MB, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {"Trace Buffer", WMAP_TRACE_BUFFER, 512*BYTES_PER_MB, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
#ifdef CONFIG_SILKWORM_WMAP_DEBUG
    {"Warm and unshared Test Module", WMAP_UNSHARED_TEST_MODULE, 1*BYTES_PER_MB, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
#endif
};

#ifdef CONFIG_SILKWORM_WMAP_DEBUG
static int WMAP_unshared_TM_config_read(char *page, char **start, off_t off, int count, int *eof, void *data);
static int WMAP_unshared_TM_config_write(struct file * file, const char *buffer, unsigned long count, void *data);
#endif


#if defined(CONFIG_SILKWORM_WMAP_LC) || defined(CONFIG_SILKWORM_WMAP_PB)

#define PROC_WMAP_SHARED_TEST_MOD_FILE "shared_test_chunk"

unsigned int WMAP_get_shared_chunk_range(unsigned int chunkID, void **start_addr, int *chunk_size);
unsigned int WMAP_get_shared_seg_size(void );

static int shared_start_paddr, shared_start_vaddr;

WMAP_chunk_info_t shared_chunks[MAX_SHARED_CHUNK_ID] = {
    {"Warm Memory", WMAP_WMEM, (CONFIG_SILKWORM_WMAP_SHARED_SIZE-(32*BYTES_PER_MB)), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
#ifdef CONFIG_GTG_ENET
    {"GTG Memory", WMAP_GTG_ENET, 16*BYTES_PER_MB, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
#endif
#ifdef CONFIG_SILKWORM_WMAP_DEBUG
    {"Warm and Shared Test Module", WMAP_SHARED_TEST_MODULE, 1*BYTES_PER_MB, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
#endif
/*Place allways platform  as last entry*/
	{"Platform warm Memory", WMAP_PLATFORM, 3*BYTES_PER_MB, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
};

#ifdef CONFIG_SILKWORM_WMAP_DEBUG
static int WMAP_shared_TM_config_read(char *page, char **start, off_t off, int count, int *eof, void *data);
static int WMAP_shared_TM_config_write(struct file * file, const char *buffer, unsigned long count, void *data);
#endif

#endif
static int WMAP_num_shared_chunk_ids(void){
	int chunks = MAX_SHARED_CHUNK_ID;

	if( silkworm_is_plat_wmem_config()){
               return chunks;
        }else{
		return (chunks -1);
	}


}
void WMAP_init()
{
    struct proc_dir_entry *WMAP_dir, *WMAP_config;
    int i;
    int num_shared_chunks;
    struct proc_dir_entry *WMAP_chunk_rw_config, *lmb_res_info_file, *magic_ver_file;
#ifdef CONFIG_SILKWORM_WMAP_DEBUG
    struct proc_dir_entry *WMAP_unshared_TM_config;
#endif

    wmem_start = lowmem_end_addr - CONFIG_SILKWORM_WMAP_SHIFT_OFFSET;

#if defined(CONFIG_SILKWORM_WMAP_LC) || defined(CONFIG_SILKWORM_WMAP_PB)
#ifdef CONFIG_SILKWORM_WMAP_DEBUG
    struct proc_dir_entry *WMAP_shared_TM_config;
#endif

    /* Sanity check */
    if (WMAP_get_shared_seg_size() > CONFIG_SILKWORM_WMAP_SHARED_SIZE) 
	panic("Shared memory segment size is not sufficient \n");

    shared_start_paddr = wmem_start - CONFIG_SILKWORM_WMAP_SHARED_SIZE;
    shared_start_vaddr = __va(shared_start_paddr);

    /* Fill up start address of each chunk in Warm & shared memory */
    shared_chunks[0].top_magic_start = shared_start_vaddr;
    shared_chunks[0].top_magic_end = shared_chunks[0].usr_chunk_start = shared_chunks[0].top_magic_start + CHUNK_MAGIC_SIZE;
    shared_chunks[0].usr_chunk_end = shared_chunks[0].bot_magic_start = shared_chunks[0].usr_chunk_start + shared_chunks[0].usr_chunk_size;
    shared_chunks[0].bot_magic_end = shared_chunks[0].usr_chunk_end + CHUNK_MAGIC_SIZE;
    shared_chunks[0].tot_chunk_size  = shared_chunks[0].bot_magic_end - shared_chunks[0].top_magic_start;
    shared_chunks[0].magic_verify1 = WMAP_validate_magic(shared_chunks[0].top_magic_start);
    shared_chunks[0].magic_verify2 = WMAP_validate_magic(shared_chunks[0].bot_magic_start);
	num_shared_chunks = WMAP_num_shared_chunk_ids();
    for (i=1; i<num_shared_chunks; i++)
    {
        shared_chunks[i].top_magic_start = shared_chunks[i-1].bot_magic_end;
        shared_chunks[i].top_magic_end = shared_chunks[i].usr_chunk_start = shared_chunks[i].top_magic_start + CHUNK_MAGIC_SIZE;
        shared_chunks[i].usr_chunk_end = shared_chunks[i].bot_magic_start = shared_chunks[i].usr_chunk_start + shared_chunks[i].usr_chunk_size;
        shared_chunks[i].bot_magic_end = shared_chunks[i].usr_chunk_end + CHUNK_MAGIC_SIZE;
        shared_chunks[i].tot_chunk_size  = shared_chunks[i].bot_magic_end - shared_chunks[i].top_magic_start;
        shared_chunks[i].magic_verify1 = WMAP_validate_magic(shared_chunks[i].top_magic_start);
        shared_chunks[i].magic_verify2 = WMAP_validate_magic(shared_chunks[i].bot_magic_start);
    }
#endif

#if defined(CONFIG_SILKWORM_WMAP_LC) || defined(CONFIG_SILKWORM_WMAP_PB)
    unshared_start_paddr = wmem_start - CONFIG_SILKWORM_WMAP_SHARED_SIZE - WMAP_get_unshared_seg_size();
#else
    unshared_start_paddr = wmem_start - WMAP_get_unshared_seg_size();
#endif
    unshared_start_vaddr = __va(unshared_start_paddr);

    /* Fill up start address of each chunk in Warm & Unshared memory */
    unshared_chunks[0].top_magic_start = unshared_start_vaddr;
    unshared_chunks[0].top_magic_end = unshared_chunks[0].usr_chunk_start = unshared_chunks[0].top_magic_start + CHUNK_MAGIC_SIZE;
    unshared_chunks[0].usr_chunk_end = unshared_chunks[0].bot_magic_start = unshared_chunks[0].usr_chunk_start + unshared_chunks[0].usr_chunk_size;
    unshared_chunks[0].bot_magic_end = unshared_chunks[0].usr_chunk_end + CHUNK_MAGIC_SIZE;
    unshared_chunks[0].tot_chunk_size  = unshared_chunks[0].bot_magic_end - unshared_chunks[0].top_magic_start;
    unshared_chunks[0].magic_verify1 = WMAP_validate_magic(unshared_chunks[0].top_magic_start);
    unshared_chunks[0].magic_verify2 = WMAP_validate_magic(unshared_chunks[0].bot_magic_start);

    for (i=1; i<MAX_UNSHARED_CHUNK_ID; i++)
    {
        unshared_chunks[i].top_magic_start = unshared_chunks[i-1].bot_magic_end;
        unshared_chunks[i].top_magic_end = unshared_chunks[i].usr_chunk_start = unshared_chunks[i].top_magic_start + CHUNK_MAGIC_SIZE;
        unshared_chunks[i].usr_chunk_end = unshared_chunks[i].bot_magic_start = unshared_chunks[i].usr_chunk_start + unshared_chunks[i].usr_chunk_size;
        unshared_chunks[i].bot_magic_end = unshared_chunks[i].usr_chunk_end + CHUNK_MAGIC_SIZE;
        unshared_chunks[i].tot_chunk_size  = unshared_chunks[i].bot_magic_end - unshared_chunks[i].top_magic_start;
        unshared_chunks[i].magic_verify1 = WMAP_validate_magic(unshared_chunks[i].top_magic_start);
        unshared_chunks[i].magic_verify2 = WMAP_validate_magic(unshared_chunks[i].bot_magic_start);
    }

    /* Create profs file for this module */
    WMAP_dir = proc_mkdir(PROC_WMAP_DIR_NAME, NULL);
    if (!WMAP_dir)
        return;

    /* Create WMAP shared configuration file */
    WMAP_config = create_proc_entry(PROC_WMAP_CONFIG_DATA, 0644, WMAP_dir); /* TBD: make 0644 a macro*/
    if (!WMAP_config)
        goto bailout1;
    WMAP_config->read_proc = WMAP_config_read;
    WMAP_config->write_proc = NULL;

#ifdef CONFIG_SILKWORM_WMAP_DEBUG
#if defined(CONFIG_SILKWORM_WMAP_LC) || defined(CONFIG_SILKWORM_WMAP_PB)
    /* Create WMAP shared test module configuration file */
    WMAP_shared_TM_config = create_proc_entry(PROC_WMAP_SHARED_TEST_MOD_FILE, 0644, WMAP_dir); /* TBD: make 0644 a macro*/
    if (!WMAP_shared_TM_config)
        goto bailout2;
    WMAP_shared_TM_config->read_proc = WMAP_shared_TM_config_read;
    WMAP_shared_TM_config->write_proc = WMAP_shared_TM_config_write;

    /* Write test data to shared test module's memory chunk */
    shared_chunks[WMAP_SHARED_TEST_MODULE].allocated =1;
    if ((! shared_chunks[WMAP_SHARED_TEST_MODULE].magic_verify1) || (!shared_chunks[WMAP_SHARED_TEST_MODULE].magic_verify2))
    {
        /* One of the Magic strings already doesn't exist, So set it afresh */
        WMAP_set_magic(shared_chunks[WMAP_SHARED_TEST_MODULE].top_magic_start);
        WMAP_set_magic(shared_chunks[WMAP_SHARED_TEST_MODULE].bot_magic_start);
        sprintf(shared_chunks[WMAP_SHARED_TEST_MODULE].usr_chunk_start, "Shared Test Module");
    }
#endif

    /* Create WMAP unshared test module configuration file */
    WMAP_unshared_TM_config = create_proc_entry(PROC_WMAP_UNSHARED_TEST_MOD_FILE, 0644, WMAP_dir); /* TBD: make 0644 a macro*/
    if (!WMAP_unshared_TM_config)
        goto bailout3;
    WMAP_unshared_TM_config->read_proc = WMAP_unshared_TM_config_read;
    WMAP_unshared_TM_config->write_proc = WMAP_unshared_TM_config_write;

    /* Write test data to unshared test module's memory chunk */
    unshared_chunks[WMAP_UNSHARED_TEST_MODULE].allocated =1;
    if ((! unshared_chunks[WMAP_UNSHARED_TEST_MODULE].magic_verify1) || (!unshared_chunks[WMAP_UNSHARED_TEST_MODULE].magic_verify2))
    {
        /* One of the Magic strings already doesn't exist, So set it afresh */
        WMAP_set_magic(unshared_chunks[WMAP_UNSHARED_TEST_MODULE].top_magic_start);
        WMAP_set_magic(unshared_chunks[WMAP_UNSHARED_TEST_MODULE].bot_magic_start);
        sprintf(unshared_chunks[WMAP_UNSHARED_TEST_MODULE].usr_chunk_start, "Unshared Test Module");
    }
#endif

    /* Create WMAP chunk test read/write file */
    WMAP_chunk_rw_config = create_proc_entry(PROC_WMAP_CHUNK_RW_FILE, 0644, WMAP_dir); /* TBD: make 0644 a macro*/
    if (!WMAP_chunk_rw_config)
        goto bailout4;
    WMAP_chunk_rw_config->read_proc = WMAP_chunk_read;
    WMAP_chunk_rw_config->write_proc = WMAP_chunk_write;

    /* Create lmb_reserve users info file */
    lmb_res_info_file = create_proc_entry(PROC_WMAP_LMB_FILE, 0644, WMAP_dir); /* TBD: make 0644 a macro*/
    if (!lmb_res_info_file)
        goto bailout5;
    lmb_res_info_file->read_proc = WMAP_lmb_res_read;
    lmb_res_info_file->write_proc = NULL;

    /* Create WMAP magic code read file */
    magic_ver_file = create_proc_entry(PROC_WMAP_MAGIC_FILE, 0644, WMAP_dir); /* TBD: make 0644 a macro*/
    if (!magic_ver_file)
        goto bailout6;
    magic_ver_file->read_proc = WMAP_check_magic_codes;
    magic_ver_file->write_proc = WMAP_set_magic_codes;

    WMAP_initialized=1;

    return;

bailout6:
    remove_proc_entry(PROC_WMAP_MAGIC_FILE, NULL);
bailout5:
    remove_proc_entry(PROC_WMAP_LMB_FILE, NULL);
bailout4:
#ifdef CONFIG_SILKWORM_WMAP_DEBUG
    remove_proc_entry(PROC_WMAP_UNSHARED_TEST_MOD_FILE, NULL);
bailout3:
#if defined(CONFIG_SILKWORM_WMAP_LC) || defined(CONFIG_SILKWORM_WMAP_PB)
    remove_proc_entry(PROC_WMAP_SHARED_TEST_MOD_FILE, NULL);
#endif
#endif
bailout2:
    remove_proc_entry(PROC_WMAP_CONFIG_DATA, NULL);
bailout1:
    remove_proc_entry(PROC_WMAP_DIR_NAME, NULL);
    return;
}

#if defined(CONFIG_SILKWORM_WMAP_LC) || defined(CONFIG_SILKWORM_WMAP_PB)
unsigned int WMAP_get_shared_seg_size(void)
{
    int i, size=0;
    int num_shared_chunks;

    num_shared_chunks = WMAP_num_shared_chunk_ids();
    for (i=0; i< num_shared_chunks; i++)
        size += shared_chunks[i].usr_chunk_size;


    /*Multiplying 2 because we store magic code at the begining as well as end of the chunk*/
    size += (CHUNK_MAGIC_SIZE* num_shared_chunks *2); 
    return size;
}

unsigned int WMAP_get_shared_chunk_range(unsigned int chunkID, void **start_addr, int *chunk_size)
{
    if (!WMAP_initialized) return WMAP_UNINITIALIZED;
    int num_shared_chunks;

    num_shared_chunks = WMAP_num_shared_chunk_ids();
    if (chunkID < num_shared_chunks)
    {
        *start_addr = shared_chunks[chunkID].usr_chunk_start;
        *chunk_size = shared_chunks[chunkID].usr_chunk_size;

        shared_chunks[chunkID].allocated =1;
        if ((! shared_chunks[chunkID].magic_verify1) && (!shared_chunks[chunkID].magic_verify2))
        {
            /* Magic string already doesn't exist, So set it afresh */
            WMAP_set_magic(shared_chunks[chunkID].top_magic_start);
            WMAP_set_magic(shared_chunks[chunkID].bot_magic_start);
        }
    } else {
        return WMAP_BAD_CHUNK_ID;
    }

    return WMAP_SUCCESS;
}

#else

unsigned int WMAP_get_shared_chunk_range(unsigned int chunkID, void **start_addr, int *chunk_size)
{
    //*start_addr = NULL;
    *chunk_size = 0;

    return WMAP_SUCCESS;
}

#endif

EXPORT_SYMBOL(WMAP_get_shared_chunk_range);

unsigned int WMAP_get_addr_range(void **start_addr, ulong *size)
{
    if (!WMAP_initialized) return WMAP_UNINITIALIZED;

	*start_addr = unshared_start_vaddr;
	*size = wmem_start - unshared_start_paddr;

    return WMAP_SUCCESS;
}
EXPORT_SYMBOL(WMAP_get_addr_range);

unsigned int WMAP_get_unshared_seg_size(void)
{
    int i, size=0;

    for (i=0; i< MAX_UNSHARED_CHUNK_ID; i++)
        size += unshared_chunks[i].usr_chunk_size;

    /*Multiplying 2 because we store magic code at the begining as well as end of the chunk*/
    size += (CHUNK_MAGIC_SIZE* MAX_UNSHARED_CHUNK_ID *2); 

    return size;
}

unsigned int WMAP_get_unshared_chunk_range(unsigned int chunkID, void **start_addr, int *chunk_size)
{
    if (!WMAP_initialized) return WMAP_UNINITIALIZED;

    if (chunkID < MAX_UNSHARED_CHUNK_ID)
    {
        *start_addr = unshared_chunks[chunkID].usr_chunk_start;
        *chunk_size = unshared_chunks[chunkID].usr_chunk_size;

        unshared_chunks[chunkID].allocated =1;
	/* Lets reset always so we can catch corruptions that recur after
 	 * warm reset */
        WMAP_set_magic(unshared_chunks[chunkID].top_magic_start);
        WMAP_set_magic(unshared_chunks[chunkID].bot_magic_start);
    } else {
        return WMAP_BAD_CHUNK_ID;
    }

    return WMAP_SUCCESS;
}
EXPORT_SYMBOL(WMAP_get_unshared_chunk_range);

static int WMAP_check_magic_codes(char *page, char **start, off_t off, int count, int *eof, void *data)
{
    char buff1[300] ="\0";
    int size=0, i;
    int num_shared_chunks;
    if (!WMAP_initialized)
    {
        sprintf(buff1, "WMAP is not initalized");
        sprintf(page,"\n\n%s", buff1);
        size += strlen(buff1)+2;
    } else {
        sprintf(buff1, "Warm & Unshared Chunks:");
        sprintf(&page[size],"%s\n", buff1);
        size += strlen(buff1)+1;
        for (i=0; i<MAX_UNSHARED_CHUNK_ID; i++)
        {
            if (unshared_chunks[i].allocated) 
            {
                sprintf(buff1, "    Chunk ID : %d", i);
                sprintf(&page[size],"%s\n", buff1);
                size += strlen(buff1)+1;
                sprintf(buff1, "    Chunk Name: %s", unshared_chunks[i].chunk_name);
                sprintf(&page[size],"%s\n", buff1);
                size += strlen(buff1)+1;

                if (WMAP_validate_magic(unshared_chunks[i].top_magic_start))
                {
                    sprintf(buff1, "    Magic Code Check: Validated at the top of chunk ");
                    sprintf(&page[size],"%s\n", buff1);
                    size += strlen(buff1)+1;
                } else {
                    sprintf(buff1, "    Magic Code Check: Failed at the top of chunk ");
                    sprintf(&page[size],"%s\n", buff1);
                    size += strlen(buff1)+1;
                }

                if (WMAP_validate_magic(unshared_chunks[i].bot_magic_start))
                {
                    sprintf(buff1, "    Magic Code Check: Validated at the bottom of chunk ");
                    sprintf(&page[size],"%s\n", buff1);
                    size += strlen(buff1)+1;
                } else {
                    sprintf(buff1, "    Magic Code Check: Failed at the bottom of chunk ");
                    sprintf(&page[size],"%s\n", buff1);
                    size += strlen(buff1)+1;
                }
                sprintf(&page[size],"\n");
                size++;
	    }
        }
#if defined(CONFIG_SILKWORM_WMAP_LC) || defined(CONFIG_SILKWORM_WMAP_PB)
        sprintf(buff1, "Warm & Shared Chunks:");
        sprintf(&page[size],"%s\n", buff1);
        size += strlen(buff1)+1;


	num_shared_chunks = WMAP_num_shared_chunk_ids();
        for (i=0; i<num_shared_chunks; i++)
        {
            if (shared_chunks[i].allocated) 
            {
                sprintf(buff1, "    Chunk ID : %d", i);
                sprintf(&page[size],"%s\n", buff1);
                size += strlen(buff1)+1;
                sprintf(buff1, "    Chunk Name: %s", shared_chunks[i].chunk_name);
                sprintf(&page[size],"%s\n", buff1);
                size += strlen(buff1)+1;

                if (WMAP_validate_magic(shared_chunks[i].top_magic_start))
                {
                    sprintf(buff1, "    Magic Code Check: Validated at the top of chunk ");
                    sprintf(&page[size],"%s\n", buff1);
                    size += strlen(buff1)+1;
                } else {
                    sprintf(buff1, "    Magic Code Check: Failed at the top of chunk ");
                    sprintf(&page[size],"%s\n", buff1);
                    size += strlen(buff1)+1;
                }

                if (WMAP_validate_magic(shared_chunks[i].bot_magic_start))
                {
                    sprintf(buff1, "    Magic Code Check: Validated at the bottom of chunk ");
                    sprintf(&page[size],"%s\n", buff1);
                    size += strlen(buff1)+1;
                } else {
                    sprintf(buff1, "    Magic Code Check: Failed at the bottom of chunk ");
                    sprintf(&page[size],"%s\n", buff1);
                    size += strlen(buff1)+1;
                }
                sprintf(&page[size],"\n");
                size++;
	    }
        }
#endif
    }

    return size;
}

static int WMAP_set_magic_codes(struct file * file, const char *buffer, unsigned long count, void *data)
{
    char *buff, *buff1, **end_ptr;
    unsigned int i, shared, chunk_id;
    char cbuff[100];
    int num_shared_chunks;

	num_shared_chunks = WMAP_num_shared_chunk_ids();
    if (!WMAP_initialized)
        printk("\n\n WMAP is not initalized\n\n");
    else {
        memset(cbuff, 0, 100);
        copy_from_user(cbuff, buffer, count);
        cbuff[count] = '\0';
    
        buff = cbuff;
        for (i=1; i<=2 /* Max config entires */; i++)
        {
            buff1 = buff;
            while ((*buff != ' ') && (*buff != '\0') && (*buff != '\n')) buff++;
            if (*buff != '\n')
            {
                if (*buff == ' ')
                    buff++;
                else
                    return 0;
            }
    
            switch(i) {
                case 1:
                    shared = simple_strtoul(buff1, end_ptr, 16);
                    break;
                case 2:
                    chunk_id= simple_strtoul(buff1, end_ptr, 16);
                    break;
            }
        }

	if (shared) 
        {
#if defined(CONFIG_SILKWORM_WMAP_LC) || defined(CONFIG_SILKWORM_WMAP_PB)
            if (chunk_id < num_shared_chunks)
            {
                WMAP_set_magic(shared_chunks[chunk_id].top_magic_start);
                WMAP_set_magic(shared_chunks[chunk_id].bot_magic_start);
                printk("Magic code reset for chunk: %s \n", shared_chunks[chunk_id].chunk_name);
            } else if (chunk_id == num_shared_chunks) {
                for (i=0; i<num_shared_chunks; i++)
                {
                    WMAP_set_magic(shared_chunks[i].top_magic_start);
                    WMAP_set_magic(shared_chunks[i].bot_magic_start);
                    printk("Magic code reset for chunk: %s \n", shared_chunks[i].chunk_name);
                }
            }
#else
            printk("Shared chunk not supported in this configuration \r\n");
#endif
	} else {
            if (chunk_id < MAX_UNSHARED_CHUNK_ID)
            {
                WMAP_set_magic(unshared_chunks[chunk_id].top_magic_start);
                WMAP_set_magic(unshared_chunks[chunk_id].bot_magic_start);
                printk("Magic code reset for chunk: %s \n", unshared_chunks[chunk_id].chunk_name);
            } else if (chunk_id == MAX_UNSHARED_CHUNK_ID) {
                for (i=0; i<MAX_UNSHARED_CHUNK_ID; i++)
                {
                    WMAP_set_magic(unshared_chunks[i].top_magic_start);
                    WMAP_set_magic(unshared_chunks[i].bot_magic_start);
                    printk("Magic code reset for chunk: %s \n", unshared_chunks[i].chunk_name);
                }
            }
        }
    }

    return count;
}


static int WMAP_lmb_res_read(char *page, char **start, off_t off, int count, int *eof, void *data)
{
    char buff1[300] ="\0";
    int size=0;
    unsigned long long base, size1;
    int i;
    struct lmb_region *region;

    if (!WMAP_initialized)
    {
        sprintf(buff1, "WMAP is not initalized");
        sprintf(page,"\n\n%s", buff1);
        size += strlen(buff1)+2;
    } 

    sprintf(buff1, "LMB configuration:");
    sprintf(&page[size],"\n%s\n", buff1);
    size += strlen(buff1)+2;
    sprintf(buff1, " rmo_size    = 0x%llx", (unsigned long long)lmb.rmo_size);
    sprintf(&page[size],"%s\n", buff1);
    size += strlen(buff1)+1;
    sprintf(buff1, " memory.size = 0x%llx", (unsigned long long)lmb.memory.size);
    sprintf(&page[size],"%s\n", buff1);
    size += strlen(buff1)+1;

    region = &lmb.memory;
    sprintf(buff1, " lmb.memory.cnt  = 0x%lx", region->cnt);
    sprintf(&page[size],"%s\n", buff1);
    size += strlen(buff1)+1;

    for (i = 0; i < region->cnt; i++) {
            base = region->region[i].base;
            size1 = region->region[i].size;

            sprintf(buff1, " lm.memory[0x%x]\t0x%016llx - 0x%016llx, 0x%llx bytes",
                i, base, base + size1 - 1, size1);
            sprintf(&page[size],"%s\n", buff1);
            size += strlen(buff1)+1;
    }
    
    region = &lmb.reserved;
    sprintf(buff1, " lmb.reserved.cnt  = 0x%lx", region->cnt);
    sprintf(&page[size],"%s\n", buff1);
    size += strlen(buff1)+1;

    for (i = 0; i < region->cnt; i++) {
            base = region->region[i].base;
            size1 = region->region[i].size;

            sprintf(buff1, " lm.reserved[0x%x]\t0x%016llx - 0x%016llx, 0x%llx bytes",
                i, base, base + size1 - 1, size1);
            sprintf(&page[size],"%s\n", buff1);
            size += strlen(buff1)+1;
    }

    return size;
}

static int WMAP_config_read(char *page, char **start, off_t off, int count, int *eof, void *data)
{
    char buff1[300] ="\0";;
    int size=0, i;
    int num_shared_chunks;

    num_shared_chunks = WMAP_num_shared_chunk_ids();
    if (!WMAP_initialized)
    {
        sprintf(buff1, "WMAP is not initalized");
        sprintf(page,"\n\n%s", buff1);
        size += strlen(buff1)+2;
    } else {
        sprintf(buff1, "WMAP Region Start Offset from the end of low mem = "
			"%d bytes", CONFIG_SILKWORM_WMAP_SHIFT_OFFSET);
        sprintf(&page[size],"%s\n", buff1);
        size += strlen(buff1)+1;

        sprintf(buff1, "Warm and UnShared Memory Segment Info: ");
        sprintf(&page[size],"\n\n%s\n", buff1);
        size += strlen(buff1)+3;
        sprintf(buff1, "    GOS Physcial Address = 0x%p", unshared_start_paddr);
        sprintf(&page[size],"%s\n", buff1);
        size += strlen(buff1)+1;
        sprintf(buff1, "    GOS Virtual Address = 0x%p", unshared_start_vaddr);
        sprintf(&page[size],"%s\n", buff1);
        size += strlen(buff1)+1;
        sprintf(buff1, "    Segment Size = %d MB", WMAP_get_unshared_seg_size()/BYTES_PER_MB);
        sprintf(&page[size],"%s\n\n", buff1);
        size += strlen(buff1)+2;

        sprintf(buff1, "    Warm and Unshared Memory chunks : ");
        sprintf(&page[size],"%s\n", buff1);
        size += strlen(buff1)+1;
        for (i=0; i<MAX_UNSHARED_CHUNK_ID; i++)
        {
            sprintf(buff1, "        Chunk ID : %d", i);
            sprintf(&page[size],"%s\n", buff1);
            size += strlen(buff1)+1;
            sprintf(buff1, "        Chunk Name: %s", unshared_chunks[i].chunk_name);
            sprintf(&page[size],"%s\n", buff1);
            size += strlen(buff1)+1;

            if (unshared_chunks[i].allocated) 
            {
                sprintf(buff1, "        Allocation Status: Allocated ");
                sprintf(&page[size],"%s\n", buff1);
                size += strlen(buff1)+1;
                if (unshared_chunks[i].magic_verify1)
                {
                    sprintf(buff1, "        Magic Code Check: Validated at the top of chunk during bootup");
                    sprintf(&page[size],"%s\n", buff1);
                    size += strlen(buff1)+1;
                } else {
                    sprintf(buff1, "        Magic Code Check: Failed at the top of chunk during bootup");
                    sprintf(&page[size],"%s\n", buff1);
                    size += strlen(buff1)+1;
                }

                if (unshared_chunks[i].magic_verify2)
                {
                    sprintf(buff1, "        Magic Code Check: Validated at the bottom of chunk during bootup");
                    sprintf(&page[size],"%s\n", buff1);
                    size += strlen(buff1)+1;
                } else {
                    sprintf(buff1, "        Magic Code Check: Failed at the bottom of chunk during bootup");
                    sprintf(&page[size],"%s\n", buff1);
                    size += strlen(buff1)+1;
                }
                sprintf(buff1, "        GOS Physcial Address: 0x%x", __pa(unshared_chunks[i].usr_chunk_start));
                sprintf(&page[size],"%s\n", buff1);
                size += strlen(buff1)+1;
                sprintf(buff1, "        GOS Virtual Address: 0x%x", unshared_chunks[i].usr_chunk_start);
                sprintf(&page[size],"%s\n", buff1);
                size += strlen(buff1)+1;
                sprintf(buff1, "        Chunk Size :  %d MB", unshared_chunks[i].usr_chunk_size/BYTES_PER_MB);
                sprintf(&page[size],"%s\n\n", buff1);
                size += strlen(buff1)+2;
            } else {
                sprintf(buff1, "        Allocation Status: Not Allocated ");
                sprintf(&page[size],"%s\n\n", buff1);
                size += strlen(buff1)+2;
            }
        }

#if defined(CONFIG_SILKWORM_WMAP_LC) || defined(CONFIG_SILKWORM_WMAP_PB)
        sprintf(buff1, "Warm and Shared Memory Segment Info: ");
        sprintf(&page[size],"\n\n%s\n", buff1);
        size += strlen(buff1)+3;
        sprintf(buff1, "    GOS Physcial Address = 0x%p", shared_start_paddr);
        sprintf(&page[size],"%s\n", buff1);
        size += strlen(buff1)+1;
        sprintf(buff1, "    GOS Virtual Address = 0x%p", shared_start_vaddr);
        sprintf(&page[size],"%s\n", buff1);
        size += strlen(buff1)+1;
        sprintf(buff1, "    Segment Size = %d MB", CONFIG_SILKWORM_WMAP_SHARED_SIZE/BYTES_PER_MB);
        sprintf(&page[size],"%s\n\n", buff1);
        size += strlen(buff1)+2;

        sprintf(buff1, "    Warm and Shared Memory chunks : ");
        sprintf(&page[size],"%s\n", buff1);
        size += strlen(buff1)+1;
        for (i=0; i<num_shared_chunks; i++)
        {
            sprintf(buff1, "        Chunk ID : %d", i);
            sprintf(&page[size],"%s\n", buff1);
            size += strlen(buff1)+1;
            sprintf(buff1, "        Chunk Name: %s", shared_chunks[i].chunk_name);
            sprintf(&page[size],"%s\n", buff1);
            size += strlen(buff1)+1;

            if (shared_chunks[i].allocated) 
            {
                sprintf(buff1, "        Allocation Status: Allocated ");
                sprintf(&page[size],"%s\n", buff1);
                size += strlen(buff1)+1;
                if (shared_chunks[i].magic_verify1)
                {
                    sprintf(buff1, "        Magic Code Check: Validated at the top of chunk during bootup");
                    sprintf(&page[size],"%s\n", buff1);
                    size += strlen(buff1)+1;
                } else {
                    sprintf(buff1, "        Magic Code Check: Failed at the top of chunk during bootup");
                    sprintf(&page[size],"%s\n", buff1);
                    size += strlen(buff1)+1;
                }

                if (shared_chunks[i].magic_verify2)
                {
                    sprintf(buff1, "        Magic Code Check: Validated at the bottom of chunk during bootup");
                    sprintf(&page[size],"%s\n", buff1);
                    size += strlen(buff1)+1;
                } else {
                    sprintf(buff1, "        Magic Code Check: Failed at the bottom of chunk during bootup");
                    sprintf(&page[size],"%s\n", buff1);
                    size += strlen(buff1)+1;
                }
                sprintf(buff1, "        GOS Physcial Address: 0x%x", __pa(shared_chunks[i].usr_chunk_start));
                sprintf(&page[size],"%s\n", buff1);
                size += strlen(buff1)+1;
                sprintf(buff1, "        GOS Virtual Address: 0x%x", shared_chunks[i].usr_chunk_start);
                sprintf(&page[size],"%s\n", buff1);
                size += strlen(buff1)+1;
                sprintf(buff1, "        Chunk Size :  %d MB", shared_chunks[i].usr_chunk_size/BYTES_PER_MB);
                sprintf(&page[size],"%s\n\n", buff1);
                size += strlen(buff1)+2;
            } else {
                sprintf(buff1, "        Allocation Status: Not Allocated ");
                sprintf(&page[size],"%s\n\n", buff1);
                size += strlen(buff1)+2;
            }
	}
#endif
    }

    sprintf(&page[size], "\n\n");
    size += 2;

    *eof = 1;
    return (size);
}

#ifdef CONFIG_SILKWORM_WMAP_DEBUG

static int WMAP_unshared_TM_config_read(char *page, char **start, off_t off, int count, int *eof, void *data)
{
    char *buff = (char *)unshared_chunks[WMAP_UNSHARED_TEST_MODULE].usr_chunk_start;
    char buff1[100] ="Warm and Shared Test Module not mapped \0";;
    int ret=1;

    if (!WMAP_initialized)
    {
        sprintf(buff1, "WMAP is not initalized");
        sprintf(page,"\n\n%s\n\n", buff1);
        ret+= strlen(buff1)+4;
    } else {
        if (buff)
        {
            sprintf(page, "\n%s \n\n", buff);
            ret = strlen(buff)+4;
        } else {
            sprintf(page, "\n%s\n", buff1);
            ret = strlen(buff1)+2; 
        }
    }

    *eof = 1;
    return ret;
}

static int WMAP_unshared_TM_config_write(struct file * file, const char *buffer, unsigned long count, void *data)
{
    char *buff = (char *)unshared_chunks[WMAP_UNSHARED_TEST_MODULE].usr_chunk_start;

    if (!WMAP_initialized)
        printk("\n\n WMAP is not initalized\n\n");
    else {
        if (buff)
        {
            copy_from_user(buff, buffer, count);
            buff[count] = '\0';
        } 
    }

    return count;
}

#if defined(CONFIG_SILKWORM_WMAP_LC) || defined(CONFIG_SILKWORM_WMAP_PB)
static int WMAP_shared_TM_config_read(char *page, char **start, off_t off, int count, int *eof, void *data)
{
    char *buff = (char *)shared_chunks[WMAP_SHARED_TEST_MODULE].usr_chunk_start;
    char buff1[100] ="Warm and Shared Test Module not mapped \0";;
    int ret=1;

    if (!WMAP_initialized)
    {
        sprintf(buff1, "WMAP is not initalized");
        sprintf(page,"\n\n%s\n\n", buff1);
        ret+= strlen(buff1)+4;
    } else {
        if (buff)
        {
            sprintf(page, "\n%s \n\n", buff);
            ret = strlen(buff)+4;
        } else {
            sprintf(page, "\n%s\n", buff1);
            ret = strlen(buff1)+2; 
        }
    }

    *eof = 1;
    return ret;
}

static int WMAP_shared_TM_config_write(struct file * file, const char *buffer, unsigned long count, void *data)
{
    char *buff = (char *)shared_chunks[WMAP_SHARED_TEST_MODULE].usr_chunk_start;

    if (!WMAP_initialized)
        printk("\n\n WMAP is not initalized\n\n");
    else {
        if (buff)
        {
            copy_from_user(buff, buffer, count);
            buff[count] = '\0';
        } 
    }

    return count;
}
#endif

#endif

static int WMAP_chunk_read(char *page, char **start, off_t off, int count, int *eof, void *data)
{
    char *buff;
    char buff1[100]= "Invalid buffer pointer\0";
    int size=0;

    if (!WMAP_initialized)
    {
        sprintf(buff1, "WMAP is not initalized");
        sprintf(page,"\n\n%s\n\n", buff1);
        size += strlen(buff1)+4;
    } else {
        /*verify whether chunk id is valid */
        /* verify whether offset & size are within the bounds of that chunk */

        buff = (char *)(wmap_rw_vaddr);
        if (buff)
        {
            sprintf(page, "\n");
            size++;
            sprintf(buff1, "Data at vaddr %x ", buff);
            sprintf(&page[size], "%s", buff1);
            size += strlen(buff1);
            sprintf(&page[size], "\n");
            size++;
            memcpy(&page[size], buff, wmap_rw_size);
            size +=wmap_rw_size;
            sprintf(&page[size], "\n\n");
            size += 3;
        } else {
            sprintf(page, "\n%s\n", buff1);
            size = strlen(buff1)+2; 
        }
    }

    *eof = 1;
    return size;
}

static int WMAP_chunk_write(struct file * file, const char *buffer, unsigned long count, void *data)
{
    char *buff, *buff1, **end_ptr;
    unsigned int i;
    char WMAP_rw_buff[100];

    if (!WMAP_initialized)
        printk("\n\n WMAP is not initalized\n\n");
    else {
        memset(WMAP_rw_buff, 0, 100);
        copy_from_user(WMAP_rw_buff, buffer, count);
        WMAP_rw_buff[count] = '\0';
    
        buff = WMAP_rw_buff;
        for (i=1; i<=3 /* Max config entires */; i++)
        {
            buff1 = buff;
            while ((*buff != ' ') && (*buff != '\0') && (*buff != '\n')) buff++;
            if (*buff != '\n')
            {
                if (*buff == ' ')
                    buff++;
                else
                    return 0;
            }
    
            switch(i) {
                case 1:
                    wmap_rw = simple_strtoul(buff1, end_ptr, 16);
                    break;
                case 2:
                    wmap_rw_vaddr = simple_strtoul(buff1, end_ptr, 16);
                    break;
                case 3:
                    wmap_rw_size = simple_strtoul(buff1, end_ptr, 16);
                    break;
            }
        }
        
        if (wmap_rw)
        {
            buff = (char *)wmap_rw_vaddr;
            memset(buff, 'a', wmap_rw_size);
        }
    }

    return count;
}

static int WMAP_validate_magic(int magic_start)
{
    int *buff = (int *)magic_start;
    int i;

    for (i=0; i< (CHUNK_MAGIC_SIZE/sizeof(int)); i++)
    {
        if (*buff != CHUNK_MAGIC_NUM) 
            return 0;
        buff++;
    }

    return 1;
}

static int WMAP_set_magic(int magic_start)
{
    int *buff = (int *)magic_start;
    int i;

    for (i=0; i< (CHUNK_MAGIC_SIZE/sizeof(int)); i++)
    {
        *buff  = CHUNK_MAGIC_NUM;
        buff++;
    }

    return 1;
}

#endif
