/*
 * Fixed MDIO bus (MDIO bus emulation with fixed PHYs)
 *
 * Author: Vitaly Bordug <vbordug@ru.mvista.com>
 *         Anton Vorontsov <avorontsov@ru.mvista.com>
 *
 * Copyright (c) 2006-2007 MontaVista Software, Inc.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */
#ifdef CONFIG_SILKWORM

#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/netdevice.h>
#include <linux/phy.h>
#include <linux/phy_fixed.h>
#include "../gianfar.h"


struct phy_device *fixed_link_add(struct net_device *dev, const char *phy_id,
        u32 phyless, struct fixed_phy_status *status, int *no_phy)
{
	int err;
	struct phy_device *phydev;
	struct device *d;

	*no_phy = 0;

	/* Search the list of PHY devices on the mdio bus for the
	 * PHY with the requested name */
	d = bus_find_device_by_name(&mdio_bus_type, NULL, phy_id);
	if (d) {
		phydev = to_phy_device(d);
	} else {
		phydev = kzalloc(sizeof(struct phy_device),
			GFP_KERNEL);

		if(NULL == phydev) {
			printk(KERN_ERR "%s: Could not allocate phydev\n",
					dev->name);
			return NULL;
		}
		phydev->attached_dev = dev;
		dev_set_name(&phydev->dev, ":%s", phy_id);
		*no_phy = 1;
	}
	if (*no_phy == 1)
		printk(KERN_INFO "%s: No PHY\n", dev->name);
	else if (phyless) {
		printk(KERN_INFO "%s: Pseudo PHY\n", dev->name);
		// to avoid del_timer warning
		INIT_DELAYED_WORK(&phydev->state_queue, NULL);
	}

	phydev->link = status->link;
	phydev->speed = status->speed;
	phydev->duplex = status->duplex;
	phydev->pause = status->pause;
	phydev->asym_pause = status->asym_pause;

	if (!phyless) {
		phydev->autoneg = AUTONEG_DISABLE;
		err = phy_start_aneg(phydev);
		if (err < 0) {
			printk(KERN_ERR "%s: started PHY failed\n", dev->name);
			return NULL;
		}
	}

	return phydev;
}
EXPORT_SYMBOL(fixed_link_add);

MODULE_DESCRIPTION("Fixed-link ethernet port (for FSL)");
MODULE_AUTHOR("Ming Lei");
MODULE_LICENSE("GPL");

#endif /* CONFIG_SILKWORM */
