#include <linux/init.h>
#include <linux/kernel_stat.h>
#include <linux/proc_fs.h>
#include <linux/seq_file.h>

#ifdef CONFIG_SILKWORM

#include <linux/module.h>
#include <linux/delay.h>
#include <asm/time.h>
#include <asm/uaccess.h>
#include <linux/kallsyms.h>
#include <linux/vmalloc.h>
#include <linux/string.h>

extern unsigned long tb_ticks_per_jiffy;
unsigned tb_cycles_per_sec;
static cycles_t tm_insert, tm_sec, tm_sec_sum, tm_sum;
unsigned long insert_cnt, sec_cnt;
int  softirqs_enabled = 1;
static DEFINE_MUTEX(softirqs_file_mutex);

#define SIRQ_TIME_INTERVAL 10	/* 10 seconds to capture stats */
#define CYCLESFORPERCENT (tb_ticks_per_jiffy/100 * SIRQ_TIME_INTERVAL)  /* Just for calculating % */
cycles_t time_cycles = 0;


#define SIRQ_ARRAY_MAX_TOP  10  /* Maximum number of Softirq callers we want to monitor */
#define SIRQ_ARRAY_MAX_CUR  20  
#define MAX_STRING_LENGTH   20


typedef struct softirq_caller_struct {
	void  *softirqcaller;
	unsigned long long count;
	unsigned long max_cycles;
	unsigned long long cpu_cycles;
	unsigned int print;
} softirq_caller_struct_t;

int sort_callers(void *a, void *b);


softirq_caller_struct_t softirq_latency_cur[NR_CPUS][SIRQ_ARRAY_MAX_CUR];
softirq_caller_struct_t softirq_count_cur[NR_CPUS][SIRQ_ARRAY_MAX_CUR];
softirq_caller_struct_t softirq_latency_top[NR_CPUS][SIRQ_ARRAY_MAX_TOP];
softirq_caller_struct_t softirq_count_top[NR_CPUS][SIRQ_ARRAY_MAX_TOP];

static inline void shift_softirq(softirq_caller_struct_t *list, int start, int size)
{
	int i;

	if ((size-1) <= start)
		return;
	for (i=size-1; i>start; i--)
		memcpy(&list[i], &list[i-1], sizeof(softirq_caller_struct_t));
}

static inline int find_softirq_func(void *func, softirq_caller_struct_t *s, int size)
{
	int i;

	for (i=0; i<size; i++)
	{
		if (s[i].max_cycles == 0)
			return -1;
		if (s[i].softirqcaller == func)
			return i;
	}
	return -1;
}

static inline void swapsoftirq(softirq_caller_struct_t *x,softirq_caller_struct_t *y)
{
	softirq_caller_struct_t temp;

	memcpy(&temp, x, sizeof(softirq_caller_struct_t));
	memcpy(x, y, sizeof(softirq_caller_struct_t));
	memcpy(y, &temp, sizeof(softirq_caller_struct_t));
}
 
static inline cycles_t calc_cycles(cycles_t start, cycles_t end)
{
	if (end >= start)
		return (end - start);
	else
		return (0xffffffffUL - start + end + 1);
}

static int our_symbol_print(unsigned long addr, char *buffer)
{
	char str1[KSYM_NAME_LEN+1];
	const char *name;
	char *our_mod_name;
	unsigned long size, offset;

	name = kallsyms_lookup((unsigned long)(addr), 
			       &size,&offset,&our_mod_name,str1);

        if (!name)
		sprintf(buffer, "0x%lx", addr);
	else {
		if (our_mod_name)
			sprintf(buffer, "%s [%s]", name, our_mod_name);
		else
			sprintf(buffer, "%s", name);
	}
 
	return 0;
}

static void print_latency_softirqs(int cpu_nr, softirq_caller_struct_t *s)
{
	int i;
	char buf[256];
	unsigned int max_cpu = 0;
	unsigned int most_sig_percent =0, least_sig_percent=0;
		
	for (i =0; i < SIRQ_ARRAY_MAX_TOP; i++) {
		if (softirq_latency_top[cpu_nr][i].max_cycles == 0 ||
		    softirq_latency_top[cpu_nr][i].softirqcaller == NULL ) {
			break;
		}
		max_cpu = softirq_latency_top[cpu_nr][i].max_cycles / (unsigned int)(CYCLESFORPERCENT);
		most_sig_percent = (max_cpu / 100 );
		least_sig_percent = max_cpu - (most_sig_percent * 100);
		if (most_sig_percent >= 100)
		{
			most_sig_percent = 100;
			least_sig_percent = 0;
		}
		if (most_sig_percent < 50)
			break;
		our_symbol_print((unsigned long)(softirq_latency_top[cpu_nr][i].softirqcaller), buf);
		printk(KERN_EMERG "SIRQ: %10lu   %3u.%02u   %p  %s\n", softirq_latency_top[cpu_nr][i].max_cycles, most_sig_percent, least_sig_percent, softirq_latency_top[cpu_nr][i].softirqcaller, buf);
	}
}

static void print_count_softirqs(softirq_caller_struct_t *s)
{
	int i;
	char buf[256];
	unsigned int max_cpu = 0;
	unsigned int most_sig_percent =0, least_sig_percent=0;
		
	for (i =0; i < SIRQ_ARRAY_MAX_TOP; i++) {
		if (s[i].cpu_cycles == 0 || s[i].softirqcaller == NULL ) {
			break;
		}

		max_cpu = (unsigned int) (s[i].cpu_cycles) / (unsigned int)(CYCLESFORPERCENT);
		most_sig_percent = (max_cpu / 100 );
		least_sig_percent = max_cpu - (most_sig_percent * 100);
		if (most_sig_percent >= 100)
		{
			most_sig_percent = 100;
			least_sig_percent = 0;
		}
		if (most_sig_percent < 50)
			break;
		our_symbol_print((unsigned long)(s[i].softirqcaller),buf);
		printk(KERN_EMERG "SIRQ: %10llu   %3u.%02u %8llu   %p %s\n", s[i].cpu_cycles, most_sig_percent, least_sig_percent, s[i].count, s[i].softirqcaller, buf);
	}
}

void do_Nsec_processing(void)
{
	int i = 0, top_cnt = 0, dup_index = 0, again = 0, print_lat = 0, print_cnt = 0, cpu_nr = 0;
	unsigned int max_cpu;
	cycles_t tm_start;

	tm_start = get_cycles();
	/* insert into max latency list */
	for_each_possible_cpu(cpu_nr) {
		top_cnt = 0;
		print_lat = 0;
		again = 0;
		/* go through top latency list to see if new entries from the 1 second list can be added */
		for (i=0; i < SIRQ_ARRAY_MAX_TOP; i++) {

			/* exit if nothing to do */
			if ((softirq_latency_top[cpu_nr][SIRQ_ARRAY_MAX_TOP-1].max_cycles >= softirq_latency_cur[cpu_nr][top_cnt].max_cycles) || 
			    (top_cnt >= SIRQ_ARRAY_MAX_TOP))
				break;

			/* this happens when an entry is already higher on the list. Rerun with the next entry on current 1 second list */
			if (again)
			{
				i--;
				again = 0;
			}

			/* if we have a high latency entry, put it on the list */
			if (softirq_latency_top[cpu_nr][i].max_cycles < softirq_latency_cur[cpu_nr][top_cnt].max_cycles) {
				dup_index = find_softirq_func(softirq_latency_cur[cpu_nr][top_cnt].softirqcaller, softirq_latency_top[cpu_nr], SIRQ_ARRAY_MAX_TOP);
				/* check if function is already on the list */
				if (dup_index != -1)
				{
					/* shift the list down to insert */
					if (dup_index >= i)
					{
						/* don't update print field to avoid printing again */
						shift_softirq(softirq_latency_top[cpu_nr], i, dup_index+1);
						softirq_latency_top[cpu_nr][i].softirqcaller = softirq_latency_cur[cpu_nr][top_cnt].softirqcaller;
						softirq_latency_top[cpu_nr][i].count = softirq_latency_cur[cpu_nr][top_cnt].count;
						softirq_latency_top[cpu_nr][i].max_cycles = softirq_latency_cur[cpu_nr][top_cnt].max_cycles;
						softirq_latency_top[cpu_nr][i].cpu_cycles = softirq_latency_cur[cpu_nr][top_cnt].cpu_cycles;
					}
					/* skip this current entry and try the next as existing entry is higher */
					else
						again = 1;
				}
				/* put a new entry on the list */
				else
				{
					shift_softirq(softirq_latency_top[cpu_nr], i, SIRQ_ARRAY_MAX_TOP);
					memcpy(&softirq_latency_top[cpu_nr][i], &softirq_latency_cur[cpu_nr][top_cnt], sizeof(softirq_caller_struct_t));
				}
				top_cnt++;
			}
			if (softirq_latency_top[cpu_nr][i].print == 0)
			{
				max_cpu = softirq_latency_top[cpu_nr][i].max_cycles / (unsigned int)(CYCLESFORPERCENT) / 100;
				if (max_cpu >= 50)
				{
					softirq_latency_top[cpu_nr][i].print = 1;
					print_lat = 1;
				}
			}
		}
	} /* for cpu_nr =0...... */

	/* insert into count list */
	for_each_possible_cpu(cpu_nr) {
		top_cnt = 0;
		again = 0;
		print_cnt = 0;


		/* go through top max count list to see if new entries from the 1 second list can be added */
		for (i=0; i < SIRQ_ARRAY_MAX_TOP; i++) {

			/* exit if nothing to do */
			if ((softirq_count_top[cpu_nr][SIRQ_ARRAY_MAX_TOP-1].cpu_cycles >= softirq_count_cur[cpu_nr][top_cnt].cpu_cycles) || 
			    (top_cnt >= SIRQ_ARRAY_MAX_TOP))
				break;

			/* this happens when an entry is already higher on the list. Rerun with the next entry on current 1 second list */
			if (again)
			{
				i--;
				again = 0;
			}

			/* skip entries with only 1 count as this is picked up in the latency list */
			if (softirq_count_cur[cpu_nr][top_cnt].count == 1)
			{
				top_cnt++;
				again = 1;
				continue;
			}

			/* if we have a high latency entry, put it on the list */
			if (softirq_count_top[cpu_nr][i].cpu_cycles < softirq_count_cur[cpu_nr][top_cnt].cpu_cycles) {
				dup_index = find_softirq_func(softirq_count_cur[cpu_nr][top_cnt].softirqcaller, softirq_count_top[cpu_nr], SIRQ_ARRAY_MAX_TOP);

				/* check if function is already on the list */
				if (dup_index != -1)
				{
					/* skip this current entry and try the next as existing entry is higher */
					if (dup_index >= i)
					{
						/* don't update print field to avoid printing again */
						shift_softirq(softirq_count_top[cpu_nr], i, dup_index+1);
						softirq_count_top[cpu_nr][i].softirqcaller = softirq_count_cur[cpu_nr][top_cnt].softirqcaller;
						softirq_count_top[cpu_nr][i].count = softirq_count_cur[cpu_nr][top_cnt].count;
						softirq_count_top[cpu_nr][i].max_cycles = softirq_count_cur[cpu_nr][top_cnt].max_cycles;
						softirq_count_top[cpu_nr][i].cpu_cycles = softirq_count_cur[cpu_nr][top_cnt].cpu_cycles;
					}
					/* skip this current entry and try the next as existing entry is higher */
					else
						again = 1;
				}
				/* put a new entry on the list */
				else
				{
					shift_softirq(softirq_count_top[cpu_nr], i, SIRQ_ARRAY_MAX_TOP);
					memcpy(&softirq_count_top[cpu_nr][i], &softirq_count_cur[cpu_nr][top_cnt], sizeof(softirq_caller_struct_t));
				}
				top_cnt++;
			}
			if (softirq_count_top[cpu_nr][i].print == 0)
			{
				max_cpu = (unsigned int)(softirq_count_top[cpu_nr][i].cpu_cycles) / (unsigned int)(CYCLESFORPERCENT) / 100;
				if (max_cpu >= 50)
				{
					softirq_count_top[cpu_nr][i].print = 1;
					print_cnt = 1;
				}
			}
		}


		/* clear 1 second lists */
		memset(softirq_latency_cur[cpu_nr], 0, sizeof(softirq_latency_cur[cpu_nr]));
		memset(softirq_count_cur[cpu_nr], 0, sizeof(softirq_count_cur[cpu_nr]));

	} /* end of  for (cpu_nr =0.... )  */
	tm_sec = calc_cycles(tm_start, get_cycles());
	tm_sec_sum += tm_sec;
	sec_cnt++;

}

void insert_softirq_caller(int cpu_nr, void *func, cycles_t start, cycles_t end)
{
	int i, dup_index;
	int found;
	cycles_t diff, tm_start;
	softirq_caller_struct_t *s, tmp;


	if (softirqs_enabled) {
		tm_start = get_cycles();
		diff = calc_cycles(start, end);

		/* insert into max latency list */
		s = softirq_latency_cur[cpu_nr];
		dup_index = find_softirq_func(func, s, SIRQ_ARRAY_MAX_CUR);

		/* if function is already on list, then increment count and reorder it */
		if (dup_index != -1)
		{
			s[dup_index].cpu_cycles += diff;

			/* check if we need to reorder the list */
			if (s[dup_index].max_cycles < diff)
			{
				/* find new spot on list */
				for (i=0; i < SIRQ_ARRAY_MAX_CUR; i++) {
					if (s[i].max_cycles < diff)
						break;
				}

				/* if we need to move up, then shift everything down so we can move it to the proper spot */
				if (i < dup_index)
				{
					memcpy(&tmp, &s[dup_index], sizeof(softirq_caller_struct_t));
					tmp.max_cycles = diff;
					shift_softirq(s, i, dup_index+1);
					memcpy(&s[i], &tmp, sizeof(softirq_caller_struct_t));
				}
			}
		}
		/* not on list yet.  Add it to the ordered list */
		else
		{
			/* find correct spot to keep list ordered */
			for (i=0; i < SIRQ_ARRAY_MAX_CUR; i++) {
				if (s[i].max_cycles < diff) {
					/* insert new entry */
					shift_softirq(s, i, SIRQ_ARRAY_MAX_CUR);
					s[i].softirqcaller =  func;
					s[i].count = 1;
					s[i].max_cycles = diff;
					s[i].cpu_cycles = diff;
					s[i].print = 0;
					break;
				}
			}
		}

		/* insert into count list */
		s = softirq_count_cur[cpu_nr];

		found = 0;

		/* check if function is already on the list */
		dup_index = find_softirq_func(func, s, SIRQ_ARRAY_MAX_CUR);
		if (dup_index != -1)
		{
			s[dup_index].count++;
			s[dup_index].cpu_cycles += diff;

			/* find correct spot on ordered list */
			for (i=0; i < SIRQ_ARRAY_MAX_CUR; i++) {
				if (s[i].cpu_cycles < s[dup_index].cpu_cycles)
					break;
			}

			/* shift everything down and put it on the list */
			if (i < dup_index)
			{
				memcpy(&tmp, &s[dup_index], sizeof(softirq_caller_struct_t));
				shift_softirq(s, i, dup_index+1);
				memcpy(&s[i], &tmp, sizeof(softirq_caller_struct_t));
			}
			found = 1;
		} 
		/* new funtion to add to list */
		else 
		{
			/* if we have a free spot, put it in immediately */
			for (i=0; i < SIRQ_ARRAY_MAX_CUR; i++) {
				if (s[i].cpu_cycles == 0 ) {
					s[i].softirqcaller =  func;
					s[i].count = 1;
					s[i].max_cycles = diff;
					s[i].cpu_cycles = diff;
					s[i].print = 0;
					found = 1;
					break;
				}
			}
		}

		/* out of space, so just replace the last entry */
		if (!found) {
			s[SIRQ_ARRAY_MAX_CUR-1].softirqcaller =  func;
			s[SIRQ_ARRAY_MAX_CUR-1].count = 1;
			s[SIRQ_ARRAY_MAX_CUR-1].max_cycles = diff;
			s[SIRQ_ARRAY_MAX_CUR-1].cpu_cycles = diff;
			s[SIRQ_ARRAY_MAX_CUR-1].print = 0;
		}

		tm_insert = calc_cycles(tm_start, get_cycles());
		tm_sum += tm_insert;
		insert_cnt++;

		/* check if 10 sec has elapsed, and do the calculations  */
		if (calc_cycles(time_cycles, end) >= (tb_cycles_per_sec * SIRQ_TIME_INTERVAL) ) { 
			time_cycles = end;
			do_Nsec_processing();
		}
	} 
	return;


}

EXPORT_SYMBOL(insert_softirq_caller);

#endif /* CONFIG_SILKWORM */

/*
 * /proc/softirqs  ... display the number of softirqs
 */
static int show_softirqs(struct seq_file *p, void *v)
{
	int i, j;
#ifdef CONFIG_SILKWORM
	int  ret = 0, cpu_nr = 0;
	char buf[256];
	unsigned int max_cpu = 0;
	unsigned int most_sig_percent =0, least_sig_percent=0;
		
	mutex_lock(&softirqs_file_mutex);
#endif /* CONFIG_SILKWORM */
	seq_printf(p, "                ");
	for_each_possible_cpu(i)
		seq_printf(p, "CPU%-8d", i);
	seq_printf(p, "\n");

	for (i = 0; i < NR_SOFTIRQS; i++) {
		seq_printf(p, "%8s:", softirq_to_name[i]);
		for_each_possible_cpu(j)
			seq_printf(p, " %10u", kstat_softirqs_cpu(i, j));
		seq_printf(p, "\n");
	}

#ifdef CONFIG_SILKWORM
	seq_printf(p, "\n");
	if (!softirqs_enabled ) {
		mutex_unlock(&softirqs_file_mutex);
		return  0;
	}

	seq_printf(p, "   Maximum Softirq Latency (cycles) in %d Seconds\n", SIRQ_TIME_INTERVAL);
	seq_printf(p, "   Latency |  CPU %% | Softirq\n");
	for_each_possible_cpu(cpu_nr) {
		seq_printf(p, "\n    CPU %d \n", cpu_nr);

		for (i =0; i < SIRQ_ARRAY_MAX_TOP; i++) {
			max_cpu = softirq_latency_top[cpu_nr][i].max_cycles;
			seq_printf(p, "%10u  ", max_cpu);
			max_cpu = max_cpu / (unsigned int)(CYCLESFORPERCENT);
			most_sig_percent = (max_cpu / 100 );
			least_sig_percent = max_cpu - (most_sig_percent * 100);
			if (most_sig_percent >= 100)
			{
				most_sig_percent = 100;
				least_sig_percent = 0;
			}
			seq_printf(p, " %3u.", most_sig_percent);
			seq_printf(p, "%02u ", least_sig_percent);
			seq_printf(p, "  %p ", softirq_latency_top[cpu_nr][i].softirqcaller);
			ret  = our_symbol_print((unsigned long)(softirq_latency_top[cpu_nr][i].softirqcaller), buf);
			seq_printf(p, "%s\n", buf);
		}
	} /* end of for (cpu_nr=0 ...*/

	seq_printf(p, "\nSoftirqs Activated in %d Seconds\n", SIRQ_TIME_INTERVAL);
	seq_printf(p, "Tot Cycles |  CPU %% |  Count | Softirq\n");
	for_each_possible_cpu(cpu_nr) {

		seq_printf(p, "\n");
		seq_printf(p, "    CPU %d \n", cpu_nr);

		for (i =0; i < SIRQ_ARRAY_MAX_TOP; i++) {
			seq_printf(p, "%10llu ", softirq_count_top[cpu_nr][i].cpu_cycles);
			max_cpu = (unsigned int) (softirq_count_top[cpu_nr][i].cpu_cycles) / (unsigned int)(CYCLESFORPERCENT);
			most_sig_percent = (max_cpu / 100 );
			least_sig_percent = max_cpu - (most_sig_percent * 100);
			if (most_sig_percent >= 100)
			{
				most_sig_percent = 100;
				least_sig_percent = 0;
			}
			seq_printf(p, "  %3u.", most_sig_percent);
			seq_printf(p, "%02u ", least_sig_percent);
			seq_printf(p, "%8llu   ", softirq_count_top[cpu_nr][i].count);		
			seq_printf(p, "%p ", softirq_count_top[cpu_nr][i].softirqcaller);
			ret = our_symbol_print((unsigned long)(softirq_count_top[cpu_nr][i].softirqcaller),buf);
			seq_printf(p, "%s\n", buf);
		}
	} /* for (cpu_nr=) ......*/
	seq_printf(p, "\ncycles/sec=%d tm_insert=%lu avg=%lu tm_sec=%lu avg=%lu ins_cnt=%lu sec_cnt=%lu\n\n", 
		   tb_cycles_per_sec, tm_insert, tm_sum/(insert_cnt+1), tm_sec, tm_sec_sum/(sec_cnt+1), insert_cnt, sec_cnt);

	mutex_unlock(&softirqs_file_mutex);

#endif  /* CONFIG_SILKWORM */	
	return 0;
}

static int softirqs_open(struct inode *inode, struct file *file)
{
	return single_open(file, show_softirqs, NULL);
}

#ifdef CONFIG_SILKWORM

static void clear_counters(void) 
{
	memset(softirq_latency_cur, 0, sizeof(softirq_latency_cur));
	memset(softirq_count_cur, 0, sizeof(softirq_count_cur));
	memset(softirq_latency_top, 0, sizeof(softirq_latency_top));
	memset(softirq_count_top, 0, sizeof(softirq_count_top));
	tm_insert = tm_sec = tm_sec_sum = tm_sum = 0;
	insert_cnt = sec_cnt = 0;
	return;
}

static int softirqs_write(struct file *file, const char __user *user, size_t insize, 
			  loff_t *offset)
{
	int len  = 8, i;
	unsigned char output_enable[9];
	unsigned int user_input;
	int result  = 0;


	mutex_lock(&softirqs_file_mutex);

	if ( insize < len )
		len  = insize;
	result = copy_from_user(output_enable,  user, len );
	mutex_unlock(&softirqs_file_mutex);


	if (result) {
		return (-EFAULT);
	}
	i = sscanf(output_enable,"%d", &user_input);


	mutex_lock(&softirqs_file_mutex);
	if (user_input > 10 ) {
		if (softirqs_enabled) {
			softirqs_enabled = 0;
			msleep_interruptible(200); /* wait for 200 msec for any softirq to finish */
			clear_counters();
			softirqs_enabled = 1;
			printk(KERN_INFO "User cleared statistics after softirqs  by  = %x\n", user_input);
		} else {
		printk(KERN_INFO "Softirq statistics already disabled\n");
		}

	} else if (user_input > 0) {
		softirqs_enabled = 1;
		printk(KERN_INFO "User Enabled Softirq statistics collection by  = %x\n", user_input);
		
	} else {
		softirqs_enabled  = 0;
		printk(KERN_INFO "User Disabled Softirq statistics collection by = %x\n", user_input);
	}
	mutex_unlock(&softirqs_file_mutex);


	return  SIRQ_ARRAY_MAX_TOP;
}

#endif  /* CONFIG_SILKWORM */
		
static const struct file_operations proc_softirqs_operations = {
	.open 		= softirqs_open,
	.read 		= seq_read,
	.llseek 	= seq_lseek,
	.release 	= single_release,
#ifdef CONFIG_SILKWORM
	.write 		= softirqs_write,
#endif /* CONFIG_SILKWORM */
};

static int __init proc_softirqs_init(void)
{
#ifdef CONFIG_SILKWORM
	proc_create("softirqs", 0666, NULL, &proc_softirqs_operations);
	tb_cycles_per_sec = tb_ticks_per_jiffy * HZ;
	time_cycles = get_cycles();
	memset(softirq_latency_cur, 0, sizeof(softirq_latency_cur));
	memset(softirq_count_cur, 0, sizeof(softirq_count_cur));
	memset(softirq_latency_top, 0, sizeof(softirq_latency_top));
	memset(softirq_count_top, 0, sizeof(softirq_count_top));
	tm_insert = tm_sec = tm_sec_sum = tm_sum = 0;
	insert_cnt = sec_cnt = 0;
	softirqs_enabled = 1;
#else
	 proc_create("softirqs", 0, NULL, &proc_softirqs_operations);
#endif /* CONFIG_SILKWORM */
	return 0;
}
module_init(proc_softirqs_init); 
