/* 
 * Header file for Brocade polled-mode ATA dump driver
 * Loosely based on Rusty Russell's OOPSER patch.
 */
#ifndef _ATA_DUMPER_H
#define _ATA_DUMPER_H

#include <linux/ioctl.h>

#define ATA_DUMPER_MAGIC		('a')
#define ATA_DUMPER_IOC_CLEAR_MAP	_IOW(ATA_DUMPER_MAGIC, 0, unsigned int)
#define ATA_DUMPER_IOC_ADD_MAP		_IOW(ATA_DUMPER_MAGIC, 1, struct ata_dumper_blkmap)
#define ATA_DUMPER_IOC_ARM		_IOW(ATA_DUMPER_MAGIC, 2, unsigned int)
#define ATA_DUMPER_IOC_DISARM		_IOW(ATA_DUMPER_MAGIC, 3, unsigned int)
#define ATA_DUMPER_IOC_TEST_EXCEPTION	_IO(ATA_DUMPER_MAGIC, 4)
#define ATA_DUMPER_IOC_TEST_DIE		_IO(ATA_DUMPER_MAGIC, 5)
#define ATA_DUMPER_IOC_ALLOC		_IOW(ATA_DUMPER_MAGIC, 6, unsigned int)
#define ATA_DUMPER_IOC_FREE		_IOW(ATA_DUMPER_MAGIC, 7, unsigned int)

/* The 'blocks' are filesystem block numbers.  Tell the kernel which device, 
 * and let the driver figure out the LBA sector numbers.
 */
struct ata_dumper_blkmap {
	unsigned int map_id;		/* ID of map */
	unsigned long device;		/* cannot use dev_t here... */
	int num_blocks;	/* size of array */
	unsigned long *blks;		/* array of blocks */
};

#define ATA_DUMPER_MAP_ID_TRACE 1
#define ATA_DUMPER_MAP_ID_PANIC_DUMP 2
#define ATA_DUMPER_MAP_ID_RB_REASON 3

#ifdef __KERNEL__

int ata_dump_write(int handle, char *buf, int size);
int ata_dump_open(unsigned int map_id);
int ata_dump_close(int handle);

#endif /* __KERNEL__ */

#endif /* !(_ATA_DUMPER_H) */
