
#ifndef _LINUX_MARKER_BROCADE_H
#define _LINUX_MARKER_BROCADE_H

#define GENERIC_TP_SIGNATURE "_replace_with_fos_module_name__"

#include <linux/marker.h>

#ifdef CONFIG_MARKERS
#define CONFIG_MARKERS_BROCADE
#else
#undef CONFIG_MARKERS_BROCADE
#endif

/*
 * Code markup for dynamic and static tracing.
 *
 * See Documentation/marker.txt.
 *
 * (C) Copyright 2006 Mathieu Desnoyers <mathieu.desnoyers@polymtl.ca>
 *
 * This file is released under the GPLv2.
 * See the file COPYING for more details.
 */

#include <stdarg.h>
#include <linux/types.h>
#include <linux/immediate.h>

#ifdef CONFIG_MARKERS_BROCADE


extern marker_probe_func __mark_empty_function_brocade;

extern void marker_probe_cb_brocade(const struct marker *mdata,
	void *call_private, ...);

/* define Brocade's own that makes reference to:
 1) mark_empty_function_brocade 
 2) marker_probe_cb_brocade
 */
#undef _DEFINE_MARKER
#define _DEFINE_MARKER(channel, name, tp_name_str, tp_cb, format)	\
		static const char __mstrtab_##channel##_##name[]	\
		__attribute__((section("__markers_strings")))		\
		= #channel "\0" #name "\0" format;			\
		static struct marker __mark_##channel##_##name		\
		__attribute__((section("__markers"), aligned(8))) =	\
		{ __mstrtab_##channel##_##name,				\
		  &__mstrtab_##channel##_##name[sizeof(#channel)],	\
		  &__mstrtab_##channel##_##name[sizeof(#channel) +	\
						sizeof(#name)],		\
		  0, 0, 0, 0, marker_probe_cb_brocade,				\
		  { __mark_empty_function_brocade, NULL},			\
		  NULL, tp_name_str, tp_cb }


#undef DEFINE_MARKER_TP
#define DEFINE_MARKER_TP(channel, name, tp_name, tp_cb, format)		\
		_DEFINE_MARKER(channel, name, #tp_name, tp_cb, format)

#undef __trace_mark_tp
#define __trace_mark_tp(channel, name, call_private, tp_name, tp_cb,	\
			format, args...)				\
	do {								\
		void __check_tp_type(void)				\
		{							\
			register_trace_##tp_name(tp_cb);		\
		}							\
		DEFINE_MARKER_TP(channel, name, tp_name, tp_cb, format);\
		__mark_check_format(format, ## args);			\
		(*__mark_##channel##_##name.call)(&__mark_##channel##_##name, \
			call_private, ## args);				\
	} while (0)


/**
 * trace_mark_tp - Marker in a tracepoint callback
 * @channel: marker channel (where to send the data), not quoted.
 * @name: marker name, not quoted.
 * @tp_name: tracepoint name, not quoted.
 * @tp_cb: tracepoint callback. Should have an associated global symbol so it
 *         is not optimized away by the compiler (should not be static).
 * @format: format string
 * @args...: variable argument list
 *
 * Places a marker in a tracepoint callback.
 */
#undef trace_mark_tp
#define trace_mark_tp(channel, name, tp_name, tp_cb, format, args...)	\
	__trace_mark_tp(channel, name, NULL, tp_name, tp_cb, format, ## args)


/*
 * This macro is specific to fos generic tracepoints. Tracepoint and marker 
 * have the _replace_with_fos_module_name\0" string tacked on at the end of LTT could concatenate the
 * module name at the time the module is inserted. The fist '\0' character is
 * followed by the number of padded characters, 32, for instance, so that at
 * concatenttion time we do not overrun the name.
*/

#define _DEFINE_MARKER_FOS_GENERIC(channel, name, tp_name_str, tp_cb, format)	\
		static const char __mstrtab_##channel##_##name[]	\
		__attribute__((section("__markers_strings")))		\
		= #channel "\0"  #name GENERIC_TP_SIGNATURE "\0" format;  \
		static struct marker __mark_##channel##_##name		\
		__attribute__((section("__markers"), aligned(8))) =	\
		{ __mstrtab_##channel##_##name,				\
		  &__mstrtab_##channel##_##name[sizeof(#channel)],	\
		  &__mstrtab_##channel##_##name[sizeof(#channel) +	\
						sizeof(#name) + sizeof(GENERIC_TP_SIGNATURE) - 1],		\
		  0, 0, 0, 0, marker_probe_cb_brocade,				\
		  { __mark_empty_function_brocade, NULL},			\
		  NULL, tp_name_str GENERIC_TP_SIGNATURE, tp_cb }


#define DEFINE_MARKER_TP_FOS_GENERIC(channel, name, tp_name, tp_cb, format)		\
		_DEFINE_MARKER_FOS_GENERIC(channel, name, #tp_name, tp_cb, format)

#define __trace_mark_tp_fos_generic(channel, name, call_private, tp_name, tp_cb,	\
			format, args...)				\
	do {								\
		void __check_tp_type(void)				\
		{							\
			register_trace_##tp_name(tp_cb);		\
		}							\
		DEFINE_MARKER_TP_FOS_GENERIC(channel, name, tp_name, tp_cb, format);\
		__mark_check_format(format, ## args);			\
		(*__mark_##channel##_##name.call)(&__mark_##channel##_##name, \
			call_private, ## args);				\
	} while (0)

/* generic tracepoints */
#define trace_mark_tp_fos_generic(channel, name, tp_name, tp_cb, format, args...)	\
	__trace_mark_tp_fos_generic(channel, name, NULL, tp_name, tp_cb, format, ## args)

#define DEFINE_TRACE_FN_GENERIC(name, reg, unreg)				\
	static const char __tpstrtab_##name[]				\
	__attribute__((section("__tracepoints_strings"))) = #name GENERIC_TP_SIGNATURE;	\
	struct tracepoint __tracepoint_##name				\
	__attribute__((section("__tracepoints"), aligned(32))) =	\
		{ __tpstrtab_##name, 0, reg, unreg, NULL }

#define DEFINE_TRACE_GENERIC(name)						\
	DEFINE_TRACE_FN_GENERIC(name, NULL, NULL);


#else /* !CONFIG_MARKERS_BROCADE */

#define DEFINE_TRACE_GENERIC DEFINE_TRACE_NOP

#define trace_mark_tp_fos_generic(channel, name, tp_name, tp_cb, format, args...)	\
	__trace_mark_tp(channel, name, NULL, tp_name, tp_cb, format, ## args)

#endif /* CONFIG_MARKERS_BROCADE */

#endif
