/*
 * include/brocade/cpld.h
 *
 * Definitions for the Brocade Silkworm product family CPLD
 *
 * Author:
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or
 * (at your option) any later version.
 */

#ifndef _BROCADE_CPLD_H
#define _BROCADE_CPLD_H

#define CPLD_BASE_ADDRESS	0xF8000000

/* Meteor (physical) base addresses */
#define MET_MGMT_BASE_ADDR	0x1F8000000ULL	/* CP, PB, & Core mgmt regs */
#define MET_PDC_BASE_ADDR	0x1F8100000ULL	/* Private Data Channel regs */
#define MET_PDC_SLOT_SIZE	0x20		/* PDC registers per slot */
#define MET_PDC_SLOTS		0x0A		/* number of PDC slots */
#define MET_PDC_SIZE		((MET_PDC_SLOTS) * (MET_PDC_SLOT_SIZE))

/* Work around for condor bug related to PCI hang
 * The following physical memory is reserved in a set of platforms
 * The Condor descriptors pages will required to be allocated from these
 * physical addresses
 * Early kernel bootup code reserves this part of memory for Condor use
 */
#define MET_CONDOR_DESC_BASE_ADDR 0x006000000ULL
#define MET_CONDOR_DESC_SIZE	  0x00040000

/*
 * Define main-CPU memory region to be shared with the co-CPU.
 * The start address must be an even multiple of the size.
 * Chose the base address to start after the condor descriptor
 * memory. This allows both regions to be reserved as a single
 * memory block.
 */
#define NEPTUNE_CONDOR_DESC_BASE_ADDR 0x06000000UL
#define NEPTUNE_CONDOR_DESC_SIZE	  0x00048000UL

#define MAINCPU_SHARED_MEM_BASE_ADDR  0x08000000UL
#define MAINCPU_SHARED_MEM_SIZE       0x08000000UL

#define MAINCPU_RESERVED_MEM_BASE_ADDR NEPTUNE_CONDOR_DESC_BASE_ADDR
#define MAINCPU_RESERVED_MEM_SIZE     (MAINCPU_SHARED_MEM_BASE_ADDR + \
									   MAINCPU_SHARED_MEM_SIZE - \
									   MAINCPU_RESERVED_MEM_BASE_ADDR)

/*
 * CPLD arbitrate i2c mux error definition
 */
#define I2C_CPLD_BASE_ERROR	-20
#define I2C_TIMEOUT_ERR		-21	
#define I2C_CONFLICT_ERR	-22	
#define I2C_FATAL_ERR		-23	
#define I2C_PASS_RECOVER_ERR	-24	
#define I2C_CP_ERR		-25	
#define I2C_FENCE_ERR		-26

/*
 * Ulysses
 */

#define CPLD_BLADE_ATTEN_MASK	(CPLD_BASE_ADDRESS | 0x04)
#define CPLD_BLADE_FENCE_CTL	(CPLD_BASE_ADDRESS | 0x05)
#define CPLD_BLADE_FORCE_ERROR	(CPLD_BASE_ADDRESS | 0x06)
#define CPLD_BRIDGE_PS_MASK	(CPLD_BASE_ADDRESS | 0x0A)
#define CPLD_CP_STATE_MASK	(CPLD_BASE_ADDRESS | 0x12)
#define CPLD_STATUS_OFFSET	0x13

#define CPLD_CP_FENCE_OFFSET 0x14
#define CPLD_CP_FENCE_CTL	(CPLD_BASE_ADDRESS | CPLD_CP_FENCE_OFFSET)
#define  CPLD_CP_FENCE_PCI	0x20
#define  CPLD_CP_FENCE_HOUSEKEEPING 0x10
#define  CPLD_CP_FENCE_PWRUP	(CPLD_CP_FENCE_PCI | CPLD_CP_FENCE_HOUSEKEEPING)
#define  CPLD_CP_FORCE_FENCE_IN	0x80
#define CPLD_CP_FORCE_ERROR	(CPLD_BASE_ADDRESS | 0x15)
#define  CPLD_THIS_CP_LED	0x8
#define  CPLD_OTHER_CP_LED	0x1
#define CPLD_PCI_RESET		(CPLD_BASE_ADDRESS | 0x16)
#define  CPLD_PCI_BUS		0x2
#define  CPLD_PCI_BUS2		0x4
#define  CPLD_PCI_ALL_BUSSES	(CPLD_PCI_BUS1 | CPLD_PCI_BUS2)
#define CPLD_PCI_MASTER		(CPLD_BASE_ADDRESS | 0x16)
#define  PCI_BUS1_MASTER_IN	0x2
#define  PCI_BUS2_MASTER_IN	0x4
#define  PCI_BUSSES_MASTER_IN	(PCI_BUS1_MASTER_IN | PCI_BUS2_MASTER_IN)
#define  PCI_BUS1_MASTER_OUT	0x20
#define  PCI_BUS2_MASTER_OUT	0x40
#define  PCI_BUSSES_MASTER_OUT	(PCI_BUS1_MASTER_OUT | PCI_BUS2_MASTER_OUT)
#define  PCI_BUSSES_MASTERS	(PCI_BUSSES_MASTER_IN | PCI_BUSSES_MASTER_OUT)
#define CPLD_PCI_STATUS		(CPLD_BASE_ADDRESS | 0x17)

#define MISC_WRITE_OFFSET	0x18
#define CPLD_MISC_WRITE		(CPLD_BASE_ADDRESS | MISC_WRITE_OFFSET)
#define  CPLD_CP_ERROR_OUT	0x04  // 1 = error, 0 = no error
#define  CPLD_HA_MASTER_OUT	0x08
#define  CPLD_CP_FCCLK_ON	0x10
#define  CPLD_CP_CTR_STOP	0x20

#define MISC_READ_OFFSET	0x19
#define CPLD_MISC_READ		(CPLD_BASE_ADDRESS | MISC_READ_OFFSET)
#define  CPLD_SLOT_ID_FIELD	0x1
#define  CPLD_HA_MASTER_IN	0x08
#define I2C_SELECT_OFFSET	0x1A
#define CPLD_I2C_SELECT		(CPLD_BASE_ADDRESS | I2C_SELECT_OFFSET)
#define  CPLD_CP0_I2C_ADDRESS	0x5
#define  CPLD_CP1_I2C_ADDRESS	0x6
#define  ULYSSES_CP0_SLOT	CPLD_CP0_I2C_ADDRESS
#define  ULYSSES_CP1_SLOT	CPLD_CP1_I2C_ADDRESS
#define  CPLD_I2C_SEL_MASK	0x1F


#define CPLD_BOOTFLASH_CTRL	(CPLD_BASE_ADDRESS | 0x1B)
#define  CPLD_BOOTFLASH_WRITE_BITS 0x01 /* 1 = allow, 0 = disable */
#define CPLD_USERFLASH_CTRL	(CPLD_BASE_ADDRESS | 0x1C)

/*
 * Ulysse and Meteor - Common
 */
#define  CP_I2C_SELECT_MASK	0x1F    /* I2C unit select - bit 0-4 */
#define  CP_I2C_RD_ACTIVE_MASK	0x20	/* I2C read-active - bit 5 */
#define  CP_I2C_WR_ACTIVE_MASK	0x40	/* I2C write-active - bit 6 */
#define  CP_I2C_WRITE_MASK	0x80
#define  CP_I2C_CONTROL_MASK	(CP_I2C_WRITE_MASK | \
				CP_I2C_WR_ACTIVE_MASK | \
				CP_I2C_RD_ACTIVE_MASK)
/*
 * Terminator
 */

#define CPLD_TERM_BLOOM_IRQ_MASK (CPLD_BASE_ADDRESS | 0x48) /* Dflt to 0xFF */
#define CPLD_TERM_PERIF_IRQ_MASK (CPLD_BASE_ADDRESS | 0x58) /* Dflt to 0xFF */
#define CPLD_TERM_SWITCH_RST_MASK (CPLD_BASE_ADDRESS | 0x64) /* Dflt to 0xFF */
#define CPLD_TERM_SYSTEM_RST_ENABLE (CPLD_BASE_ADDRESS | 0x6C) /* Dflt to 0 */
#define CPLD_TERM_GLOBAL_RST_ENABLE (CPLD_BASE_ADDRESS | 0x6C) /* Dflt to 0 */

#define CPLD_TERM_BOOTFLASH_CTRL (CPLD_BASE_ADDRESS | 0x90) /* Default 0 */
#define  CPLD_TERM_BOOTFLASH_WRITE_BITS 1  /* 1 = allow, 0 = disable */
#define CPLD_TERM_USERFLASH_CTRL (CPLD_BASE_ADDRESS | 0x94) /* Default 0 */

#define CPLD_TERM_LED_CTRL	(CPLD_BASE_ADDRESS | 0xA0)
#define  CPLD_TERM_POWER_LED	0x01
#define  CPLD_TERM_WARN_LED	0x02
#define	 CPLD_TERM_BLINK_LED	0x04

#define CPLD_TERM_NMI_MASK	(CPLD_BASE_ADDRESS | 0xA2) /* Default to 0xFF */
#define CPLD_TERM_PCI_RST_MASK	(CPLD_BASE_ADDRESS | 0xA8) /* Default to 0xFF */


/*------------------------------------------------------------------------------
 * Meteor
 *----------------------------------------------------------------------------*/

/* Register offsets */
#define MET_CPLD_REV		0x00
#define MET_PB_STATE_CHG	0x01
#define MET_PB_STATE_CHG_SET	0x02
#define MET_PB_STATE_CHG_MASK	0x03
#define MET_PB_ATTN		0x04
#define MET_PB_ATTN_MASK	0x05
#define MET_PB_PRESENT		0x06
#define MET_PB_LED		0x07
#define MET_PB_FENCE		0x08
#define MET_PB_PWR_FAIL		0x09
#define MET_PB_PWR_FAIL_MASK	0x0A
#define MET_PB_PWR_FAIL_SET	0x0B
#define MET_CP_ATTN		0x0C
#define MET_CP_ATTN_MASK	0x0D
#define MET_CORE_ATTN		0x0E
#define MET_CORE_ATTN_MASK	0x0F
#define MET_CORE_FENCE		0x10
#define MET_PS_BRIDGE		0x11
#define MET_PS_BRIDGE_SET	0x12
#define MET_PS_BRIDGE_MASK	0x13
#define MET_PS4_STATUS		0x14
#define MET_PS3_STATUS		0x15
#define MET_PS2_STATUS		0x16
#define MET_PS1_STATUS		0x17
#define MET_PS_INT_CAUSE_SEL	0x18
#define MET_CP_STATE_CHG	0x19
#define MET_CP_STATE_CHG_SET	0x1A
#define MET_CP_STATE_CHG_MASK	0x1B
#define MET_CP_PCI_FENCE	0x1C
#define MET_CP_HK_FENCE		0x1D
#define MET_CP_LED		0x1E
#define MET_PCI_ARB_RESET	0x1F
#define MET_PCI_MASTER_ENB_CHG	0x20
#define MET_PCI_MASTER_STATUS	0x21
#define MET_CP_MISC_WRITE	0x22
#define MET_CP_MISC_READ	0x23
#define MET_SW_RESV		0x24
#define MET_BOOTROM_CTRL	0x25
#define MET_USER_FLASH_CTRL	0x26
#define MET_DEADMAN_CTR_TC	0x27
#define MET_BACKPLANE_VER	0x28
#define MET_CP_MISC_ERR		0x29
#define MET_I2C_SELECT		0x2A
#define MET_PDC_STATUS		0x30
#define MET_PDC_SLOT_SELECT	0x31
#define MET_PDC_COMMAND		0x32
#define MET_PDC_PB_ERR		0x33
#define MET_PDC_PB_ERR_MASK	0x34
#define MET_PDC_CORE_ERR	0x35
#define MET_PDC_CORE_ERR_MASK	0x36

#define MET_CP0_SLOT		MET_I2C_SEL_CP0
#define MET_CP1_SLOT		MET_I2C_SEL_CP1

/* Register masks */

/*
 * 0x0C - CP Attention status and mask
 */
#define MET_OCP_PWR_FAIL		0x01
#define MET_OCP_CP_SECTION_ERR		0x02
#define MET_OCP_CORE_SECTION_ERR	0x04
#define MET_OCP_MASTER_CHANGE		0x08
#define MET_OCP_ATTN_MASK		0x0F

#define MET_CP_DM_CNTR_EXP		0x10
#define MET_CP_SLOT_ID_ERR		0x20
#define MET_CP_PARITY_ERR		0x40
#define MET_THIS_CP_ATTN_MASK		0x70
#define MET_CPLD_PDC_ERR		0x80

/*
 * 0x0E - Core Attention Interrupt and Mask
 */
#define MET_CORE0_ASIC_ATTN		0x01
#define MET_CORE1_ASIC_ATTN		0x02
#define MET_CORE0_PWR_FAIL		0x04
#define MET_CORE1_PWR_FAIL		0x08
#define MET_CORE_ASIC_ATTN_MASK		0x03
#define MET_CORE_PWR_FAIL_MASK		0x0C

/*
 * 0x13, 0x14, 0x15, 0x16, 0x17 - PS Interrupt and Mask
 */
#define MET_CP_PS_AC_FAIL_MASK		0x01    /* AC line voltage */
#define MET_CP_PS_PW_OK_MASK		0x02    /* PS output level */
#define MET_CP_PS_PR_FAIL_MASK		0x04    /* predictive failure */
#define MET_CP_PS_ALL_FAIL_MASK		0x08    /* error condition */
#define MET_CP_PS_ABSENT_MASK		0x10    /* absent bit */
#define MET_CP_PS_ALL_MASKS		0x1F    /* all of the above */

/*
 * 0x18, 0x19, 0x1A - CP state change interrupt status and mask
 */
#define MET_OCP_STATE_CHG		0x01
#define MET_OCP_INSTALLED		0x10

/*
 * 0x1C - PCI fence Control
 */
#define MET_PCI_SELF_FENCE		0x01
#define MET_PCI_BUS1_FENCE_IN		0x02
#define MET_PCI_BUS2_FENCE_IN		0x04
#define MET_PCI_BUS3_FENCE_IN		0x08
#define MET_PCI_ALL_BUS_FENCE_IN	0x0E

#define MET_PCI_FENCE_OCP_BUS1		0x10
#define MET_PCI_FENCE_OCP_BUS2		0x20
#define MET_PCI_FENCE_OCP_BUS3		0x40
#define MET_PCI_FENCE_OCP_ALL		0x70

/*
 * 0x1D - House-keeping fence Control
 */
#define MET_HK_SELF_FENCE		0x01
#define MET_HK_FORCE_FENCE_IN		0x02
#define MET_HK_FORCE_FENCE_OUT		0x04
#define MET_HK_PDC_SELF_FENCE		0x08

/*
 * 0x1E - CP LED Control
 */
#define MET_OCP_ATTN_LED_ON		0x01
#define MET_CP_ATTN_LED_ON		0x02
#define MET_CP_ACTIVE_LED_ON		0x04
#define MET_OCP_ATTN_LED_OFF		0x08

/*
 * 0x1F - PCI arbiter enable and PCI bus reset
 */
#define MET_PCI_BUS1_ARB_ENB		0x01
#define MET_PCI_BUS2_ARB_ENB		0x02
#define MET_PCI_BUS3_ARB_ENB		0x04
#define MET_PCI_ARB_ENB_ALL		0x07

#define MET_PCI_BUS1_RESET		0x08
#define MET_PCI_BUS2_RESET		0x10
#define MET_PCI_BUS3_RESET		0x20
#define MET_PCI_ALL_BUS_RESET		0x38
#define MET_PCI_BUS0_BRIGE_RESET	0x40

/*
 * 0x20 - PCI master enable and master change clear
 */
#define MET_PCI_BUS1_MASTER_ENB		0x01
#define MET_PCI_BUS2_MASTER_ENB		0x02
#define MET_PCI_BUS3_MASTER_ENB		0x04
#define MET_PCI_MASTER_ENB_ALL		0x07

#define MET_PCI_CLEAR_MASTER_CHG	0x08

#define MET_OCP_MASTER_CHG_BUS1		0x10
#define MET_OCP_MASTER_CHG_BUS2		0x20
#define MET_OCP_MASTER_CHG_BUS3		0x40
#define MET_OCP_MASTER_CHG_ALL		0x70

/*
 * 0x21 - PCI master status
 */
#define MET_PCI_MASTER_BUS1_IN		0x01
#define MET_PCI_MASTER_BUS2_IN		0x02
#define MET_PCI_MASTER_BUS3_IN		0x04
#define MET_PCI_ALL_MASTER_IN		0x07

#define MET_PCI_MASTER_BUS1_OUT		0x08
#define MET_PCI_MASTER_BUS2_OUT		0x10
#define MET_PCI_MASTER_BUS3_OUT		0x20
#define MET_PCI_ALL_MASTER_OUT		0x38

/*
 * 0x22 - Misc Write CP Control
 */
#define MET_CP_SECTION_ERR_OUT		0x01	/* CP CP-sect Err Out (local) */
#define MET_CP_CORE_ERR_SET		0x02	/* CP Core-sect Err Out(local) */
#define MET_CP_HA_MSTR_OUT		0x04	/* HA arbition set Master Out */
#define MET_CP_PANIC_DUMP_OUT		0x08	/* Start Panic Dump */
#define MET_CP_RESET_OCP		0x10	/* This CP Reset Other CP */
#define MET_CP_STOP_DM_CNTER		0x20	/* Stop Deadman Counter */
#define MET_CP_ENB_MY_RESET		0x40	/* Enable Local CP Reset line */

/*
 * 0x23 - Misc Read CP Control
 */
#define MET_OCP_CORE_ERR_IN		0x02	/* Other CP Core-sect err in */
#define MET_CP_HA_MSTR_IN		0x04	/* HA Master In */
#define MET_CP_PANIC_DUMP_IN		0x08	/* Read Panic Dump assertion */
#define MET_CP_CP_SLOT_ID		0x20	/* Read Slot ID */
#define MET_CP_CP_SLOT_ID_P		0x40	/* Read Odd Parity bit */

/*
 * 0x24 - SW reserve
 */
#define MET_DM_CNTER_DATA		0x01	/* Save DeadManCounter status */
#define MET_BOGUS_PB_ATTN_INTR		0x02	/* Save bogus PB attn intr status */

/*
 * 0x25 - BootPROM Control
 */
#define MET_BOOTPROM_WE			0x01	/* enable writing to bootflash*/
#define COCPU_CP_SLOT_ID		0x08	/* CP Slot ID for the co-CPU */

/*
 * 0x26 - User Flash Control
 * 0x27 - Dead Man Counter TC
 * 0x28 - Backplane Version
 */
#define BKPLANE_VER_REGL_ULYSSES			0x1F
#define BKPLANE_VER_LPBK_METEOR				0x1E
#define BKPLANE_VER_REGL_METEOR				0x1D
#define BKPLANE_VER_REGL_SATURN				0x1C
#define BKPLANE_VER_BKDRL_LPBK_METEOR		0x1B
#define BKPLANE_VER_LPBK_SATURN				0x1A

/*
 * 0x29 - CP miscelaneous error status
 */
#define CP_SLOT_ID_ERR      		0x01
#define CP_DATA_PARITY_ERR		0x02
#define SERIAL_CABLE_DISC		0x04
#define CONSOL_CABLE_DISC		0x08
#define OCP_SECT_ERR      		0x10

/*
 * 0x2A - I2C select
 */
#define MET_CP_I2C_SEL_MASK         	0x1F    /* I2C Select unit - bit 0-4 */
#define MET_CP_I2C_RD_ACTIVE_MASK   	0x20    /* I2C read-active - bit 5 */
#define MET_CP_I2C_WR_ACTIVE_MASK   	0x40    /* I2C write-active - bit 6*/
#define MET_CP_I2C_WRITE_MASK       	0x80    /* Select Write Flag - bit 7*/
#define MET_I2C_SEL_CP0			0x05	/* CP0 in slot 5 */
#define MET_I2C_SEL_CP1			0x06	/* CP1 in slot 6 */
#define MET_CP_I2C_CONTROL_MASK	(MET_CP_I2C_WRITE_MASK | \
				MET_CP_I2C_WR_ACTIVE_MASK | \
				MET_CP_I2C_RD_ACTIVE_MASK)
#define  MET_CPLD_CP0_I2C_ADDR		0x5
#define  MET_CPLD_CP1_I2C_ADDR		0x6

/*
 * 0x2B - Parity Syndrom Low byte
 * 0x2C - Parity Syndrom High byte
 * 0x2D - CP Attention Interrupt Set (Diagnostic)
 * 0x2E - 0x2F - Reserved
 */

/*
 * 0x30 - PDC error status
 */
#define MET_PCD_ERR_MASK       		0x37
#define MET_PCD_DONE   			0x80

/*
 * 0x31 - PDC Select
 */
#define MET_PCD_SEL_MASK		0x0F

/*
 * 0x32 - PDC command
 */
#define MET_PCD_REG_OFFSET_MASK		0x1F
#define MET_PCD_COMMAND_MASK		0xE0
#define MET_PCD_COMMAND_READ		0xA0
#define MET_PCD_COMMAND_WRITE		0x20

/*
 * 0x33 - Port Blade PDC Error
 * 0x34 - Port Blade PDC Error Mask
 */

/*
 * 0x35 PDC core error
 */
#define MET_PDC_CORE0_ERR		0x01
#define MET_PDC_CORE1_ERR		0x02
#define MET_PDC_CORES_ERR_MASK		0x03

/*
 * Port Blade PDC offsets
 */
#define MET_PB_PDC_CP_BP_INTR_CTRL	0x09	/* CP interrupts BP */
#define MET_PB_PDC_CP_BP_INTR_ACK	0x0A	/* CP acks BP interrupt */
#define MET_PB_PDC_BP_CP_INTR_CTRL	0x0B	/* BP interrupts CP */
#define MET_PB_PDC_BP_BOOT_STATUS	0x0C	/* BP boot status */
#define MET_PB_PDC_BP_BOOT_ERROR	0x0D	/* BP error status */
#define MET_PB_PDC_SW_RESET		0x0E
#define MET_PB_PDC_BP_CP_INTR_MASK	0x0F	/* BP to CP interrupt mask */

/*
 * CE2 defines
 */

/*------------------------------------------------------------------------------
 * Pulsar
 *----------------------------------------------------------------------------*/
/* Pulsar (physical) base addresses */
#define PULSAR_CPLD_BASE_ADDR			0x1F8000000ULL	
#define PULSAR_CPLD_IDE_BASE_ADDR		0x1F8F00000ULL	
#define PULSAR_CPLD_SIZE			4096
#define PULSAR_CPLD_IDE_SIZE			1024

/* Register offsets */
#define PULSAR_CPLD_REV_REG			0x00
#define PULSAR_RESET_CAUSE_REG			0x04
#define PULSAR_CPLD_ID_REG			0x08
#define PULSAR_CP_STATUS_REG			0x29
#define PULSAR_PCI_WATCHDOG_CNTRL_REG		0x2A
#define PULSAR_CONDOR_INT_STATUS_REG		0x40
#define PULSAR_PIC_INT_STATUS_REG		0x41
#define PULSAR_SYS_INT_STATUS_REG		0x42
#define PULSAR_SYS_INT_STATUS_B_REG		0x43
#define PULSAR_CONDOR_INT_MASK_REG		0x48
#define PULSAR_PIC_INT_MASK_REG			0x49
#define PULSAR_SYSTEM_INT_MASK_REG		0x4A
#define PULSAR_SYSTEM_INT_B_MASK_REG		0x4B
#define PULSAR_PIC_SB_LVL_MISMATCH_STAT_REG	0x50
#define PULSAR_PIC_SB_LVL_MISMATCH_STAT_B_REG	0x51
#define PULSAR_PIC_SELF_CHK_MISMATCH_STAT_REG	0x56
#define PULSAR_SYS_SELF_CHK_MISMATCH_STAT_REG	0x59
#define PULSAR_CONDOR_RESET_CNTRL_REG		0x60
#define PULSAR_CONDOR_PLL_RESET_CNTRL_REG	0x61
#define PULSAR_CONDOR_RESET_ENABLE_REG		0x64
#define PULSAR_CONDOR_PLL_RESET_ENABLE_REG	0x65
#define PULSAR_SYS_RESET_SHUTDOWN_REG		0x68
#define PULSAR_INDIVIDUAL_RESET_CNTRL_REG	0x69
#define PULSAR_SYS_RESET_ENABLE_REG		0x6C
#define PULSAR_INDIVIDUAL_RESET_ENABLE_REG	0x6D
#define PULSAR_POWER_SUPPLY_PRESENT_REG		0x70
#define PULSAR_POWER_SUPPLY_LED_CNTRL_REG	0x71
#define PULSAR_POWER_SUPPLY_GOOD_REG		0x78
#define PULSAR_FAN_PRESENT_REG			0x80
#define PULSAR_FAN_ENABLE_A_REG			0x81
#define PULSAR_FAN_ENABLE_B_REG			0x82
#define PULSAR_GLBL_CONDOR_RESET_ENABLE_REG	0x88
#define PULSAR_CPLD_CAPABILITY_REG		0x90
#define PULSAR_KERNEL_FLASH_CTRL_REG		0x94
#define PULSAR_SEEPROM_WRITE_ENABLE_REG		0x98
#define PULSAR_SYS_STATUS_LED_CNTRL_REG		0xA0
#define PULSAR_CONDOR_PCI_RESET_CNTRL_REG	0xA4
#define PULSAR_CONDOR_PCI_RESET_ENABLE_REG	0xA8
#define PULSAR_FAN_LED_CNTRL_REG		0xAA
#define PULSAR_PIC_RAW_INT_STATUS_REG		0xC0
#define PULSAR_PIC_CNTRL_REG			0xC8
#define PULSAR_PIC_STATUS_REG			0xCC
#define PULSAR_TEST_MODE_GPIO_REG		0x10D
#define PULSAR_TEST_MODE_STATUS_REG		0x10E
#define PULSAR_SEVEN_SEGMENT_LED_BLANK_REG	0x215
#define PULSAR_SEVEN_SEGMENT_REG		0x216
#define PULSAR_JTAG_CPLD_REVISION_REG		0x217
#define PULSAR_REGISTER_FILE_REG		0xFC0	/* 64 registers */


/* Register bits */

/*
 * 0x00 - Revision Register
 */
#define PULSAR_REV_ATA_CAP			0x80	/* capable of ATA */

/*
 * 0x04 - Reset Cause Register
 */
#define PULSAR_RESET_POWER_UP			0xF0
#define PULSAR_RESET_SW_INIT			0x80
#define PULSAR_RESET_PCI_WATCHDOG		0x40
#define PULSAR_RESET_NONE			0x00

/*
 * 0x29 - CP Status Register
 */
#define PULSAR_SERIAL_1_DISCON			0x08
#define PULSAR_SERIAL_0_DISCON			0x04
#define PULSAR_PARITY_ERROR			0x02

/*
 * 0x2A - PCI Watchdog Control Register
 */
#define PULSAR_WATCHDOG_ENABLE			0x01
	
/*
 * 0x40 - Condor Interrupt Status Register
 */
#define PULSAR_CONDOR_INTERRUPT_STATUS		0x01

/*
 * 0x41 - PIC Interrupt Status Register
 */
#define PULSAR_PIC_SELF_CHECK_MISMATCH_INT	0x40
#define PULSAR_PIC_SIDEBAND_LEVEL_MISMATCH_INT	0x20
#define PULSAR_PIC_ATTENTION_INTERRUPT		0x1C
#define PULSAR_PIC_FAULT_CHANGE			0x02
#define PULSAR_PIC_INSERTION_STATE_CHANGE	0x01

/*
 * 0x42 - System Interrupt Status Register
 */
#define PULSAR_FAN_CNTRL_INTERRUPT		0x80
#define PULSAR_POWER_SUPPLY_GOOD_CHANGE		0x60
#define PULSAR_FAN_FRU_STATE_CHANGE		0x1C
#define PULSAR_POWER_SUPPLY_STATE_CHANGE	0x03

/*
 * 0x43 - System Interrupt Status B Register
 */
#define PULSAR_SYSTEM_SELF_CHECK_MISMATCH_INT	0x20
#define PULSAR_VOLTAGE_MONITOR_INTERRUPT	0x10
#define PULSAR_THERMAL_INTERRUPT		0x08
#define PULSAR_SERIAL_CABLE_CHANGE_INTERRUPT	0x06
#define PULSAR_PCI_TRACE_FPGA_INTERRUPT		0x01

/*
 * 0x48 - Condor Interrupt Mask Register
 */
#define PULSAR_INTERRUPT_MASK			0x01

/*
 * 0x49 - PIC Interrupt Mask Register
 */
#define PULSAR_PIC_SELF_CHECK_MISMATCH_INT_MASK	0x40
#define PULSAR_PIC_SIDEBAND_LEVEL_MISMATCH_MASK	0x20
#define PULSAR_PIC_ATTENTION_INTERRUPT_MASK	0x1C
#define PULSAR_PIC_FAULT_CHANGE_MASK		0x02
#define PULSAR_PIC_INSERTION_STATE_CHANGE_MASK	0x01

/*
 * 0x4A - System Interrupt Mask Register
 */
#define PULSAR_FAN_CNTRL_INTERRUPT_MASK		0x80
#define PULSAR_POWER_SUPPLY_GOOD_CHANGE_MASK	0x60
#define PULSAR_FAN_FRU_STATE_CHANGE_MASK	0x1C
#define PULSAR_POWER_SUPPLY_STATE_CHANGE_MASK	0x03

/*
 * 0x4B - System Interrupt B Mask Register
 */
#define PULSAR_SYSTEM_SELF_CHECK_MISMATCH_MASK	0x20
#define PULSAR_VOLTAGE_MONITOR_INTERRUPT_MASK	0x10
#define PULSAR_THERMAL_INTERRUPT_MASK		0x08
#define PULSAR_SERIAL_CABLE_CHANGE_INT_MASK	0x06
#define PULSAR_PCI_TRACE_FPGA_INTERRUPT_MASK	0x01

/*
 * 0x50 - PIC Sideband Level Mismatch Status Register
 */
#define PULSAR_PIC_INT2_HH_DETECTED		0x20
#define PULSAR_PIC_INT2_LL_DETECTED		0x10
#define PULSAR_PIC_INT1_HH_DETECTED		0x08
#define PULSAR_PIC_INT1_LL_DETECTED		0x04
#define PULSAR_PIC_INT0_HH_DETECTED		0x02
#define PULSAR_PIC_INT0_LL_DETECTED		0x01

/*
 * 0x51 - PIC Sideband Level Mismatch Status B Register
 */
#define PULSAR_PIC_FAULT_HH_DETECTED		0x08
#define PULSAR_PIC_FAULT_LL_DETECTED		0x04
#define PULSAR_PIC_PRESENT_HH_DETECTED		0x02
#define PULSAR_PIC_PRESENT_LL_DETECTED		0x01

/*
 * 0x56 - PIC Self Check Mismatch Status Register
 */
#define PULSAR_PIC_STATUS_LED_SELF_CHK_MISMATCH	0x01

/*
 * 0x59 - System Self Check Mismatch Status Register
 */
#define PULSAR_PS_STATUS_LED_SELF_CHK_MISMATCH	0x18
#define PULSAR_FAN_STATUS_LED_SELF_CHK_MISMATCH	0x07

/*
 * 0x60 - Condor Switch Reset Control Register
 */
#define PULSAR_CONDOR_SWITCH_RESET		0x01

/*
 * 0x61 - Condor PLL Reset Control Register
 */
#define PULSAR_CONDOR_PLL_RESET			0x01

/*
 * 0x64 - Condor Switch Reset Enable Register
 */
#define PULSAR_CONDOR_SWITCH_RESET		0x01

/*
 * 0x65 - Condor PLL Reset Enable Register
 */
#define PULSAR_CONDOR_PLL_RESET_ENABLE		0x01

/*
 * 0x68 - System Reset and Shutdown Control Register
 */
#define PULSAR_SYSTEM_SHUTDOWN			0x02
#define PULSAR_SYSTEM_RESET			0x01

/*
 * 0x69 - Individual Reset Control Register
 */
#define PULSAR_IIC_MUX_RESET			0x80
#define PULSAR_IIC_RESET			0x40
#define PULSAR_BOOT_FLASH_RESET			0x20
#define PULSAR_KERNEL_FLASH_RESET		0x10
#define PULSAR_ENET_PHY1_RESET			0x08
#define PULSAR_ENET_PHY2_RESET			0x04
#define PULSAR_SECURITY_PROC_PCI_RESET		0x02
#define PULSAR_CF_RESET				0x01

/*
 * 0x6C - System Reset Enable Register
 */
#define PULSAR_SYSTEM_SHUTDOWN_ENABLE		0x02
#define PULSAR_SYSTEM_RESET_ENABLE		0x01

/*
 * 0x6D - Individual Reset Enable Register
 */
#define PULSAR_IIC_MUX_RESET_ENABLE		0x80
#define PULSAR_IIC_RESET_ENABLE			0x40
#define PULSAR_BOOT_FLASH_RESET_ENABLE		0x20
#define PULSAR_KERNEL_FLASH_RESET_ENABLE	0x10
#define PULSAR_RESET_I2C_PWR_ENABLE		0x08
#define PULSAR_ENET_PHY2_RESET_ENABLE		0x04
#define PULSAR_SECURITY_PROC_PCI_RESET_ENABLE	0x02
#define PULSAR_CF_RESET_ENABLE			0x01

/*
 * 0x70 - Power Supply Present Register
 */
#define PULSAR_POWER_SUPPLY1_PRESENT		0x02
#define PULSAR_POWER_SUPPLY0_PRESENT		0x01

/*
 * 0x71 - Power Supply LED Control Register
 */
#define PULSAR_POWER_SUPPLY1_LED_BLINK		0x20
#define PULSAR_POWER_SUPPLY0_LED_BLINK		0x10
#define PULSAR_POWER_SUPPLY1_LED_CNTRL		0x02
#define PULSAR_POWER_SUPPLY0_LED_CNTRL		0x01

/*
 * 0x78 - Power Supply Good Register
 */
#define PULSAR_POWER_SUPPLY1_GOOD		0x02
#define PULSAR_POWER_SUPPLY0_GOOD		0x01

/*
 * 0x80 - Fan Present Register
 */
#define PULSAR_FAN_FRU2_PRESENT			0x04
#define PULSAR_FAN_FRU1_PRESENT			0x02
#define PULSAR_FAN_FRU0_PRESENT			0x01

/*
 * 0x81 - Fan Enable A Register
 */
#define PULSAR_FAN_ENABLE_7			0x80
#define PULSAR_FAN_ENABLE_6			0x40
#define PULSAR_FAN_ENABLE_5			0x20
#define PULSAR_FAN_ENABLE_4			0x10
#define PULSAR_FAN_ENABLE_3			0x08
#define PULSAR_FAN_ENABLE_2			0x04
#define PULSAR_FAN_ENABLE_1			0x02
#define PULSAR_FAN_ENABLE_0			0x01

/*
 * 0x82 - Fan Enable B Register
 */
#define PULSAR_FAN_ENABLE_11			0x08
#define PULSAR_FAN_ENABLE_10			0x04
#define PULSAR_FAN_ENABLE_9			0x02
#define PULSAR_FAN_ENABLE_8			0x01

/*
 * 0x88 - Global Condor Reset Enable Register
 */
#define PULSAR_CONDOR_RESET_ENABLE		0x01

/*
 * 0x90 - CPLD Capability Register
 */
#define PULSAR_PERIPHERAL_BUS_BASED_IDE_PRESENT	0x02
#define PULSAR_BOOTPROM_WE			0x01

/*
 * 0x94 - Kernel Flash Write Enable Register
 */
#define PULSAR_KERNEL_FLASH_WE			0x01

/*
 * 0x98 - SEEPROM Write Enable Register
 */
#define PULSAR_IIC_SEEPROM_WRITE_ENABLE		0x01

/*
 * 0xA0 - System Status LED Control Register
 */
#define PULSAR_AMBER_LED_BLINK_CNTRL		0x20
#define PULSAR_AMBER_LED_CNTRL			0x02

/*
 * 0xA4 - Condor PCI Reset Control Register
 */
#define PULSAR_CONDOR_PCI_RESET_CNTRL		0x01

/*
 * 0xA8 - Condor PCI Reset Enable Register
 */
#define PULSAR_CONDOR_PCI_RESET_ENABLE		0x01

/*
 * 0xAA - Fan LED Control Register
 */
#define PULSAR_FAN2_LED_BLINK_CNTRL		0x40
#define PULSAR_FAN1_LED_BLINK_CNTRL		0x20
#define PULSAR_FAN0_LED_BLINK_CNTRL		0x10
#define PULSAR_FAN2_LED_CNTRL			0x04
#define PULSAR_FAN1_LED_CNTRL			0x02
#define PULSAR_FAN0_LED_CNTRL			0x01

/*
 * 0xC0 - PIC Raw Interrupt Status Register
 */
#define PULSAR_PIC_INT2_H_STATUS		0x20
#define PULSAR_PIC_INT2_L_STATUS		0x10
#define PULSAR_PIC_INT1_H_STATUS		0x08
#define PULSAR_PIC_INT1_L_STATUS		0x04
#define PULSAR_PIC_INT0_H_STATUS		0x02
#define PULSAR_PIC_INT0_L_STATUS		0x01

/*
 * 0xC8 - PIC Control Register
 */
#define PULSAR_PIC_STATUS_LED_BLINK_CNTRL	0x80
#define PULSAR_PIC_PCI_RESET			0x40
#define PULSAR_PIC_FENCE_CNTRL			0x10
#define PULSAR_PIC_STATUS_LED_CNTRL		0x08
#define PULSAR_PIC_MUX_IIC_RESET		0x04
#define PULSAR_PIC_IIC_RESET			0x02
#define PULSAR_PIC_EEPROM_WRITE_ENABLE		0x01

/*
 * 0xCC - PIC Status Register
 */
#define PULSAR_PIC_FAULT_H_STATE		0x20
#define PULSAR_PIC_FAULT_L_STATE		0x10
#define PULSAR_PIC_PRESENT_H_STATE		0x08
#define PULSAR_PIC_PRESENT_L_STATE		0x04
#define PULSAR_PIC_FAULT_STATE			0x02
#define PULSAR_PIC_PRESENT_STATE		0x01

/*
 * 0x10D - Test Mode GPIO Register
 */
#define PULSAR_TEST_MODE_IO_DIR_CNTRL		0xF0
#define PULSAR_TEST_MODE_IO_VAL_CTRL		0x0F

/*
 * 0x10E - Test Mode Status Register
 */
#define PULSAR_TEST_MODE_9			0x10
#define PULSAR_TEST_MODE_GPIO			0x0F

/*
 * 0x215 - Seven Segment LED Blank Register
 */
#define PULSAR_SEVEN_SEGMENT_BLANK_ENABLE	0x01

/*
 * 0x216 - Seven Segment LED Register
 */
#define PULSAR_BOOT_STATUS_LED			0x01

/*
 * 0x217 -  JTAG CPLD Revision Register
 */
#define PULSAR_JTAG_MUX_CPLD_REV		0xE0

/*
 * 0xF00 - Register File
 */
#define PULSAR_REGISTER_FILE_SIZE		0x40

/*------------------------------------------------------------------------------
 * Pulsar2
 *----------------------------------------------------------------------------*/
/* Register offsets */
#define	PULSAR2_VOLT_MARGIN_REG			0x89
#define PULSAR2_SOFTWARE_RESTART_REG		0xAB

/*------------------------------------------------------------------------------
 * Viking
 *----------------------------------------------------------------------------*/
/* Viking (physical base addresses */
#define VIKING_CPLD_BASE_ADDR			0x1F8000000ULL
#define VIKING_CPLD_IDE_BASE_ADDR		0x1F8F001F0ULL
#define VIKING_CPLD_SIZE				8192
#define VIKING_CPLD_IDE_SIZE			8192

/* Register offsets */
#define VIKING_CPLD_MB_REV_REG				0x00
#define VIKING_CPLD_DC_REV_REG				0x100
#define VIKING_RESET_CAUSE_REG				0x04
#define VIKING_CPLD_ID_REG					0x08
#define VIKING_CP_STATUS_REG				0x29
#if 0
#define VIKING_PCI_WATCHDOG_CNTRL_REG		0x2A
#endif
#define VIKING_CONDOR_INT_STATUS_REG		0x40
#define VIKING_SYS_INT_STATUS_REG			0x42
#define VIKING_SYS_INT_STATUS_B_REG			0x43
#define VIKING_CONDOR_INT_MASK_REG			0x48
#define VIKING_SYSTEM_INT_MASK_REG			0x4A
#define VIKING_SYSTEM_INT_B_MASK_REG		0x4B
#define VIKING_MB_THERMO_INT_STATUS_REG		0x4C
#define VIKING_DC_THERMO_INT_STATUS_REG		0x14C
#define VIKING_CONDOR_RESET_CNTRL_REG		0x60
#define VIKING_CONDOR_PLL_RESET_CNTRL_REG	0x61
#define VIKING_CONDOR_RESET_ENABLE_REG		0x64
#define VIKING_CONDOR_PLL_RESET_ENABLE_REG 	0x65
#define VIKING_SYS_RESET_SHUTDOWN_REG		0x68
#define VIKING_INDIVIDUAL_RESET_CNTRL_REG	0x69
#define VIKING_SYS_RESET_ENABLE_REG			0x6C
#define VIKING_INDIVIDUAL_RESET_ENABLE_REG	0x6D
#define VIKING_POWER_SUPPLY_PRESENT_REG 	0x70
#define VIKING_POWER_SUPPLY_LED_CNTRL_REG	0x71
#define VIKING_POWER_SUPPLY_GOOD_REG		0x78
#define VIKING_FAN_PRESENT_REG				0x80
#define VIKING_GLBL_CONDOR_RESET_ENABLE_REG	0x88
#define VIKING_CAPABILITY_REG				0x90
#define VIKING_SEEPROM_WRITE_ENABLE_REG		0x98
#define VIKING_SYS_STATUS_LED_CNTRL_REG		0xA0
#define VIKING_CONDOR_PCI_RESET_CNTRL_REG	0xA4
#define VIKING_CONDOR_PCI_RESET_ENABLE_REG	0xA8
#define VIKING_FAN_LED_CNTRL_REG			0xAA
#define VIKING_SW_RESTARTS_REG				0xAB
#define VIKING_DEBUG_REG					0xFC


/*
 * 0x04 - Reset Cause Register
 */
#define VIKING_RESET_POWER_UP				0xF0
#define VIKING_RESET_SW_INIT				0x80
#if 0
#define VIKING_RESET_PCI_WATCHDOG			0x40
#endif
#define VIKING_RESET_NONE					0x00

/*
 * 0x29 - CP Status Register
 */
#define VIKING_SERIAL_DISCON				0x08

#if 0
/*
 * 0x2A - PCI Watchdog Control Register
 */
#define VIKING_WATCHDOG_ENABLE      	    0x01
#endif

/*
 * 0x40 - Condor Interrupt Status Register
 */
#define VIKING_CONDOR_INTERRUPT_STATUS      0x01

/*
 * 0x42 - System Interrupt Status Register
 */
#define VIKING_FAN_CNTRL_INTERRUPT				0x80
#define VIKING_POWER_SUPPLY_GOOD_CHANGE			0x60
#define VIKING_FAN_FRU_STATE_CHANGE				0x1C
#define VIKING_POWER_SUPPLY_STATE_CHANGE		0x03

/*
 * 0x43 - System Interrupt Status B Register
 */
#define VIKING_VOLTAGE_MONITOR_INTERRUPT_DC		0x40
#define VIKING_THERMAL_INTERRUPT_DC				0x20
#define VIKING_VOLTAGE_MONITOR_INTERRUPT_MB		0x10
#define VIKING_THERMAL_INTERRUPT_MB				0x08
#define VIKING_SERIAL_CABLE_CHANGE_INTERRUPT    0x02

/*
 * 0x48 - Condor Interrupt Mask Register
 * (Not sure if we need separate defines for each
 * condor.
 */
#define VIKING_INTERRUPT_MASK				   0x2F

/*
 * 0x4A - System Interrupt Mask Register
 */
#define VIKING_FAN_CNTRL_INTERRUPT_MASK			0x80
#define VIKING_POWER_SUPPLY_GOOD_CHANGE_MASK	0x60
#define VIKING_FAN_FRU_STATE_CHANGE_MASK		0x1C
#define VIKING_POWER_SUPPLY_STATE_CHANGE_MASK	0x03

/*
 * 0x4B - System Interrupt B Mask Register
 */
#define VIKING_VOLTAGE_MONITOR_INTERRUPT_MASK_DC	0x40
#define VIKING_THERMAL_INTERRUPT_MASK_DC	       	0x20
#define VIKING_VOLTAGE_MONITOR_INTERRUPT_MASK_MB	0x10
#define VIKING_THERMAL_INTERRUPT_MASK_MB		    0x04

/*
 * 0x68 - System Reset and Shutdown Control Register
 */
#define VIKING_SYSTEM_SHUTDOWN			0x02
#define VIKING_SYSTEM_RESET				0x01

/*
 * 0x69 - Individual Reset Control Register
 */
#define VIKING_IIC_MUX_RESET			0x80
#define VIKING_IIC_RESET				0x40
#define VIKING_ENET_PHY1_RESET          0x04
#define VIKING_ENET_PHY2_RESET          0x02
#define VIKING_SECURITY_PROC_PCI_RESET 	0x02
#define VIKING_CF_RESET					0x01

/*
 * 0x6C - System Reset Enable Register
 */
#define VIKING_SYSTEM_SHUTDOWN_ENABLE	0x02
#define VIKING_SYSTEM_RESET_ENABLE		0x01

/*
 * 0x6D - Individual Reset Enable Register
 */
#define VIKING_IIC_MUX_RESET_ENABLE				0x80
#define VIKING_IIC_RESET_ENABLE					0x40
#define VIKING_BOOT_FLASH_RESET_ENABLE			0x20
#define VIKING_KERNEL_FLASH_RESET_ENABLE		0x10
#define VIKING_RESET_I2C_PWR_ENABLE				0x08
#define VIKING_ENET_PHY1_RESET_ENABLE			0x04
#define VIKING_ENET_PHY2_RESET_ENABLE			0x02


/*
 * 0x98 - SEEPROM Write Enable Register
 */
#define VIKING_IIC_SEEPROM_WRITE_ENABLE_DC		0x04
#define VIKING_IIC_CONFIG_SEEPROM_WRITE_ENABLE	0x02
#define VIKING_IIC_SEEPROM_WRITE_ENABLE_MB		0x01

/*------------------------------------------------------------------------------
 * Stealth
 *----------------------------------------------------------------------------*/
/* Stealth (physical) base addresses */
#define STEALTH_CPLD_BASE_ADDR			0x1F8000000ULL	
#define STEALTH_CPLD_IDE_BASE_ADDR		0x1F8F00000ULL	
#define STEALTH_CPLD_SIZE			4096
#define STEALTH_CPLD_IDE_SIZE			1024

/* Register offsets */
#define STEALTH_CPLD_REV_REG			0x00
#define STEALTH_RESET_CAUSE_REG			0x04
#define STEALTH_CPLD_ID_REG			0x08
#define STEALTH_CP_STATUS_REG			0x29
#define STEALTH_GEYE_INT_STATUS_REG		0x40
#define STEALTH_SYS_INT_STATUS_REG		0x42
#define STEALTH_SYS_INT_STATUS_B_REG		0x43
#define STEALTH_GEYE_INT_MASK_REG		0x48
#define STEALTH_SYSTEM_INT_MASK_REG		0x4A
#define STEALTH_SYSTEM_INT_B_MASK_REG		0x4B
#define STEALTH_GEYE_RESET_CNTRL_REG		0x60
#define STEALTH_GEYE_PLL_RESET_CNTRL_REG	0x61
#define STEALTH_GEYE_RESET_ENABLE_REG		0x64
#define STEALTH_GEYE_PLL_RESET_ENABLE_REG	0x65
#define STEALTH_SYS_RESET_SHUTDOWN_REG		0x68
#define STEALTH_INDIVIDUAL_RESET_CNTRL_REG	0x69
#define STEALTH_SYS_RESET_ENABLE_REG		0x6C
#define STEALTH_INDIVIDUAL_RESET_ENABLE_REG	0x6D
#define STEALTH_POWER_SUPPLY_GOOD_REG		0x78
#define STEALTH_GLBL_GEYE_RESET_ENABLE_REG	0x88
#define STEALTH_VOLTAGE_MARGIN_REG		0x89
#define STEALTH_CPLD_CAPABILITY_REG		0x90
#define STEALTH_KERNEL_FLASH_CTRL_REG		0x94
#define STEALTH_SEEPROM_WRITE_ENABLE_REG	0x98
#define STEALTH_SYS_STATUS_LED_CNTRL_REG	0xA0
#define STEALTH_GEYE_PCI_RESET_CNTRL_REG	0xA4
#define STEALTH_GEYE_PCI_RESET_ENABLE_REG	0xA8
#define STEALTH_RESET_STATUS_REG		0xAB
#define STEALTH_FAN_PWM_REG			0xAC

/* Register bits */

/*
 * 0x00 - Revision Register
 */

/*
 * 0x04 - Reset Cause Register
 */
#define STEALTH_RESET_POWER_UP			0xF0
#define STEALTH_RESET_SW_INIT			0x80
#define STEALTH_RESET_PCI_WATCHDOG		0x40
#define STEALTH_RESET_NONE			0x00

/*
 * 0x29 - CP Status Register
 */
#define STEALTH_SERIAL_0_DISCON			0x04

/*
 * 0x40 - Condor Interrupt Status Register
 */
#define STEALTH_GEYE_INTERRUPT_STATUS		0x01

/*
 * 0x42 - System Interrupt Status Register
 */
#define STEALTH_POWER_SUPPLY_GOOD_CHANGE	0x60

/*
 * 0x43 - System Interrupt Status B Register
 */
#define STEALTH_VOLTAGE_MONITOR_INTERRUPT	0x10
#define STEALTH_THERMAL_INTERRUPT		0x08
#define STEALTH_SERIAL_CABLE_CHANGE_INTERRUPT	0x02

/*
 * 0x48 - Condor Interrupt Mask Register
 */
#define STEALTH_INTERRUPT_MASK			0x01

/*
 * 0x4A - System Interrupt Mask Register
 */
#define STEALTH_POWER_SUPPLY_GOOD_CHANGE_MASK	0x60

/*
 * 0x4B - System Interrupt B Mask Register
 */
#define STEALTH_VOLTAGE_MONITOR_INTERRUPT_MASK	0x10
#define STEALTH_THERMAL_INTERRUPT_MASK		0x08
#define STEALTH_SERIAL_CABLE_CHANGE_INT_MASK	0x02

/*
 * 0x60 - Condor Switch Reset Control Register
 */
#define STEALTH_CONDOR_SWITCH_RESET		0x01

/*
 * 0x61 - Condor PLL Reset Control Register
 */
#define STEALTH_CONDOR_PLL_RESET		0x01

/*
 * 0x64 - Condor Switch Reset Enable Register
 */
#define STEALTH_CONDOR_SWITCH_RESET		0x01

/*
 * 0x65 - Condor PLL Reset Enable Register
 */
#define STEALTH_CONDOR_PLL_RESET_ENABLE		0x01

/*
 * 0x68 - System Reset and Shutdown Control Register
 */
#define STEALTH_SYSTEM_SHUTDOWN			0x02
#define STEALTH_SYSTEM_RESET			0x01

#define DRAGON_SYSTEM_RESET			0x01
#define DRAGON_SYSTEM_SHUTDOWN			0x08
#define SUPERHAWK_SYSTEM_RESET			0x01


/*
 * 0x69 - Individual Reset Control Register
 */
#define STEALTH_PWR_MON_RESET			0x08
#define STEALTH_ENET_PHY2_RESET			0x04
#define STEALTH_SECURITY_PROC_PCI_RESET		0x02
#define STEALTH_CF_RESET			0x01

#define STEALTH_DIS_I2C_PWR_RESET		0x02

/*
 * 0x6C - System Reset Enable Register
 */
#define STEALTH_SYSTEM_SHUTDOWN_ENABLE		0x02
#define STEALTH_SYSTEM_RESET_ENABLE		0x01

#define DRAGON_SYSTEM_RESET_ENABLE		0x01
#define SUPERHAWK_SYSTEM_RESET_ENABLE		0x01

/*
 * 0x6D - Individual Reset Enable Register
 */
#define STEALTH_ENET_PHY2_RESET_ENABLE		0x04
#define STEALTH_DIS_I2C_PWR_RESET_ENABLE	0x02
#define STEALTH_CF_RESET_ENABLE			0x01

#define GLACIER2_CF_ETH_RESET_ENABLE		0x05
#define GLACIER2_ENET_PHY2_RESET_ENABLE		0x04
#define GLACIER2_CF_RESET_ENABLE		0x01

#define DRAGON_ENET_PHY2_RESET_ENABLE		0x04
#define DRAGON_CF_RESET_ENABLE			0x01

#define SUPERHAWK_CF_ETH_RESET_ENABLE		0x05
#define SUPERHAWK_ENET_PHY2_RESET_ENABLE	0x04
#define SUPERHAWK_CF_RESET_ENABLE		0x01

/*
 * 0x78 - Power Supply Good Register
 */
#define STEALTH_FAN_3_BAD			0x020
#define STEALTH_FAN_2_BAD			0x010
#define STEALTH_FAN_1_BAD			0x08
#define STEALTH_FAN_0_BAD			0x04
#define STEALTH_POWER_SUPPLY1_GOOD		0x02
#define STEALTH_POWER_SUPPLY0_GOOD		0x01

/*
 * 0x88 - Global Condor Reset Enable Register
 */
#define STEALTH_CONDOR_RESET_ENABLE		0x01

/*
 * 0x89 - Voltage margin control
 */
#define STEALTH_3_3V_UP				0x080
#define STEALTH_3_3V_DOWN			0x040
#define STEALTH_2_5V_UP				0x020
#define STEALTH_2_5V_DOWN			0x010
#define STEALTH_1_8V_UP				0x08
#define STEALTH_1_8V_DOWN			0x04
#define STEALTH_1_2V_UP				0x02
#define STEALTH_1_2V_DOWN			0x01

/*
 * 0x90 - CPLD Capability Register
 */
#define STEALTH_PERIPHERAL_BUS_BASED_IDE_PRESENT	0x02
#define STEALTH_BOOTPROM_WE			0x01

/*
 * 0x94 - Kernel Flash Write Enable Register
 */
#define STEALTH_KERNEL_FLASH_WE			0x01

/*
 * 0x98 - SEEPROM Write Enable Register
 */
#define STEALTH_IIC_SEEPROM_WRITE_ENABLE	0x01
#define DRAGON_IIC_SEEPROM_WRITE_ENABLE		0x01
#define SUPERHAWK_IIC_SEEPROM_WRITE_ENABLE	0x01

/*
 * 0xA0 - System Status LED Control Register
 */
#define STEALTH_AMBER_LED_BLINK_CNTRL		0x20
#define STEALTH_AMBER_LED_CNTRL			0x02

/*
 * 0xA4 - Condor PCI Reset Control Register
 */
#define STEALTH_CONDOR_PCI_RESET_CNTRL		0x01

/*
 * 0xA8 - Condor PCI Reset Enable Register
 */
#define STEALTH_CONDOR_PCI_RESET_ENABLE		0x01

/*
 * 0xAB - Restart Status Register
 */
/* value based register */

/*
 * 0xAC - Fan Speed Control Register
 */
#define STEALTH_FAN_NORMAL_SPEED_CNTRL		0x01

/*------------------------------------------------------------------------------
 * Athena
 *----------------------------------------------------------------------------*/

#define ATHENA_CPLD_BASE_ADDR                   0x00000001F8000000ULL
#define ATHENA_CPLD_SIZE                        4096
#define ATHENA_MAX_SPI_DATA_LEN			10	/* 10 bytes per transaction */

/* Semaphore defines */
#define ATHENA_SEM_BASE				0x00000001F8800000ULL
#define ATHENA_SEM_TOTAL_SIZE			0x600
#define ATHENA_SEM_BLOCK_SIZE			0x100

#define ATHENA_SEM_CTRL_STAT_REG		0x0000
#define ATHENA_SEM_INT_STAT1_REG		0x0001
#define ATHENA_SEM_INT_STAT2_REG		0x0002
#define ATHENA_SEM_INT_MASK1_REG		0x0003
#define ATHENA_SEM_INT_MASK2_REG		0x0004
#define ATHENA_SEM_ERRINJ_EN_REG		0x0005
#define ATHENA_SEM_ERRINJ_REG			0x0006
#define ATHENA_SEM_CMD_ERRINJ_REG		0x0007

#define ATHENA_SEM_RESERVE			0x0100
#define ATHENA_SEM_INT_RESERVE			0x0200
#define ATHENA_SEM_RELEASE			0x0300
#define ATHENA_SEM_DIRECT_HYBRID		0x0400
#define ATHENA_SEM_DIRECT_LOCAL			0x0500
#define ATHENA_SEM_CNTRL_W_ENB			0x10

/* Semaphore Ctrl/status reg */
#define ATHENA_SEM_ENABLED			0x01
#define ATHENA_SEM_WRITE_EN			0x10
#define ATHENA_SEM_MASTER_STAT			0x40
#define ATHENA_SEM_FORCE_MASTER		0x80

/* Semaphore Interrupt Status registers */
#define ATHENA_SEM_FREE_INTR			0x01
#define ATHENA_SEM_INTR_TIMEOUT		0x02
#define ATHENA_SEM_VALID_TIMEOUT		0x04
#define ATHENA_SEM_OUTBOUND_ERR		0x08
#define ATHENA_SEM_INTERNAL_ERR		0x10
#define ATHENA_SEM_PROTO_ERR			0x20
#define ATHENA_SEM_PARITY_ERR			0x40
#define ATHENA_SEM_STAT2			0x80

#define ATHENA_SEM_VALID_LL			0x01
#define ATHENA_SEM_VALID_HH			0x02
#define ATHENA_SEM_VALID_STUCK			0x04
#define ATHENA_SEM_INTR_LL			0x08
#define ATHENA_SEM_INTR_HH			0x10
#define ATHENA_SEM_INTR_STUCK			0x20


/* Register offsets */
#define ATHENA_CPLD_ID_REG				0x0008
#define ATHENA_PB_STATE_CHANGE_RESET_REG		0x02A0
#define ATHENA_PB_FORCE_STATE_CHANGE_INT_REG		0x02A1
#define ATHENA_PB_STATE_CHANGE_MASK_REG			0x02A2
#define ATHENA_PB_ATTENTION_REG				0x02A3
#define ATHENA_PB_ATTENTION_MASK_REG			0x02A4
#define ATHENA_PB_FENCE_CNTRL_REG			0x02A5
#define ATHENA_PB_STATUS_LED_CNTRL_REG			0x02A6
#define ATHENA_PB_STATE_REG				0x02A7
#define ATHENA_PB_FORCE_PRESENT_REG			0x02A8
#define ATHENA_PB_FAULT_RESET_REG			0x02A9
#define ATHENA_PB_FAULT_MASK_REG			0x02AA
#define ATHENA_PB_LED_BLINK_CNTRL_REG			0x02AB
#define ATHENA_PB_FAULT_STATUS_REG			0x02AC
#define ATHENA_CB_STATE_CHANGE_RESET_REG		0x02B0
#define ATHENA_CB_FORCE_STATE_CHANGE_INT_REG		0x02B1
#define ATHENA_CB_STATE_CHANGE_MASK_REG			0x02B2
#define ATHENA_CB_ATTENTION_REG				0x02B3
#define ATHENA_CB_ATTENTION_MASK_REG			0x02B4
#define ATHENA_CB_FENCE_CNTRL_REG			0x02B5
#define ATHENA_CB_FORCE_ERROR_REG			0x02B6
#define ATHENA_CB_BLADE_STATE_REG			0x02B7
#define ATHENA_CB_FORCE_PRESENT_REG			0x02B8
#define ATHENA_CB_FAULT_RESET_REG			0x02B9
#define ATHENA_CB_FAULT_MASK_REG			0x02BA
#define ATHENA_CB_LED_BLINK_CNTRL_REG			0x02BB
#define ATHENA_CB_FAULT_STATUS_REG			0x02BC
#define ATHENA_PCI_MASTERS_RESET_REG			0x016E
#define ATHENA_PCI_BRIDGE_RESET_STATUS_REG		0x0101
#define ATHENA_CP_INT_RESET_REG				0x0130
#define ATHENA_CP_INT_MASK_REG				0x0131
#define ATHENA_LCP_ATTENTION_INT_REASON_REG		0x0132
#define ATHENA_LCP_ATTENTION_INT_MASK_REG		0x0133
#define ATHENA_LCP_MISC_INT_REASON_REG			0x0134
#define ATHENA_LCP_MISC_INT_MASK_REG			0x0135
#define ATHENA_OCP_ATTENTION_INT_REASON_REG		0x0136
#define ATHENA_OCP_ATTENTION_INT_MASK_REG		0x0137
#define ATHENA_LCP_FAULT_INT_REASON_REG			0x0138
#define ATHENA_LCP_FAULT_INT_MASK_REG			0x0139
#define ATHENA_RAS_SC_INT_REASON_REG			0x0220
#define ATHENA_RAS_SC_INT_MASK_REG			0x0221
#define ATHENA_OCP_FAULT_INT_CAUSE_REG			0x013A
#define ATHENA_OCP_FAULT_INT_MASK_REG			0x013B
#define ATHENA_OCP_MISC_STATUS_REG			0x013C
#define ATHENA_SERIAL_CABLE_STATE_REG			0x0123
#define ATHENA_INTER_CP_LL_MISMATCH_INT_REG		0x013D
#define ATHENA_INTER_CP_HH_MISMATCH_INT_REG		0x013E
#define ATHENA_INTER_CP_OUT_MISMATCH_INT_RESET_REG	0x0151
#define ATHENA_FAN_INT_REG				0x0230
#define ATHENA_FAN_INT_MASK_REG				0x0231
#define ATHENA_FAN0_INT_CAUSE_STATUS_REG		0x0232
#define ATHENA_FAN1_INT_CAUSE_STATUS_REG		0x0233
#define ATHENA_FAN2_INT_CAUSE_STATUS_REG		0x0234
#define ATHENA_FAN_FORCE_INT_REG			0x0235
#define ATHENA_FAN_FORCE_VALUE_REG			0x0236
#define ATHENA_FAN0_STATE_FAULT_CHANGE_MASK_REG		0x0237
#define ATHENA_FAN1_STATE_FAULT_CHANGE_MASK_REG		0x0238
#define ATHENA_FAN2_STATE_FAULT_CHANGE_MASK_REG		0x0239
#define ATHENA_FAN_STATUS_LED_CNTRL_REG			0x023A
#define ATHENA_FAN_STATUS_LED_BLINK_REG			0x023B
#define ATHENA_PS_INT_CAUSE_REG				0x0240
#define ATHENA_PS_INT_MASK_REG				0x0241
#define ATHENA_PS_FORCE_INT_REG				0x0242
#define ATHENA_PS_FORCE_STATUS_VAL_REG			0x0243
#define ATHENA_PS_STATUS_LED_CNTRL_REG			0x0244
#define ATHENA_PS_STATUS_LED_BLINK_CNTRL_REG		0x0245
#define ATHENA_PS1_INT_CAUSE_STATUS_REG			0x0248
#define ATHENA_PS2_INT_CAUSE_STATUS_REG			0x0249
#define ATHENA_PS3_INT_CAUSE_STATUS_REG			0x024A
#define ATHENA_PS4_INT_CAUSE_STATUS_REG			0x024B
#define ATHENA_PS1_STATUS_REG				0x024C
#define ATHENA_PS2_STATUS_REG				0x024D
#define ATHENA_PS3_STATUS_REG				0x024E
#define ATHENA_PS4_STATUS_REG				0x024F
#define ATHENA_PS1_INT_CAUSE_MASK_REG			0x0250
#define ATHENA_PS2_INT_CAUSE_MASK_REG			0x0251
#define ATHENA_PS3_INT_CAUSE_MASK_REG			0x0252
#define ATHENA_PS4_INT_CAUSE_MASK_REG			0x0253
#define ATHENA_PROC_BLADE_STATE_CHANGE_FORCE_REG	0x0161
#define ATHENA_PROC_BLADE_FENCE_CNTRL_STATUS_REG	0x0165
#define ATHENA_PROC_BLADE_STATUS_LED_CNTRL_REG		0x0166
#define ATHENA_PROC_BLADE_STATE_REG			0x0167
#define ATHENA_PROC_BLADE_STATUS_LED_BLINK_CNTRL_REG	0x016B
#define ATHENA_PROC_FENCE_STATUS_REG			0x016F
#define ATHENA_OCP_RESET_CNTRL_LCP_ERR_FC_CLK_REG	0x0018
#define ATHENA_BLADE_IGNORE_OCP_FENCE_CNTRL_REG		0x0150
#define ATHENA_LCP_SLOT_ID_SEATED_REG			0x0019
#define ATHENA_IIC_SELECT_REG				0x001A
#define ATHENA_IIC_RESET_CTRL_REG			0x010F
#define ATHENA_BOOTROM_CTRL_REG				0x001B
#define ATHENA_USER_FLASH_CTRL_REG			0x001C
#define ATHENA_RESET_CTRL_REG				0x0102
#define ATHENA_ETH_RESET_CTRL_REG			0x0103
#define ATHENA_SS_STATUS_REG				0x013F
#define ATHENA_OCP_STATUS_REG				0x021A
#define ATHENA_RESET_CAUSE_REG				0x0100
#define ATHENA_JTAG_PLD_VERSION_REG			0x0217
#define ATHENA_SEVEN_SEGMET_REG				0x0216
#define ATHENA_SEVEN_SEGMET_LED_BLANK_REG		0x0215
#define ATHENA_CLK_MOD_1_STATUS_REG			0x010B
#define ATHENA_CLK_MOD_2_STATUS_REG			0x010C
#define ATHENA_TEST_MODE_GPIO_CNTRL_REG			0x010D
#define ATHENA_PLD0_TEST_MODE_STATUS_REG		0x010E
#define ATHENA_CP_PLD0_VERSION_STRING_REG		0x001F
#define ATHENA_PCI_RAS_PLD_REPROG_REG			0x0218
#define ATHENA_PCI_TRACE_BUF_TRIGGER_REG		0x0106
#define ATHENA_PLD1_TEST_MODE_STATUS_REG		0x0214
#define ATHENA_CP_PLD1_VERSION_STRING_REG		0x021F
#define ATHENA_BLADE_FAN_PS_INT_FORCE_ENABLE_REG	0x0219
#define ATHENA_OCP_RESET_KEY_REG			0x0120
#define ATHENA_HK_FORCE_FENCE_KEY_REG			0x0121
#define ATHENA_PCI_FORCE_FENCE_KEY_REG			0x0122
#define ATHENA_RAS_PC_SC_MISMATCH_BLADE_REG		0x0260
#define ATHENA_RAS_CB_SC_MISMATCH_BLADE_REG		0x0261
#define ATHENA_RAS_FAN_SC_MISMATCH_REG			0x0262
#define ATHENA_RAS_PS_SC_MISMATCH_REG			0x0263
#define ATHENA_RAS_PC_SC_MISMATCH_BLADE_MASK_REG 	0x0264
#define ATHENA_RAS_CB_SC_MISMATCH_BLADE_MASK_REG 	0x0265
#define ATHENA_RAS_FAN_SC_MISMATCH_MASK_REG		0x0266
#define ATHENA_RAS_PS_SC_MISMATCH_MASK_REG		0x0267
#define ATHENA_PC0_SC_MISMATCH_CAUSE_REG		0x0270
#define ATHENA_PC1_SC_MISMATCH_CAUSE_REG		0x0271
#define ATHENA_PC2_SC_MISMATCH_CAUSE_REG		0x0272
#define ATHENA_PC3_SC_MISMATCH_CAUSE_REG		0x0273
#define ATHENA_PC4_SC_MISMATCH_CAUSE_REG		0x0274
#define ATHENA_PC5_SC_MISMATCH_CAUSE_REG		0x0275
#define ATHENA_PC6_SC_MISMATCH_CAUSE_REG		0x0276
#define ATHENA_PC7_SC_MISMATCH_CAUSE_REG		0x0277
#define ATHENA_CB0_SC_MISMATCH_CAUSE_REG		0x0278
#define ATHENA_CB1_SC_MISMATCH_CAUSE_REG		0x0279
#define ATHENA_CB2_SC_MISMATCH_CAUSE_REG		0x027A
#define ATHENA_CB3_SC_MISMATCH_CAUSE_REG		0x027B
#define ATHENA_PLD0_PARITY_ERROR_LSB_ADDR_REG		0x0107
#define ATHENA_PLD0_PARITY_ERROR_MSB_ADDR_REG		0x0108
#define ATHENA_PLD0_PARITY_ERROR_DATA_REG		0x0109
#define ATHENA_PLD0_PARITY_ERROR_REG			0x010A
#define ATHENA_PLD1_PARITY_ERROR_LSB_ADDR_REG		0x0210
#define ATHENA_PLD1_PARITY_ERROR_MSB_ADDR_REG		0x0211
#define ATHENA_PLD1_PARITY_ERROR_DATA_REG		0x0212
#define ATHENA_PLD1_PARITY_ERROR_REG			0x0213
#define ATHENA_SPI_SWITCH_SEL_CNTRL_REG			0x0110
#define ATHENA_SPI_CNTRL_REG				0x0111
#define ATHENA_SPI_PAGE_ADDR_REG			0x0112
#define ATHENA_SPI_OFFSET_ADDR_REG			0x0113
#define ATHENA_SPI_DATA_BYTE0_REG			0x0114
#define ATHENA_SPI_DATA_BYTE1_REG			0x0115
#define ATHENA_SPI_DATA_BYTE2_REG			0x0116
#define ATHENA_SPI_DATA_BYTE3_REG			0x0117
#define ATHENA_SPI_DATA_BYTE4_REG			0x0118
#define ATHENA_SPI_DATA_BYTE5_REG			0x0119
#define ATHENA_SPI_DATA_BYTE6_REG			0x011A
#define ATHENA_SPI_DATA_BYTE7_REG			0x011B
#define ATHENA_SPI_DATA_BYTE8_REG			0x011C
#define ATHENA_SPI_DATA_BYTE9_REG			0x011D
#define ATHENA_SPI_STATUS_REG				0x011F
#define ATHENA_INTER_CP_SPARE_IN_STATUS_REG		0x0140
#define ATHENA_INTER_CP_SPARE_OUT_STATUS_REG		0x0141
#define ATHENA_PC0_SB_DIR_READ_REG			0x0280
#define ATHENA_PC1_SB_DIR_READ_REG			0x0281
#define ATHENA_PC2_SB_DIR_READ_REG			0x0282
#define ATHENA_PC3_SB_DIR_READ_REG			0x0283
#define ATHENA_PC4_SB_DIR_READ_REG			0x0284
#define ATHENA_PC5_SB_DIR_READ_REG			0x0285
#define ATHENA_PC6_SB_DIR_READ_REG			0x0286
#define ATHENA_PC7_SB_DIR_READ_REG			0x0287
#define ATHENA_CB0_SB_DIR_READ_REG			0x0288
#define ATHENA_CB1_SB_DIR_READ_REG			0x0289
#define ATHENA_CB2_SB_DIR_READ_REG			0x028A
#define ATHENA_CB3_SB_DIR_READ_REG			0x028B
#define ATHENA_INTER_CP_SB_DIR_READ_1_REG		0x0142
#define ATHENA_INTER_CP_SB_DIR_READ_2_REG		0x0143
#define ATHENA_SEM_ID_DIR_READ_REG			0x0144
#define ATHENA_SEM_CMD_DIR_READ_REG			0x0145
#define ATHENA_SEM_VPI_DIR_READ_REG			0x0146
#define ATHENA_RAS_PC_CB_OUT_ORIDE_ENABLE_REG		0x0290
#define ATHENA_PC_OUT_ORIDE_SELECT_REG			0x0291
#define ATHENA_CB_OUT_ORIDE_SELECT_REG			0x0292
#define ATHENA_PC_CB_OUT_ORIDE_VAL_REG			0x0293
#define ATHENA_INTER_CP_OUT_ORIDE_ENABLE_REG		0x0147
#define ATHENA_INTER_CP_SB_DIR_OUT_CNTRL_1_REG		0x0148
#define ATHENA_INTER_CP_SB_DIR_OUT_CNTRL_2_REG		0x0149
#define ATHENA_RAS_SEM_OUT_ORIDE_ENABLE_REG		0x014A
#define ATHENA_SEM_ID_DIRECT_OUT_CNTRL_REG		0x014B
#define ATHENA_SEM_CMD_DIRECT_OUT_CNTRL_REG		0x014C
#define ATHENA_SEM_VPI_DIRECT_OUT_CNTRL_REG		0x014D
#define ATHENA_REGISTER_FILE				0x0FC0
#define ATHENA_REGISTER_FILE_SIZE			0x40

/*
 * Athena Semaphore Registers
 *
 * The following registers are offset from SEM_BASE
 */
#define ATHENA_SEM_CNTRL_REG				0x0000
#define ATHENA_SEM_INT_VEC_REG				0x0001
#define ATHENA_SEM_INT_MASK_REG				0x0002
#define ATHENA_SEM_ERROR_INJECT_ENABLE_REG		0x0003
#define ATHENA_SEM_ERROR_INJECT_REG			0x0004
#define ATHENA_SEM_CMD_ERROR_INJECT_REG			0x0005

/* Register bits */

/* Generic bit masks takes care of some registers */
#define ATHENA_PORT_BLADE_0_MASK			0x01
#define ATHENA_PORT_BLADE_1_MASK			0x02
#define ATHENA_PORT_BLADE_2_MASK			0x04
#define ATHENA_PORT_BLADE_3_MASK			0x08
#define ATHENA_PORT_BLADE_4_MASK			0x10
#define ATHENA_PORT_BLADE_5_MASK			0x20
#define ATHENA_PORT_BLADE_6_MASK			0x40
#define ATHENA_PORT_BLADE_7_MASK			0x80
#define ATHENA_CORE_BLADE_0_MASK			0x01
#define ATHENA_CORE_BLADE_1_MASK			0x02
#define ATHENA_CORE_BLADE_2_MASK			0x04
#define ATHENA_CORE_BLADE_3_MASK			0x08
#define ATHENA_FAN1					0x01
#define ATHENA_FAN2					0x02
#define ATHENA_FAN3					0x04
#define ATHENA_PS1					0x01
#define ATHENA_PS2					0x02
#define ATHENA_PS3					0x04
#define ATHENA_PS4					0x08

/*
 * 0x0008 - Athena CPLD ID Register
 */
#define ATHENA_CPLD_ID					0x13

/*
 * 0x016E - PCI Masters and Reset Register
 */
#define ATHENA_PCI_BUS1_RESET				0x01
#define ATHENA_PCI_BUS2_RESET				0x02
#define ATHENA_PCI_BUS3_RESET				0x04
#define ATHENA_PCI_BUS4_RESET				0x08
#define ATHENA_PCI_ARB_ENABLE				0x10
#define ATHENA_PCI_CLK_ENABLE				0x20

/*
 * 0x0101 - PCI Bridge Reset Status Register
 */
#define ATHENA_PCI_BRIDGE1_RESET_CAUSE			0x01
#define ATHENA_PCI_BRIDGE2_RESET_CAUSE			0x02
#define ATHENA_PCI_BRIDGE3_RESET_CAUSE			0x04
#define ATHENA_PCI_BRIDGE4_RESET_CAUSE			0x08
#define ATHENA_PCI_BRIDGE1_RESET_STATUS			0x10
#define ATHENA_PCI_BRIDGE2_RESET_STATUS			0x20
#define ATHENA_PCI_BRIDGE3_RESET_STATUS			0x40
#define ATHENA_PCI_BRIDGE4_RESET_STATUS			0x80

/*
 * 0x0130 - CP Interrupt and Reset Register
 */
#define ATHENA_OCP_STATE_CHANGE_INTR			0x01
#define ATHENA_OCP_ATTN_INTR				0x02
#define ATHENA_OCP_FAULT_INTR				0x08
#define ATHENA_LCP_MISC_INTR				0x10
#define ATHENA_LCP_ATTN_INTR				0x20
#define ATHENA_LCP_FAULT_INTR				0x80

/*
 * 0x0131 - CP Interrupt Mask Register
 */
#define ATHENA_OCP_STATE_CHANGE_INTR_MASK		0x01
#define ATHENA_OCP_ATTN_INTR_MASK			0x02
#define ATHENA_OCP_FAULT_INTR_MASK			0x08
#define ATHENA_LCP_MISC_INTR_MASK			0x10
#define ATHENA_LCP_ATTN_INTR_MASK			0x20
#define ATHENA_LCP_FAULT_INTR_MASK			0x80

/*
 * 0x0132 - LCP Attention Interrupt Reason Register
 */
#define ATHENA_PCI_BRIDGE1_INTR				0x01
#define ATHENA_PCI_BRIDGE2_INTR				0x02
#define ATHENA_PCI_BRIDGE3_INTR				0x04
#define ATHENA_PCI_BRIDGE4_INTR				0x08
#define ATHENA_ETH_PHY2_INTR				0x40
#define ATHENA_ETH_PHY3_INTR				0x80

/*
 * 0x0133 - LCP Attention Interrupt Mask Register
 */
#define ATHENA_PCI_BRIDGE1_INTR_MASK			0x01
#define ATHENA_PCI_BRIDGE2_INTR_MASK			0x02
#define ATHENA_PCI_BRIDGE3_INTR_MASK			0x04
#define ATHENA_PCI_BRIDGE4_INTR_MASK			0x08
#define ATHENA_ETH_PHY2_INTR_MASK			0x40
#define ATHENA_ETH_PHY3_INTR_MASK			0x80

/*
 * 0x134 - LCP Misc Interrupt Reason Register
 */
#define ATHENA_SERIAL_CABLE1_INTR			0x01
#define ATHENA_SERIAL_CABLE2_INTR			0x02
#define ATHENA_SPI_INTR					0x04
#define ATHENA_PCI_TRACE_BUFFER_TRIG_INTR		0x80

/*
 * 0x135 - LCP Misc Interrupt Mask Register
 */
#define ATHENA_SERIAL_CABLE1_INTR_MASK			0x01
#define ATHENA_SERIAL_CABLE2_INTR_MASK			0x02
#define ATHENA_SPI_INTR_MASK				0x04
#define ATHENA_PCI_TRACE_BUFFER_TRIG_INTR_MASK		0x80

/*
 * 0x0136 - OCP Attention Interrupt Mask Register
 */
#define ATHENA_OCP_HK_FF_IN_LEVEL_CHANGE		0x01
#define ATHENA_OCP_HK_SELF_FENCE_IN_LEVEL_CHANGE	0x02
#define ATHENA_OCP_PCI_FF_IN_LEVEL_CHANGE		0x04
#define ATHENA_PCI_CLK_ENB_LEVEL_CHANGE			0x20
#define ATHENA_OCP_RESET_IN_LEVLE_CHANGE		0x40
#define ATHENA_OCP_STATUS_LED_LEVEL_CHANGE		0x80

/*
 * 0x0137 - OCP Attention Interrupt Mask Register
 */
#define ATHENA_OCP_HK_FF_IN_LEVEL_CHANGE_MASK		0x01
#define ATHENA_OCP_HK_SELF_FENCE_IN_LEVEL_CHANGE_MASK	0x02
#define ATHENA_OCP_PCI_FF_IN_LEVEL_CHANGE_MASK		0x04
#define ATHENA_PCI_CLK_ENB_LEVEL_CHANGE_MASK		0x20
#define ATHENA_OCP_RESET_IN_LEVLE_CHANGE_MASK		0x40
#define ATHENA_OCP_STATUS_LED_LEVEL_CHANGE_MASK		0x80

/*
 * 0x0138 - LCP Fault Interrupt Reason Register
 */
#define ATHENA_PCI_RAS_PLD_PLB_PAR_ERR			0x01
#define ATHENA_CP_PLD0_PLD_PLB_PAR_ERR			0x02
#define ATHENA_CP_PLD1_PLD_PLB_PAR_ERR			0x04
#define ATHENA_PCI_RAS_TIMEOUT				0x08
#define ATHENA_SS_POWER_FAIL_INTR			0x10
#define ATHENA_INTER_CP_SB_LEVEL_MISMATCH		0x20
#define ATHENA_RAS_SC_MISMATCH_INTR			0x40
#define ATHENA_INVALID_CP_SLOT_ID			0x80

/*
 * 0x0139 - LCP Fault Interrupt Mask Register
 */
#define ATHENA_PCI_RAS_PLD_PLB_PAR_ERR_MASK		0x01
#define ATHENA_CP_PLD0_PLD_PLB_PAR_ERR_MASK		0x02
#define ATHENA_CP_PLD1_PLD_PLB_PAR_ERR_MASK		0x04
#define ATHENA_PCI_RAS_TIMEOUT_MASK			0x08
#define ATHENA_SS_POWER_FAIL_INTR_MASK			0x10
#define ATHENA_INTER_CP_SB_LEVEL_MISMATCH_MASK		0x20
#define ATHENA_RAS_SC_MISMATCH_INTR_MASK		0x40
#define ATHENA_INVALID_CP_SLOT_ID_MASK			0x80

/*
 * 0x220 - RAS Self-Check Interrupt Reason Register
 */
#define ATHENA_PC_SIDEBAND_SIGNALS			0x01
#define ATHENA_CP_SIDEBAND_SIGNALS			0x02
#define ATHENA_FAN_SIDEBAND_SIGNALS			0x04
#define ATHENA_PS_SIDEBAND_SIGNALS			0x08

/*
 * 0x221 - RAS Self-Check Interrupt Mask Register
 */
#define ATHENA_PC_SIDEBAND_SIGNALS_MASK			0x01
#define ATHENA_CP_SIDEBAND_SIGNALS_MASK			0x02
#define ATHENA_FAN_SIDEBAND_SIGNALS_MASK		0x04
#define ATHENA_PS_SIDEBAND_SIGNALS_MASK			0x08

/*
 * 0x013A - OCP Fault Interrupt Cause Register
 */
#define ATHENA_OCP_ERROR_IN_LEVEL_CHANGE		0x01

/*
 * 0x013B - OCP Fault Interrupt Mask Register
 */
#define ATHENA_OCP_ERROR_IN_LEVEL_CHANGE_MASK		0x01

/*
 * 0x013C - OCP Misc Status Register
 */
#define ATHENA_OCP_ERROR_IN				0x01
#define ATHENA_OCP_RESET_IN				0x02
#define ATHENA_OCP_PCI_CLK_ENB_IN			0x04

/*
 * 0x0123 - Serial Cable State Register
 */
#define ATHENA_SERIAL_CABLE1_STATUS			0x01
#define ATHENA_SERIAL_CABLE2_STATUS			0x02

/*
 * 0x013D - Inter-CP Input Level (L-L) Mismatch Interrupt and Reset Register
 */
#define ATHENA_CP_RESET_LOW_LOW				0x01
#define ATHENA_CP_ERROR_LOW_LOW				0x02
#define ATHENA_HK_FORCE_FENCE				0x04
#define ATHENA_HK_SELF_FORCE_FENCE_LOW_LOW		0x08
#define ATHENA_PCI_FORCE_FENCE				0x10
#define ATHENA_PCI_CLOCK_ENABLE				0x20
#define ATHENA_OCP_PRESENT_LOW				0x40

/*
 * 0x013E - Inter-CP Input Level (H-H) Mismatch Interrupt and Reset Register
 */
#define ATHENA_CP_RESET_HIGH_HIGH			0x01
#define ATHENA_CP_ERROR_HIGH_HIGH			0x02
#define ATHENA_HK_FORCE_FENCE_HIGH_HIGH			0x04
#define ATHENA_HK_SELF_FORCE_FENCE_HIGH_HIGH		0x08
#define ATHENA_PCI_FORCE_FENCE_HIGH_HIGH		0x10
#define ATHENA_PCI_CLOCK_ENABLE_HIGH_HIGH		0x20
#define ATHENA_OCP_PRESENT_HIGH_HIGH			0x40

/*
 * 0x0151 - Inter-CP Output Mismatch Interrupt and Reset Register
 */
#define ATHENA_CP_RESET_OUT_MISMATCH			0x01
#define ATHENA_CP_ERROR_OUT_MISMATCH			0x02
#define ATHENA_HK_FORCE_FENCE_OUT_MISMATCH		0x04
#define ATHENA_HK_SELF_FENCE_MISMATCH	 		0x08
#define ATHENA_PCI_FORCE_FENCE_OUT_MISMATCH		0x10
#define ATHENA_PCI_CLOCK_ENABLE_OUT_MISMATCH		0x20

/*
 * 0x0230 - Fan Interrupt Register
 */
#define ATHENA_FAN0_INTR				0x01
#define ATHENA_FAN1_INTR				0x02
#define ATHENA_FAN2_INTR				0x04

/*
 * 0x0231 - Fan Interrupt Mask Register
 */
#define ATHENA_FAN0_INTR_MASK				0x01
#define ATHENA_FAN1_INTR_MASK				0x02
#define ATHENA_FAN2_INTR_MASK				0x04

/*
 * 0x0232 - 0x234 - Fan Interrupt Cause and Status Register
 */
#define ATHENA_FAN_FAULT_CHANGE_CAUSE			0x01
#define ATHENA_FAN_FAULT_STATUS				0x02
#define ATHENA_FAN_STATE_CHANGE_CAUSE			0x04
#define ATHENA_FAN_PRESENT_STATUS			0x08

/*
 * 0x0235 Fan Force Interrupt Register
 */
#define ATHENA_FAN0_FORCE_INT				0x01
#define ATHENA_FAN1_FORCE_INT				0x02
#define ATHENA_FAN2_FORCE_INT				0x04

/*
 * 0x0236 Fan Force Value Register
 */
#define ATHENA_FAN_FORCE_FAULT				0x01
#define ATHENA_FAN_FORCE_STATE				0x04

/*
 * 0x0237 - 0x239 - Fan State and Fault Change MASK Register
 */
#define ATHENA_FAN_FORCE_FAULT_CHANGE_CAUSE_MASK	0x01
#define ATHENA_FAN_FORCE_STATE_CHANGE_CAUSE_MASK	0x04

/*
 * 0x023A - Fan Status LED Control Register
 */
#define ATHENA_FAN0_LED_CNTRL				0x01
#define ATHENA_FAN1_LED_CNTRL				0x02
#define ATHENA_FAN2_LED_CNTRL				0x04

/*
 * 0x023B - Fan Status LED Blink Control Register
 */
#define ATHENA_FAN0_LED_BLINK_CNTRL			0x01
#define ATHENA_FAN1_LED_BLINK_CNTRL			0x02
#define ATHENA_FAN2_LED_BLINK_CNTRL			0x04

/*
 * 0x0240 - PS Interrupt Cause Register
 */
#define ATHENA_PS1_INTR					0x01
#define ATHENA_PS2_INTR					0x02
#define ATHENA_PS3_INTR					0x04
#define ATHENA_PS4_INTR					0x08

/*
 * 0x0241 - PS Interrupt Mask Register
 */
#define ATHENA_PS1_INTR_MASK				0x01
#define ATHENA_PS2_INTR_MASK				0x02
#define ATHENA_PS3_INTR_MASK				0x04
#define ATHENA_PS4_INTR_MASK				0x08

/*
 * 0x0242 - PS Force Interrupt Register
 */
#define ATHENA_PS1_FORCE_INTR				0x01
#define ATHENA_PS2_FORCE_INTR				0x02
#define ATHENA_PS3_FORCE_INTR				0x04
#define ATHENA_PS4_FORCE_INTR				0x08

/*
 * 0x0243 - PS Force Status Value Register
 */
#define ATHENA_AC_OK_VALUE				0x01
#define ATHENA_POWER_GOOD_VALUE				0x02
#define ATHENA_TEMP_OK_VALUE				0x04
#define ATHENA_AC_HIGH_RANGE_VALUE			0x08
#define ATHENA_PRESENT_VALUE				0x40

/*
 * 0x0244 - PS Status LED Control Register
 */
#define ATHENA_PS1_LED_CNTRL				0x01
#define ATHENA_PS2_LED_CNTRL				0x02
#define ATHENA_PS3_LED_CNTRL				0x04
#define ATHENA_PS4_LED_CNTRL				0x08

/*
 * 0x0245 - PS Status LED Blink Control Register
 */
#define ATHENA_PS1_LED_BLINK_CNTRL			0x01
#define ATHENA_PS2_LED_BLINK_CNTRL			0x02
#define ATHENA_PS3_LED_BLINK_CNTRL			0x04
#define ATHENA_PS4_LED_BLINK_CNTRL			0x08

/*
 * 0x0248 - 0x24B - PS Interrupt Cause and Status Registers
 */
#define ATHENA_AC_OK_CHANGE				0x01
#define ATHENA_POWER_OK_CHANGE				0x02
#define ATHENA_TEMP_OK_CHANGE				0x04
#define ATHENA_AC_HIGH_OK_CHANGE			0x08
#define ATHENA_PS_STATE_CHANGE				0x40
#define ATHENA_PS_PRESENT				0x80

/*
 * 0x024C - 0x24F - PS Status Registers
 */
#define ATHENA_PS_AC_OK					0x01
#define ATHENA_PS_POWER_GOOD				0x02
#define ATHENA_PS_TEMP_OK				0x04
#define ATHENA_PS_AC_HIGH_RANGE_OK			0x08

/*
 * 0x0250 - 0x253 - PS Interrupt Cause Mask Registers
 */
#define ATHENA_AC_OK_CHANGE_MASK			0x01
#define ATHENA_POWER_OK_CHANGE_MASK			0x02
#define ATHENA_TEMP_OK_CHANGE_MASK			0x04
#define ATHENA_AC_HIGH_OK_CHANGE_MASK			0x08
#define ATHENA_PS_STATE_CHANGE_MASK			0x40

/*
 * 0x0161 - Processor Blade State Change Force Register
 */
#define ATHENA_FORCE_OCP_STATE_INTR			0x01
#define ATHENA_FORCE_OCP_PRESENT_STATE			0x02

/*
 * 0x0165 - Processor Blade Fence Control and Status Register
 */
#define ATHENA_OCP_HK_FORCE_FENCE_IN			0x01
#define ATHENA_OCP_HK_SELF_FENC_IN			0x02
#define ATHENA_OCP_PCI_FORCE_FENCE_IN			0x04
#define ATHENA_PCI_FORCE_FENCE_OUT			0x10
#define ATHENA_HK_FORCE_FENCE_OUT			0x20
#define ATHENA_PCI_SELF_FENCE				0x40
#define ATHENA_HK_SELF_FENCE				0x80

/*
 * 0x0166 - Processor Blade Status LED Control Register
 */
#define ATHENA_OTHER_CP_STATUS_LED			0x01
#define ATHENA_SYSTEM_LED				0x02
#define ATHENA_LOCAL_CP_STATUS_LED			0x08
#define ATHENA_MASTER_BLUE_LED				0x80

/*
 * 0x0167 - Processor Blade State Register
 */
#define ATHENA_OTHER_CP_CURRENT_STATE			0x01

/*
 * 0x016B - Processor Blade Status LED Blink Control Register
 */
#define ATHENA_BLINK_OCP_STATUS_LED			0x01
#define ATHENA_SYSTEM_LED_BLINK				0x02
#define ATHENA_BLINK_LCP_STATUS_LED			0x08

/*
 * 0x016F - Processor Fence Status Register
 */
#define ATHENA_HK_FORCE_FENCE_OUT_STATUS		0x01
#define ATHENA_PCI_FORCE_FENCE_OUT_STATUS		0x02
#define ATHENA_OCP_FORCE_FENCE_OUT_STATUS		0x04

/*
 * 0x0018 - OCP Reset Control, LCP Error and FC Clock Enable Register
 */
#define ATHENA_RESET_OCP				0x01
#define ATHENA_ENABLE_RESET_BY_OCP			0x02
#define ATHENA_LCP_ERROR				0x04
#define ATHENA_LCP_ERROR_STATUS				0x08
#define ATHENA_FC_CLOCK_ENABLE				0x10

/*
 * 0x0150 All Blade Ignore OCP Fence Control Register
 */
#define ATHENA_IGNORE_OCP_FORCE_FENCE			0x01

/*
 * 0x0019 - LCP Slot ID and Seated Register
 */
#define ATHENA_SLOT_ID_MASK				0x7F
#define ATHENA_BOARD_SEATED				0x80
#define ATHENA_CP0_4_SLOT				0x00
#define ATHENA_CP1_4_SLOT				0x71
#define ATHENA_CP0_8_SLOT				0x32
#define ATHENA_CP1_8_SLOT				0x43
#define ATHENA_CP0_SLOT					ATHENA_CP0_4_SLOT
#define ATHENA_CP1_SLOT					ATHENA_CP1_4_SLOT

/*
 * 0x001A - I2C Select Register
 */
#define ATHENA_I2C_WRITE_SELECT_MASK			0x1F
#define ATHENA_I2C_WRITE_ENABLE				0x80

#define ATHENA_I2C_PB0_SELECT				0x01
#define ATHENA_I2C_PB1_SELECT				0x02
#define ATHENA_I2C_PB2_SELECT				0x03
#define ATHENA_I2C_PB3_SELECT				0x04
#define ATHENA_I2C_CP0_SELECT				0x0D
#define ATHENA_I2C_CP1_SELECT				0x0E
#define ATHENA_I2C_PB4_SELECT				0x05
#define ATHENA_I2C_PB5_SELECT				0x06
#define ATHENA_I2C_PB6_SELECT				0x07
#define ATHENA_I2C_PB7_SELECT				0x08
#define ATHENA_I2C_CB0_SELECT				0x09
#define ATHENA_I2C_CB1_SELECT				0x0A
#define ATHENA_I2C_CB2_SELECT				0x0B
#define ATHENA_I2C_CB3_SELECT				0x0C
#define ATHENA_I2C_SS1_SELECT				0x16
#define ATHENA_I2C_SS2_SELECT				0x17
#define ATHENA_I2C_SS3_SELECT				0x18
#define ATHENA_I2C_SS4_SELECT				0x19
#define ATHENA_I2C_FAN1_SELECT				0x0F
#define ATHENA_I2C_FAN2_SELECT				0x10
#define ATHENA_I2C_FAN3_SELECT				0x11
#define ATHENA_I2C_PS1_SELECT				0x12
#define ATHENA_I2C_PS2_SELECT				0x13
#define ATHENA_I2C_PS3_SELECT				0x14
#define ATHENA_I2C_PS4_SELECT				0x15
#define ATHENA_I2C_DISCONNECT				0x1F

/*
 * 0x010F - I2C Reset Control Register
 */
#define ATHENA_I2C_RESET_SELECT_MASK			0x1F
#define ATHENA_I2C_RESET				0x40

/*
 * 0x001B - BootPROM Control
 */
#define ATHENA_BOOTPROM_WRITE_ENB			0x01

/*
 * 0x001C - User Flash Control Register
 */
#define ATHENA_USER_FLASH0_WRITE_ENB			0x01
#define ATHENA_USER_FLASH1_WRITE_ENB			0x02

/*
 * 0x0102 - Reset Control Register
 */
#define ATHENA_BOOTPROM_FLASH_RESET			0x01
#define ATHENA_KERNEL1_FLASH_RESET			0x02
#define ATHENA_KERNEL2_FLASH_RESET			0x04
#define ATHENA_TRACE_BUFFER_RESET			0x08
#define ATHENA_PCI_PLD_PLB_RESET			0x10
#define ATHENA_PCI_PLD_PCI_RESET			0x20
#define ATHENA_SATA_RESET				0x40
#define ATHENA_SEC_PROC_RESET				0x80

/*
 * 0x0103 - Etherenet Reset Control Register
 */
#define ATHENA_ETH_PHY1_RESET				0x01
#define ATHENA_ETH_PHY2_RESET				0x02
#define ATHENA_ETH_PHY3_RESET				0x04
#define ATHENA_ETH_SW1_RESET				0x10
#define ATHENA_ETH_SW2_RESET				0x20

/*
 * 0x013F -  Secure Storage (SS) Status Register
 */
#define ATHENA_SS1_POWER_FAIL				0x01
#define ATHENA_SS2_POWER_FAIL				0x02
#define ATHENA_SS3_POWER_FAIL				0x04
#define ATHENA_SS4_POWER_FAIL				0x08

/*
 * 0x0100 - Reset Cause Register
 */
#define ATHENA_RESET_CAUSE_SW_MASK			0x0F
#define ATHENA_RESET_CAUSE_MASK				0xF0

/*
 * 0x0217 - JTAG PLD Version Register
 */
#define ATHENA_JTAG_PLD_VERSION_MASK			0xF0

/*
 * 0x0216 - Seven Segment LED Register
 */
#define ATHENA_SEVEN_SEGMENT_BIT_0			0x01
#define ATHENA_SEVEN_SEGMENT_BIT_1			0x02
#define ATHENA_SEVEN_SEGMENT_BIT_2			0x04
#define ATHENA_SEVEN_SEGMENT_BIT_3			0x08
#define ATHENA_SEVEN_SEGMENT_BIT_4			0x10
#define ATHENA_SEVEN_SEGMENT_BIT_5			0x20
#define ATHENA_SEVEN_SEGMENT_BIT_6			0x40
#define ATHENA_SEVEN_SEGMENT_BIT_7			0x80

/*
 * 0x0215 - Seven Segment LED Blank Register
 */
#define ATHENA_SEVEN_SEGMENT_BLANK			0x01

/*
 * 0x010C - Clock Module Satus Register 2
 */
#define ATHENA_CLK_MODULE_MASK				0x0F

/*
 * 0x010D - Test Mode GPIO Control Register
 */
#define ATHENA_GPIO_DIR_MASK				0xF0
#define ATHENA_GPIO_VAL_MASK				0x0F

/*
 * 0x010E - PLD0 Test Mode Status Register
 */
#define ATHENA_PLD0_TEST_MODE_MASK			0x1F

/*
 * 0x0218 - PCI RAS PLD Re-program Register
 */
#define ATHENA_PCI_PLD_PROGRAM				0x01
#define ATHENA_PCI_PLD_DONE				0x80

/*
 * 0x0106 - PCI Trace Buffer Trigger
 */
#define ATHENA_FORCE_TRIGGER				0x01

/*
 * 0x0214 - PLD1 Test Mode Status Register
 */
#define ATHENA_PLD1_TEST_MODE_MASK			0x1F

/*
 * 0x0219 - Fan & PS Interrupt Force Enable Register
 */
#define ATHENA_FORCE_INT_ENABLE				0xAA

/*
 * 0x0120 - OCP Reset Key Register
 */
#define ATHENA_KEY_FOR_RESET_OTHER_CP			0x01

/*
 * 0x0121 - HK Force Fence Key Register
 */
#define ATHENA_KEY_FOR_FORCE_FENCE			0x40

/*
 * 0x0122 - PCI Force Fence Key Register
 */
#define ATHENA_KEY_FOR_PCI_FORCE_FENCE			0x04

/*
 * 0x0270 - 0x0277 - PC Self-checking Mismatch Cause Registers
 */
#define ATHENA_PC_MISMATCH_FORCE_BLADE_FENCE_OUT	0x01
#define ATHENA_PC_MISMATCH_IGNORE_OCP_BLADE_FENCE_OUT	0x02
#define ATHENA_PC_MISMATCH_STATUS_LED_OUT		0x04
#define ATHENA_PC_MISMATCH_PRESENT_IN_LL_HH		0x08
#define ATHENA_PC_MISMATCH_ATTN_IN_LL			0x10
#define ATHENA_PC_MISMATCH_ATTN_IN_HH			0x20
#define ATHENA_PC_MISMATCH_FAULT_IN_LL			0x40
#define ATHENA_PC_MISMATCH_FAULT_IN_HH			0x80

/*
 * 0x0278 - 0x027B - CB Self-checking Mismatch Cause Registers
 */
#define ATHENA_CB_MISMATCH_FORCE_BLADE_FENCE_OUT	0x01
#define ATHENA_CB_MISMATCH_IGNORE_OCP_BLADE_FENCE_OUT	0x02
#define ATHENA_CB_MISMATCH_STATUS_LED_OUT		0x04
#define ATHENA_CB_MISMATCH_PRESENT_IN_LL_HH		0x08
#define ATHENA_CB_MISMATCH_ATTN_IN_LL			0x10
#define ATHENA_CB_MISMATCH_ATTN_IN_HH			0x20
#define ATHENA_CB_MISMATCH_FAULT_IN_LL			0x40
#define ATHENA_CB_MISMATCH_FAULT_IN_HH			0x80

/*
 * 0x010A - PLD0 Parity Error Register
 */
#define ATHENA_PLD0_DAT_PAR				0x01
#define ATHENA_PLD0_SEM_REG_CS_PAR_ERR			0x02
#define ATHENA_PLD0_FORCE_WRITE_PAR_ERROR		0x20
#define ATHENA_PLD0_FORCE_READ_PAR_ERROR		0x40
#define ATHENA_PLD0_PAR_EN				0x80

/*
 * 0x0213 - PLD1 Parity Error Register
 */
#define ATHENA_PLD1_DAT_PAR				0x01
#define ATHENA_PLD1_FORCE_WRITE_PAR_ERROR		0x20
#define ATHENA_PLD1_FORCE_READ_PAR_ERROR		0x40
#define ATHENA_PLD1_PAR_EN				0x80

/*
 * 0x0110 - SPI Switch Select Control Register
 */
#define ATHENA_SPI_SW_SEL				0x01

/*
 * 0x0111 - SPI Control Register
 */
#define ATHENA_SPI_EN					0x01
#define ATHENA_SPI_RD_WR_1				0x02
#define ATHENA_SPI_CMD_VALID				0x04
#define ATHENA_SPI_CONTINUE				0x08
#define ATHENA_SPI_LEN_MASK				0xF0

/*
 * 0x011F - SPI Status Register
 */
#define ATHENA_SPI_CMD_ERR				0x01
#define ATHENA_SPI_INVALID_CMD				0x02
#define ATHENA_SPI_SPIF_TIMEOUT				0x04
#define ATHENA_SPI_RDY_TIMEOUT				0x08
#define ATHENA_SPI_RACK_TIMEOUT				0x10
#define ATHENA_SPI_CMD_DONE				0x20
#define ATHENA_SPI_STATE_MASK				0xC0
#define ATHENA_SPI_ERR_MASK				0x1F


/*
 * 0x0140 - Inter-CP Spare Input Signal Status Register
 */
#define ATHENA_CP_CP_SPARE_IN_MASK			0x3F

/*
 * 0x0141 - Inter-CP Spare Output Signal Status Register
 */
#define ATHENA_CP_CP_SPARE_OUT_MASK			0x3F

/*
 * 0x0280 - 0x028B - PC/CB Sideband Direct Read Registers
 */
#define ATHENA_FRU_PRESENT_L				0x01
#define ATHENA_FRU_PRESENT_H				0x02
#define ATHENA_FRU_FAULT_L				0x04
#define ATHENA_FRU_FAULT_H				0x08
#define ATHENA_FRU_ATTN_INTR_L				0x10
#define ATHENA_FRU_ATTN_INTR_H				0x20
#define ATHENA_SPARE_IN_0				0x40
#define ATHENA_SPARE_IN_1				0x80

/*
 * 0x0142 - Inter-CP Sideband Direct Read Register 1
 */
#define ATHENA_OCP_PRESENT_IN_L				0x01
#define ATHENA_OCP_PRESENT_IN_H				0x02
#define ATHENA_OCP_ERROR_IN_L				0x04
#define ATHENA_OCP_ERROR_IN_H				0x08
#define ATHENA_OCP_HK_FORCE_FENCE_IN_L			0x10
#define ATHENA_OCP_HK_FORCE_FENCE_IN_H			0x20
#define ATHENA_OCP_HK_SELF_FENCE_IN_L			0x40
#define ATHENA_OCP_HK_SELF_FENCE_IN_H			0x80

/*
 * 0x0143 - Inter-CP Sideband Direct Read Register 2
 */
#define ATHENA_OCP_RESET_IN_L				0x01
#define ATHENA_OCP_RESET_IN_H				0x02
#define ATHENA_OCP_PCI_FORCE_FENCE_IN_L			0x04
#define ATHENA_OCP_PCI_FORCE_FENCE_IN_H			0x08
#define ATHENA_OCP_PCI_CLK_ENB_L			0x10
#define ATHENA_OCP_PCI_CLK_ENB_H			0x20
#define ATHENA_OCP_STATUS_LED				0x80

/*
 * 0x0144 - Semaphore ID Direct Read Register
 */
#define ATHENA_SEM_ID_IN0				0x01
#define ATHENA_SEM_ID_IN1				0x02
#define ATHENA_SEM_ID_IN2				0x04
#define ATHENA_SEM_ID_IN3				0x08
#define ATHENA_SEM_ID_IN4				0x10
#define ATHENA_SEM_ID_IN5				0x20
#define ATHENA_SEM_ID_IN6				0x40
#define ATHENA_SEM_ID_IN7				0x80

/*
 * 0x0145 - Semaphore CMD Direct Read Register
 */
#define ATHENA_SEM_CMD_IN0				0x01
#define ATHENA_SEM_CMD_IN1				0x02
#define ATHENA_SEM_CMD_IN2				0x04
#define ATHENA_SEM_CMD_IN3				0x08
#define ATHENA_SEM_CMD_IN4				0x10

/*
 * 0x0146 Semaphore Valid/Par/Int Direct Read Register
 */
#define ATHENA_SEM_VALID_IN_L				0x01
#define ATHENA_SEM_VALID_IN_H				0x02
#define ATHENA_SEM_INT_IN_L				0x04
#define ATHENA_SEM_INTIN_H				0x08
#define ATHENA_SEM_PARITY				0x10

/*
 * 0x0292 - CB Output Override Select Register
 */
#define ATHENA_CB_OUT_OVERRIDE_SEL_MASK			0x0F

/*
 * 0x0293 - PC/CB Output Override Value Register
 */
#define ATHENA_FRU_FORCE_FENCE_L			0x01
#define ATHENA_FRU_FORCE_FENCE_H			0x02
#define ATHENA_FRU_IGNORE_OCP_FORCE_FENCE_L		0x04
#define ATHENA_FRU_IGNORE_OCP_FORCE_FENCE_H		0x08
#define ATHENA_SPARE_OUT0				0x40
#define ATHENA_SPARE_OUT1				0x80

/*
 * 0x0148 - Inter-CP Sideband Direct Output Control Register 1
 */
#define ATHENA_CP_ERROR_OUT_L				0x04
#define ATHENA_CP_ERROR_OUT_H				0x08
#define ATHENA_CP_HK_FORCE_FENCE_OUT_L			0x10
#define ATHENA_CP_HK_FORCE_FENCE_OUT_H			0x20
#define ATHENA_CP_HK_SELF_FENCE_OUT_L			0x40
#define ATHENA_CP_HK_SELF_FENCE_OUT_H			0x80

/*
 * 0x0149 - Inter-CP Sideband Direct Output Control Register 1
 */
#define ATHENA_CP_RESET_OUT_L				0x01
#define ATHENA_CP_RESET_OUT_H				0x02
#define ATHENA_CP_PCI_FORCE_FENC_OUT_L			0x04
#define ATHENA_CP_PCI_FORCE_FENC_OUT_H			0x08
#define ATHENA_CP_PCI_CLK_ENB_OUT_L			0x10
#define ATHENA_CP_PCI_CLK_ENB_OUT_H			0x20

/*
 * 0x014B - Semaphore ID Direct Output Control Register
 */
#define ATHENA_SEM_ID_OUT0				0x01
#define ATHENA_SEM_ID_OUT1				0x02
#define ATHENA_SEM_ID_OUT2				0x04
#define ATHENA_SEM_ID_OUT3				0x08
#define ATHENA_SEM_ID_OUT4				0x10
#define ATHENA_SEM_ID_OUT5				0x20
#define ATHENA_SEM_ID_OUT6				0x40
#define ATHENA_SEM_ID_OUT7				0x80

/*
 * 0x014C - Semaphore CMD Direct Output Control Register
 */
#define ATHENA_SEM_CMD_OUT0				0x01
#define ATHENA_SEM_CMD_OUT1				0x02
#define ATHENA_SEM_CMD_OUT2				0x04
#define ATHENA_SEM_CMD_OUT3				0x08
#define ATHENA_SEM_CMD_OUT4				0x10

/*
 * 0x014D - Semaphore Valid/Par/Int Direct Output Control Register
 */
#define ATHENA_SEM_VALID_OUT_L				0x01
#define ATHENA_SEM_VALID_OUT_H				0x02
#define ATHENA_SEM_INT_OUT_L				0x04
#define ATHENA_SEM_INT_OUT_H				0x08
#define ATHEAN_PAR_OUT					0x10

/* Semaphore Resiter Bits */

/*
 * 0x0000 - Semaphore Control Register
 */
#define ATHENA_SEM_EN					0x01
#define ATHENA_SEM_WRITE_EN				0x10
#define ATHENA_SEM_FORCE_MASTER				0x80

/*
 * 0x0001 - Semaphore Interrupt Vector Register
 */
#define ATHENA_SEM_INTR					0x01
#define ATHENA_SEM_INTR_NOT_RETURNED			0x02
#define ATHENA_SEM_TIMEOUT				0x04
#define ATHENA_SEM_OUTBOUND_ERROR			0x08
#define ATHENA_SEM_INTERNAL_ERROR			0x10
#define ATHENA_SEM_INTR_IN_ERROR			0x20
#define ATHENA_SEM_VALID_IN_ERR_INTR			0x40
#define ATHENA_SEM_MASTER_STATUS			0x80

/*
 * 0x0002 - Semaphore Interrupt Mask Register
 */
#define ATHENA_SEM_INTR_MASK				0x01
#define ATHENA_SEM_INTR_NOT_RETURNED_MASK		0x02
#define ATHENA_SEM_TIMEOUT_MASK				0x04
#define ATHENA_SEM_OUTBOUND_ERROR_MASK			0x08
#define ATHENA_SEM_INTERNAL_ERROR_MASK			0x10
#define ATHENA_SEM_INTR_IN_ERROR_MASK			0x20
#define ATHENA_SEM_VALID_IN_ERR_INTR_MASK		0x40

/*
 * 0x0004 - Semaphore Error Inject Register
 */
#define ATHENA_GATE_SEM_VALID_OUT			0x01
#define ATHENA_GATE_SEM_INTR_OUT			0x02
#define ATHENA_GATE_SEM_VALID_IN			0x04
#define ATHENA_GATE_SEM_INTR_IN				0x08
#define ATHENA_SEM_IF_ERROR_CNTRL_MASK			0x70
#define ATHENA_INJECT_SEM_RAM_PARITY_ERR		0x80

/*
 * 0x0005 - Semaphore CMD Error Inject Register
 */
#define ATHENA_ERR_INJECT_SEM_CMD_MASK			0x1F
#define ATHENA_ERR_INJECT_REQ_SEM_CMD			0x40
#define ATHENA_ERR_INJECT_RESP_SEM_CMD			0x80


/*
 * PCI RAS PLD defines
 */
#define ATHENA_RAS_BASE				0x00000001F7000000ULL
#define ATHENA_RAS_TOTAL_SIZE			0x1000
#define ATHENA_RAS_BLOCK_SIZE			0x0200

/* Register offsets */
#define ATHENA_RAS_PLD_VERSION_REG			0x0000
#define ATHENA_RAS_PCI_INTERFACE_CTL_REG		0x0001
#define ATHENA_RAS_POST_TRIGGER_SAMPLE_CTL_REG		0x0002
#define ATHENA_RAS_RAM_TRIGGER_ADDR_REG			0x0003
#define ATHENA_RAS_RAM_CURRENT_SAMPLE_ADDR_REG		0x0004
#define ATHENA_RAS_TRIGGER_CONTROL_REG			0x0005
#define ATHENA_RAS_TRIGGER_STATUS_REG			0x0006
#define ATHENA_RAS_TRIGGER_REASON_REG			0x0007
#define ATHENA_RAS_CPLD_ID_REG				0x0008
#define ATHENA_RAS_TRIGGER_ADDR_0_REG			0x0010
#define ATHENA_RAS_TRIGGER_ADDR_1_REG			0x0011
#define ATHENA_RAS_TRIGGER_ADDR_2_REG			0x0012
#define ATHENA_RAS_TRIGGER_ADDR_3_REG			0x0013
#define ATHENA_RAS_TRIGGER_CBE_REG			0x0014
#define ATHENA_RAS_TRIGGER_ADDR_0_MASK_REG		0x0018
#define ATHENA_RAS_TRIGGER_ADDR_1_MASK_REG		0x0019
#define ATHENA_RAS_TRIGGER_ADDR_2_MASK_REG		0x001A
#define ATHENA_RAS_TRIGGER_ADDR_3_MASK_REG		0x001B
#define ATHENA_RAS_TRIGGER_CBE_MASK_REG			0x001C
#define ATHENA_RAS_TRIGGER_DATA_0_REG			0x0020
#define ATHENA_RAS_TRIGGER_DATA_1_REG			0x0021
#define ATHENA_RAS_TRIGGER_DATA_2_REG			0x0022
#define ATHENA_RAS_TRIGGER_DATA_3_REG			0x0023
#define ATHENA_RAS_TRIGGER_DATA_4_REG			0x0024
#define ATHENA_RAS_TRIGGER_DATA_5_REG			0x0025
#define ATHENA_RAS_TRIGGER_DATA_6_REG			0x0026
#define ATHENA_RAS_TRIGGER_DATA_7_REG			0x0027
#define ATHENA_RAS_TRIGGER_DATA_0_MASK_REG		0x0028
#define ATHENA_RAS_TRIGGER_DATA_1_MASK_REG		0x0029
#define ATHENA_RAS_TRIGGER_DATA_2_MASK_REG		0x002A
#define ATHENA_RAS_TRIGGER_DATA_3_MASK_REG		0x002B
#define ATHENA_RAS_TRIGGER_DATA_4_MASK_REG		0x002C
#define ATHENA_RAS_TRIGGER_DATA_5_MASK_REG		0x002D
#define ATHENA_RAS_TRIGGER_DATA_6_MASK_REG		0x002E
#define ATHENA_RAS_TRIGGER_DATA_7_MASK_REG		0x002F
#define ATHENA_RAS_PCI_AGENT_TRACE_IGNORE_REG		0x0030
#define ATHENA_RAS_PCI_FAULT_INJ_CTL_REG		0x0031
#define ATHENA_RAS_PCI_TO_ADDR_0_CAPTURE_REG		0x0032
#define ATHENA_RAS_PCI_TO_ADDR_1_CAPTURE_REG		0x0033
#define ATHENA_RAS_PCI_TO_ADDR_2_CAPTURE_REG		0x0034
#define ATHENA_RAS_PCI_TO_ADDR_3_CAPTURE_REG		0x0035
#define ATHENA_RAS_PCI_TO_CBE_CAPTURE_REG		0x0036
#define ATHENA_RAS_PLB_IF_PARITY_ERR_REG		0x0038
#define ATHENA_RAS_PLB_IF_PARITY_ERR_LSB_ADDR_REG	0x0039
#define ATHENA_RAS_PLB_IF_PARITY_ERR_MSB_ADDR_REG	0x003A
#define ATHENA_RAS_PLB_IF_PARITY_ERR_DATA_REG		0x003B
#define ATHENA_RAS_RETRY_3_ENTRIES_RLOV_CNT_REG		0x0040
#define ATHENA_RAS_RETRY_2K_ENTRIES_RLOV_CNT_REG	0x0041
#define ATHENA_RAS_RETRY_4K_ENTRIES_RLOV_CNT_REG	0x0042
#define ATHENA_RAS_RETRY_8K_ENTRIES_RLOV_CNT_REG	0x0043
#define ATHENA_RAS_RETRY_16K_ENTRIES_RLOV_CNT_REG	0x0044
#define ATHENA_RAS_RETRY_32K_ENTRIES_RLOV_CNT_REG	0x0045
#define ATHENA_RAS_TO_1_TENTH_RLOV_CNT_REG		0x0048
#define ATHENA_RAS_TO_1MS_RLOV_CNT_REG			0x0049
#define ATHENA_RAS_TO_5MS_RLOV_CNT_REG			0x004A
#define ATHENA_RAS_TO_10MS_RLOV_CNT_REG			0x004B
#define ATHENA_RAS_TO_50MS_RLOV_CNT_REG			0x004C
#define ATHENA_RAS_TO_100MS_RLOV_CNT_REG		0x004D
#define ATHENA_RAS_NT_PCI_BRIDGE_0_ADDR_1_REG		0x0050
#define ATHENA_RAS_NT_PCI_BRIDGE_0_ADDR_2_REG		0x0051
#define ATHENA_RAS_NT_PCI_BRIDGE_0_ADDR_3_REG		0x0052
#define ATHENA_RAS_NT_PCI_BRIDGE_0_ADDR_1_MASK_REG	0x0054
#define ATHENA_RAS_NT_PCI_BRIDGE_0_ADDR_2_MASK_REG	0x0055
#define ATHENA_RAS_NT_PCI_BRIDGE_0_ADDR_3_MASK_REG	0x0056
#define ATHENA_RAS_NT_PCI_BRIDGE_1_ADDR_1_REG		0x0058
#define ATHENA_RAS_NT_PCI_BRIDGE_1_ADDR_2_REG		0x0059
#define ATHENA_RAS_NT_PCI_BRIDGE_1_ADDR_3_REG		0x005A
#define ATHENA_RAS_NT_PCI_BRIDGE_1_ADDR_1_MASK_REG	0x005C
#define ATHENA_RAS_NT_PCI_BRIDGE_1_ADDR_2_MASK_REG	0x005D
#define ATHENA_RAS_NT_PCI_BRIDGE_1_ADDR_3_MASK_REG	0x005E
#define ATHENA_RAS_NT_PCI_BRIDGE_2_ADDR_1_REG		0x0060
#define ATHENA_RAS_NT_PCI_BRIDGE_2_ADDR_2_REG		0x0061
#define ATHENA_RAS_NT_PCI_BRIDGE_2_ADDR_3_REG		0x0062
#define ATHENA_RAS_NT_PCI_BRIDGE_2_ADDR_1_MASK_REG	0x0064
#define ATHENA_RAS_NT_PCI_BRIDGE_2_ADDR_2_MASK_REG	0x0065
#define ATHENA_RAS_NT_PCI_BRIDGE_2_ADDR_3_MASK_REG	0x0066
#define ATHENA_RAS_NT_PCI_BRIDGE_3_ADDR_1_REG		0x0068
#define ATHENA_RAS_NT_PCI_BRIDGE_3_ADDR_2_REG		0x0069
#define ATHENA_RAS_NT_PCI_BRIDGE_3_ADDR_3_REG		0x006A
#define ATHENA_RAS_NT_PCI_BRIDGE_3_ADDR_1_MASK_REG	0x006C
#define ATHENA_RAS_NT_PCI_BRIDGE_3_ADDR_2_MASK_REG	0x006D
#define ATHENA_RAS_NT_PCI_BRIDGE_3_ADDR_3_MASK_REG	0x006E

#define ATHENA_RAS_TRACE_RAM				0x0200
#define ATHENA_RAS_TRACE_RAM_SIZE			0xe00


/*
 * Blazer
 */
#define BLAZER_CPLD_STATUS_REG	    (CPLD_BASE_ADDRESS | 0xD4)
#define BLAZER_CPLD_DIAG_BDR	    (CPLD_BASE_ADDRESS | 0xD8)
#define BLAZER_CPLD_XCTL_REG	    (CPLD_BASE_ADDRESS | 0xDC)

#define BLAZER_STATUS_INIT_COMP	    0x40

#define	BLAZER_XCTL_EXTDIAG_MASK    0x3
#define	BLAZER_XCTL_POST_RSVD	    0x3
#define	BLAZER_XCTL_POST_EXT_EXTMEM 0x2
#define	BLAZER_XCTL_POST_EXT_NOMEM  0x1
#define	BLAZER_XCTL_POST_NORMAL	    0x0

#define BLAZER_DIAG_BDR_MEMTST	    0x4
#define BLAZER_DIAG_BDR_MEMFAIL	    0x5
#define BLAZER_DIAG_BDR_LOADOS	    0x8

#define BLAZER_FIXTURE_PRESENT		0x1

/*----------------------------------------------------------------------------
 * Blazer2
 * ---------------------------------------------------------------------------*/
/*Register Offset*/

#define BLAZER2_INT_STATUS_REG			0xB0
#define BLAZER2_INT_MASK_REG 			0xB4
#define BLAZER2_SPECIAL_REG			0xB8
#define BLAZER2_SPECIAL2_REG			0xB9
#define BLAZER2_REVISION_REG			0xBC
#define BLAZER2_BACKPLANE_CTRL_REG 		0xD0
#define BLAZER2_BACKPLANE_EXT_STATUS_REG 	0xD1
#define BLAZER2_BACKPLANE_STATUS_REG 		0xD4
#define BLAZER2_BACKPLANE_DIAG_REG 		0xD8
#define BLAZER2_BACKPLANE_EXT_CTRL_REG 		0xDC
#define BLAZER2_BACKPLANE_SUPPORT_REG		0xE0
#define BLAZER2_DEBUG_VAL_REG			0xFC

/* bit reserved in Software Restarts Register for CP warm boot */
#define BLAZER2_WARM_BOOT                   0x40
/*---------------------------------------------------------------------
 *
 * Icecube2
 *
 *--------------------------------------------------------------------*/

/* Register offset */

#define ICECUBE2_EXT_RESET_INHIBIT_REG		0xAD
#define ICECUBE2_EM_I2C_DIAG_REG                0xD0
#define ICECUBE2_SVB_PORT_ENABLE_A_REG		0xD2
#define ICECUBE2_SVB_PORT_ENABLE_B_REG		0xD4
#define ICECUBE2_SWM_POWER_CNTRL_REG		0xD6
#define ICECUBE2_SWM_HEALTH_STATUS_REG		0xD8
#define ICECUBE2_BAY_ID_REG			0xDA
#define ICECUBE2_DEBUG_REG                      0xFC

/*----------------------------------------------------------------------------
 *
 *  glacier2
 *
 * ---------------------------------------------------------------------------*/

/*Register Offset*/

#define GLACIER2_CPLD_REV_REG                     0x00
#define GLACIER2_RST_CAUSE_REG                    0x04
#define GLACIER2_CPLD_ID_REG                      0x08
#define GLACIER2_GEYE_INT_STATUS_REG              0x40
#define GLACIER2_GEYE_INT_MASK_REG                0x48
#define GLACIER2_MAX3784_ENB0_REG                 0x50
#define GLACIER2_MAX3784_ENB1_REG                 0x51
#define GLACIER2_MAX3784_MASK0_REG                0x52
#define GLACIER2_MAX3784_MASK1_REG                0x53
#define GLACIER2_FC_EN0_MMB_REG                   0x54
#define GLACIER2_FC_EN1_MMB_REG                   0x55
#define GLACIER2_GEYE_SW_RST_REG                  0x60
#define GLACIER2_GEYE_PU_RST_REG                  0x61
#define GLACIER2_GEYE_SW_RST_ENB_REG              0x64
#define GLACIER2_GEYE_PU_RST_ENB_REG              0x65
#define GLACIER2_SYS_RST_CTRL_REG                 0x68
#define GLACIER2_INDIVIDUAL_RST_CTRL_REG          0x69
#define GLACIER2_SYS_RST_ENB_REG                  0x6C
#define GLACIER2_INDIVIDUAL_RST_ENB_REG           0x6D
#define GLACIER2_GEYE_RST_ENB_REG                 0x88
#define GLACIER2_BOOTFLASH_W_ENB_REG              0x90
#define GLACIER2_IIC_SEEPROM_W_ENB_REG            0x98
#define GLACIER2_SYS_STATUS_LED_CTRL_REG          0xA0
#define GLACIER2_GEYE_PCI_RST_REG                 0xA4
#define GLACIER2_GEYE_PCI_RST_MASK_REG            0xA8
#define GLACIER2_SOFT_RESTART_REG                 0xAB
#define GLACIER2_INT_STATUS_REG                   0xB0
#define GLACIER2_INT_MASK_REG                     0xB4
#define GLACIER2_IIC_BUS_SEL_REG                  0xB8
#define GLACIER2_REV_REG                          0xBC
#define GLACIER2_MMB_CTRL_STATUS_AND_FC_ID_REG    0xD0
#define GLACIER2_MMB_IOX_REG                      0xD4
#define GLACIER2_TEST_FIXTURE_CTRL_REG            0xD6
#define GLACIER2_MIX_CTL_REG                      0xFB
#define GLACIER2_DEBUG_VALUE_REG                  0xFC


/*----------------------------------------------------------------------------
 *
 * superhawk
 *
 * ---------------------------------------------------------------------------*/

/*Register Offset*/

#define SUPERHAWK_CPLD_REV_REG                     0x00
#define SUPERHAWK_RST_CAUSE_REG                    0x04
#define SUPERHAWK_CPLD_ID_REG                      0x08
#define SUPERHAWK_GEYE_INT_STATUS_REG              0x40
#define SUPERHAWK_GEYE_INT_MASK_REG                0x48
#define SUPERHAWK_MAX3748_ENB0_REG                 0x50
#define SUPERHAWK_MAX3748_ENB1_REG                 0x51
#define SUPERHAWK_MAX3748_MASK0_REG                0x52
#define SUPERHAWK_MAX3748_MASK1_REG                0x53
#define SUPERHAWK_GEYE_SW_RST_REG                  0x60
#define SUPERHAWK_GEYE_PU_RST_REG                  0x61
#define SUPERHAWK_GEYE_SW_RST_ENB_REG              0x64
#define SUPERHAWK_GEYE_PU_RST_ENB_REG              0x65
#define SUPERHAWK_SYS_RST_CTRL_REG                 0x68
#define SUPERHAWK_INDIVIDUAL_RST_CTRL_REG          0x69
#define SUPERHAWK_SYS_RST_ENB_REG                  0x6C
#define SUPERHAWK_INDIVIDUAL_RST_ENB_REG           0x6D
#define SUPERHAWK_GEYE_RST_ENB_REG                 0x88
#define SUPERHAWK_BOOTFLASH_W_ENB_REG              0x90
#define SUPERHAWK_IIC_SEEPROM_W_ENB_REG            0x98
#define SUPERHAWK_SYS_STATUS_LED_CTRL_REG          0xA0
#define SUPERHAWK_GEYE_PCI_RST_REG                 0xA4
#define SUPERHAWK_GEYE_PCI_RST_MASK_REG            0xA8
#define SUPERHAWK_SOFT_RESTART_REG                 0xAB
#define SUPERHAWK_INT_STATUS_REG                   0xB0
#define SUPERHAWK_INT_MASK_REG                     0xB4
#define SUPERHAWK_IIC_BUS_SEL_REG                  0xB8
#define SUPERHAWK_REV_REG                          0xBC
#define SUPERHAWK_CMC_CTRL_STATUS_AND_IOM_ID_REG   0xD0
#define SUPERHAWK_CMC_SUP_REG                      0xD4
#define SUPERHAWK_I2C1_RST_REG                     0xD5
#define SUPERHAWK_TEST_FIXTURE_CTRL_REG            0xD6
#define SUPERHAWK_DEBUG_VALUE_REG                  0xFC

/*----------------------------------------------------------------------------
 *
 *  Dragon
 *
 * ---------------------------------------------------------------------------*/

/*Register Offset*/

#define DRAGON_CPLD_REV_REG				0x00
#define DRAGON_RST_CAUSE_REG				0x04
#define DRAGON_CPLD_ID_REG				0x08
#define DRAGON_BMC_REG					0x30
#define DRAGON_BMC_MASK_REG				0x31
#define DRAGON_GEYE_INT_STATUS_REG		0x40
#define DRAGON_GEYE_INT_MASK_REG			0x48
#define DRAGON_FC_RX_AMP_ENB0_REG		0x50
#define DRAGON_FC_RX_AMP_ENB1_REG		0x51
#define DRAGON_FC_RX_AMP_ENB_MASK0_REG	0x52
#define DRAGON_FC_RX_AMP_ENB_MASK1_REG	0x53
#define DRAGON_GEYE_SW_RST_REG			0x60
#define DRAGON_GEYE_PU_RST_REG			0x61
#define DRAGON_GEYE_SW_RST_ENB_REG		0x64
#define DRAGON_GEYE_PU_RST_ENB_REG		0x65
#define DRAGON_SYS_RST_CTRL_REG			0x68
#define DRAGON_INDIVIDUAL_RST_CTRL_REG	0x69
#define DRAGON_SYS_RST_ENB_REG			0x6C
#define DRAGON_INDIVIDUAL_RST_ENB_REG	0x6D
#define DRAGON_VOLT_MARGIN_ENB_REG		0x87
#define DRAGON_GEYE_RST_ENB_REG                 0x88
#define DRAGON_BOOTFLASH_W_ENB_REG		0x90
#define DRAGON_IIC_SEEPROM_W_ENB_REG	0x98
#define DRAGON_SYS_STATUS_LED_CTRL_REG	0xA0
#define DRAGON_GEYE_PCI_RST_REG			0xA4
#define DRAGON_GEYE_PCI_RST_MASK_REG	0xA8
#define DRAGON_SOFT_RESTART_REG			0xAB
#define DRAGON_INT_STATUS_REG				0xB0
#define DRAGON_INT_MASK_REG				0xB4
#define DRAGON_IIC_BUS_SEL_REG			0xB8
#define DRAGON_TEM_SENSOR_INT_STATUS_REG		0xBA
#define DRAGON_BMC_CTRL_STATUS_AND_FC_ID_REG	0xD0
#define DRAGON_BMC_IOX_REG				0xD4
#define DRAGON_CHASSIS_SEL_REG			0xF4
#define DRAGON_CHASSIS_TEST_SEL_REG		0xF5
#define DRAGON_DEBUG_VALUE_REG			0xFC
#define DRAGON_MIX_STATUS_REG			0xFD
#define DRAGON_BOOT_STATUS_REG			0xFE

/*
 * Common to Blazer, Dazzler, and other 405-based low cost systems.
 * Reset and Reset Enable registers and masks.
 */
#define CPLD_SYS_RESET_REG		0x68
#define	CPLD_CF_RESET			0x08

#define CPLD_RESET_ENABLE_REG		0x6C
#define	CPLD_CF_RESET_ENABLE		0x08


/*------------------------------------------------------------------------------
 * Forte
 *----------------------------------------------------------------------------*/

#define FORTE_CPLD_BASE_ADDR		0x00000001F8010000ULL
#define FORTE_CPLD_SIZE			0x00001000

#if 0
#define FORTE_BOOTKEYVAL_REG		0x00000040	/* flash wr enable */
#endif
#define FORTE_BOOTPROM_WRITE_ENB	0x00010000

#define FORTE_BP_CP_INTR_CTRL_REG	0x00000040	/* BP interrupts CP */
#define FORTE_CP_BP_INTR_CTRL_REG	0x00000050	/* CP interrupts BP */
#define FORTE_CP_BP_INTR_ACK_REG	0x00000058	/* BP acknowledges CP */
#define FORTE_BOOT_STATUS_REG		0x00000060
#define FORTE_BOOT_ERROR_REG		0x00000068
#define FORTE_CP_BP_INTR_MASK_REG	0x00000070
#define	FORTE_CP_BP_INTR_REAS_REG	0x00000078	/* CP to BP intr reason reg */

/*----------------------------------------------------------------------------
 * Saturn
 * - knj - Try to minimize these pound defines  - reuse them
 *   such definitions leads to duplication of C code
 *----------------------------------------------------------------------------*/
#define SATURN_CPLD_BASE_ADDR			0x1F8000000ULL	
#define SATURN_CPLD_IDE_BASE_ADDR		0x1F7000000ULL
#define SATURN_CPLD_SIZE			1024
#define SATURN_CPLD_IDE_SIZE			8192

#define SATURN_BOOTROM_CTRL_REG			0x25
#define SATURN_REV_ATA_CAP			0x02
#define SATURN_REV_ATA_RESET			0x04

/*------------------------------------------------------------------------------
 * Marathon
 *----------------------------------------------------------------------------*/

#define MARATHON_CPLD_BASE_ADDR		0x00000001F8010000ULL
#define MARATHON_CPLD_SIZE		0x00001000

#define MARATHON_BP_CP_INTR_CTRL_REG	0x00000040	/* BP interrupts CP */
#define MARATHON_CP_BP_INTR_CTRL_REG	0x00000050	/* CP interrupts BP */
#define MARATHON_CP_BP_INTR_ACK_REG	0x00000058	/* BP acknowledges CP */

#define MARATHON_BOOT_STATUS_REG	0x00000060
#define MARATHON_BP_CONSOLE_REG		0x00000060
#define MARATHON_BOOT_ERROR_REG		0x00000068
#define MARATHON_CP_BP_INTR_MASK_REG	0x00000070

/*------------------------------------------------------------------------------
 * Ironman
 *----------------------------------------------------------------------------*/

#define IRONMAN_CPLD_BASE_ADDR		0x00000001F8010000ULL
#define IRONMAN_CPLD_SIZE			0x00001000
#define IRONMAN_BP_CP_INTR_CTRL_REG	0x00000040	/* BP interrupts CP */
#define IRONMAN_CP_BP_INTR_CTRL_REG	0x00000050	/* CP interrupts BP */
#define IRONMAN_CP_BP_INTR_MASK_REG	0x00000058
#define IRONMAN_CP_BP_INTR_ACK_REG	0x00000060	/* BP acknowledges CP */
#define IRONMAN_BOOT_STATUS_REG		0x00000068
#define IRONMAN_BP_CONSOLE_REG		0x00000068
#define IRONMAN_BOOT_ERROR_REG		0x00000070
#define IRONMAN_BP_SETUP_REG        0x00000018
#define IRONMAN_BOOTKEYVAL_REG      0x00000040
#define IRONMAN_BOOTPROM_WR_EN      0x00010000
#define IRONMAN_SWR_EN              0x00000008
#define IRONMAN_PCI_CLK_LOCAL       0x00000010

/* ----------------------------------------------------------------------------
 * Elara
 * ---------------------------------------------------------------------------*/
#define ELARA_I2C_CONTROL_PATH_RST_REG          0x4C
#define ELARA_I2C_CONTROL_PATH_RST_EN_REG       0x4D
#define ELARA_I2C_MUX_RST                       0x10

/*
 * Register Mapping for Elara IIC Bus Flush Feature
 */

#define   ELARA_CPLD_IIC_FLUSH_CTRL     0x68
#define   ELARA_CPLD_BUS_FLUSH_ENABLE   0x01
#define   ELARA_CPLD_IIC_CONTROL        0x4E
#define   ELARA_CPLD_IIC_FLUSH_TIMEOUT  0x69
#define   FPGA_IIC_FLUSH                0x01
#define   ELARA_CPLD_IIC_FLUSH_STATUS   0x6A
#define   ELARA_CPLD_BUS_FLUSH_ERROR    0x02
#define   ELARA_CPLD_BUS_FLUSH_DONE     0x01

/* @kevin */
/* ----------------------------------------------------------------------------
 * Elara2/Elara-2
 * ---------------------------------------------------------------------------*/

#define ELARA2_I2C_CONTROL_PATH_RST_REG          0x4C
#define ELARA2_I2C_CONTROL_PATH_RST_EN_REG       0x4D
#define ELARA2_I2C_MUX_RST                       0x10

/*
 * Register Mapping for Elara2 IIC Bus Flush Feature
 */

#define   ELARA2_CPLD_IIC_FLUSH_CTRL     0x68
#define   ELARA2_CPLD_BUS_FLUSH_ENABLE   0x01
#define   ELARA2_CPLD_IIC_CONTROL        0x4E
#define   ELARA2_CPLD_IIC_FLUSH_TIMEOUT  0x69
#define   ELARA2_FPGA_IIC_FLUSH          0x01
#define   ELARA2_CPLD_IIC_FLUSH_STATUS   0x6A
#define   ELARA2_CPLD_BUS_FLUSH_ERROR    0x02
#define   ELARA2_CPLD_BUS_FLUSH_DONE     0x01

/* @kevin */
/* ----------------------------------------------------------------------------
 * Callisto
 * ---------------------------------------------------------------------------*/

#define CALLISTO_I2C_CONTROL_PATH_RST_REG        0x4C
#define CALLISTO_I2C_CONTROL_PATH_RST_EN_REG     0x4D
#define CALLISTO_I2C_MUX_RST                     0x10

/*
 * Register Mapping for Callisto IIC Bus Flush Feature
 */

#define   CALLISTO_CPLD_IIC_FLUSH_CTRL     0x68
#define   CALLISTO_CPLD_BUS_FLUSH_ENABLE   0x01
#define   CALLISTO_CPLD_IIC_CONTROL        0x4E
#define   CALLISTO_CPLD_IIC_FLUSH_TIMEOUT  0x69
#define   CALLISTO_FPGA_IIC_FLUSH          0x01
#define   CALLISTO_CPLD_IIC_FLUSH_STATUS   0x6A
#define   CALLISTO_CPLD_BUS_FLUSH_ERROR    0x02
#define   CALLISTO_CPLD_BUS_FLUSH_DONE     0x01

/* @kevin */
/* ----------------------------------------------------------------------------
 * Carme
 * ---------------------------------------------------------------------------*/

#define CARME_I2C_CONTROL_PATH_RST_REG        0x4C
#define CARME_I2C_CONTROL_PATH_RST_EN_REG     0x4D
#define CARME_I2C_MUX_RST                     0x10

/*
 * Register Mapping for Carme IIC Bus Flush Feature
 */

#define   CARME_CPLD_IIC_FLUSH_CTRL     0x68
#define   CARME_CPLD_BUS_FLUSH_ENABLE   0x01
#define   CARME_CPLD_IIC_CONTROL        0x4E
#define   CARME_CPLD_IIC_FLUSH_TIMEOUT  0x69
#define   CARME_FPGA_IIC_FLUSH          0x01
#define   CARME_CPLD_IIC_FLUSH_STATUS   0x6A
#define   CARME_CPLD_BUS_FLUSH_ERROR    0x02
#define   CARME_CPLD_BUS_FLUSH_DONE     0x01
/*
 * Register Mapping for Starlifter IIC Bus Flush Feature
 */
#define   STARLIFTER_CPLD_IIC_CTRL     	     0x68
#define   STARLIFTER_CPLD_IIC_MUX_SEL        0x10
#define   STARLIFTER_CPLD_IIC_MUX_ENABLE     0x01
#define   STARLIFTER_CPLD_IIC_FLUSH_CTRL     0x80
#define   STARLIFTER_CPLD_BUS_FLUSH_ENABLE   0x01
#define   STARLIFTER_CPLD_IIC_FLUSH_TIMEOUT  0x84
#define   STARLIFTER_FPGA_IIC_FLUSH          0x01
#define   STARLIFTER_CPLD_IIC_FLUSH_STATUS   0x88
#define   STARLIFTER_CPLD_BUS_FLUSH_ERROR    0x02
#define   STARLIFTER_CPLD_BUS_FLUSH_DONE     0x01


/* ----------------------------------------------------------------------------
 * Katana
 * ---------------------------------------------------------------------------*/

#define KATANA_I2C_CONTROL_PATH_RST_REG          0x6A
#define KATANA_I2C_CONTROL_PATH_RST_EN_REG       0x6E
#define KATANA_I2C_MUX_RST                       0x01

/*
 * Register Mapping for Katana IIC Bus Flush Feature
 */

#define   KATANA_BACKPLANE_SUPPORT_REG   0xE0
#define   KATANA_CPLD_IIC_FLUSH_CTRL     0xF8
#define   KATANA_CPLD_IIC_FLUSH_TIMEOUT  0xF9
#define   KATANA_CPLD_BUS_FLUSH_ERROR    0x04
#define   KATANA_CPLD_BUS_FLUSH_DONE     0x02
#define   KATANA_CPLD_BUS_FLUSH_ENABLE   0x01

/* ----------------------------------------------------------------------------
 * Harrier
 * ---------------------------------------------------------------------------*/

#define HARRIER_INDIVIDUAL_RST_REG         0x4C
#define HARRIER_INDIVIDUAL_RST_EN_REG      0x4D
#define HARRIER_I2C_SFP_MUX_RST            0x10
#define HARRIER_HEALTH_STATUS_REG          0xdc

/*
 * Register Mapping for Harrier IIC Bus Flush Feature
 */

#define   HARRIER_IIC_BUS_SELECT_CTRL    0xB8
#define   HARRIER_IIC_BUS_SHARED         1
#define   HARRIER_IIC_BUS_PRIVATE        0
#define   HARRIER_IIC_BUS_SHARED_nRDY    0x02
#define   HARRIER_IIC_BUS_PRIVATE_nRDY   0x04

#define   HARRIER_CPLD_IIC_FLUSH_CTRL    0xF8
#define   HARRIER_CPLD_IIC_FLUSH_TIMEOUT 0xF9

#define   HARRIER_CPLD_BUS_FLUSH_ENABLE  0x01
#define   HARRIER_CPLD_BUS_FLUSH_OK      0x04
#define   HARRIER_CPLD_BUS_FLUSH_DONE    0x02

/* ----------------------------------------------------------------------------
 * Spike
 * ---------------------------------------------------------------------------*/
#define SPIKE_SYS_RESET_ENABLE_REG  0x4D
#define SPIKE_SYS_RESET_CONTROL_REG  0x4C

/*
 * 0x4C - System Reset Control Register
 */
#define SPIKE_SYSTEM_SHUTDOWN_ENABLE		0x10
/*
 * 0x4D - System Reset Enable Register
 */
#define SPIKE_SYSTEM_RESET_ENABLE		0x10

/* ----------------------------------------------------------------------------
 * Europa
 * ---------------------------------------------------------------------------*/

#define EUROPA_CPLD_BASE_ADDR 0x1F8000000ULL
#define EUROPA_CPLD_SIZE 65536

#define EUROPA_IIC_RESET_ENABLE_REG 0x24
#define EUROPA_IIC_RESET_CNTRL_REG  0x1C

#define EUROPA_IIC_MUX_RESET        0x80
#define EUROPA_IIC_RESET            0x40
/*------------------------------------------------------------------------------
 * Spirit
 *----------------------------------------------------------------------------*/

 /* Register Offset */
#define SPIRIT_LLI_CPLD_REV_REG			0x00000017
#define SPIRIT_SFP0_7_PWREN_REG			0x00000038
#define SPIRIT_SFP8_15_PWREN_REG		0x00000039
#define SPIRIT_SFP16_23_PWREN_REG		0x0000003A
#define SPIRIT_SFP24_31_PWREN_REG		0x0000003B
#define SPIRIT_SFP0_7_PWRFAULT_INT_REG		0x00000050
#define SPIRIT_SFP8_15_PWRFAULT_INT_REG		0x00000051
#define SPIRIT_SFP16_23_PWRFAULT_INT_REG	0x00000052
#define SPIRIT_SFP24_31_PWRFAULT_INT_REG	0x00000053
#define SPIRIT_SFP0_7_PWRFAULT_INTMASK_REG	0x00000058
#define SPIRIT_SFP8_15_PWRFAULT_INTMASK_REG	0x00000059
#define SPIRIT_SFP16_23_PWRFAULT_INTMASK_REG	0x0000005A
#define SPIRIT_SFP24_31_PWRFAULT_INTMASK_REG	0x0000005B
#define SPIRIT_FRUA_PRESENT_REG			0x00000070
#define SPIRIT_FRUB_PRESENT_REG			0x00000072

#define SPIRIT_PS_PRESENT_REG			0x00000070
#define SPIRIT_PS_LED_CNTRL_REG			0x00000071
#define SPIRIT_POWER_SUPPLY_BAD_REG		0x00000078
#define SPIRIT_FAN_SPEED_CNTRL_REG		0x000000AC
#define SPIRIT_FAN_TACH_MUX_REG			0x000000AD

#define SPIRIT_POWER_SUPPLY_BAD_REG		0x00000078
#define SPIRIT_ASIC0_EFUSE_REG                  0x00000079
#define SPIRIT_VOLT_MARGIN_REG			0x00000089
#define SPIRIT_P1V8_FAULT_REG                   0x000000F1
#define SPIRIT_FAN_SPPED_CNTRL_REG              0x000000AC
#define SPIRIT_DEBUG_REG                        0x000000FC
#define SPIRIT_PORT0_LLI_REG                    0x00000200
#define SPIRIT_PORT_LLI_REGSIZE                 4

#define CPLD_SPRT_AMBER_LED                     0x02
#define CPLD_SPRT_GREEN_LED                     0x00
#define CPLD_SPRT_BLINK_LED                     0x20


/*------------------------------------------------------------------------------
 * LittleBear 8G
 *----------------------------------------------------------------------------*/
/* Register Offset */
#define LITTLEBEAR_8G_FC_RXAMP_DISABLE_REG	0x50
#define LITTLEBEAR_8G_VOLT_MARGIN_0_REG		0x89
#define LITTLEBEAR_8G_VOLT_MARGIN_1_REG		0x8A
#define LITTLEBEAR_8G_LED_POWER_CNTRL_REG       0x9A
#define LITTLEBEAR_8G_HP_LED_STATUS_REG		0xAC
#define LITTLEBEAR_8G_SW_HEALTH_STATUS_REG	0xAD
#define LITTLEBEAR_8G_OSC_POWER_STATUS_REG	0xAE
#define LITTLEBEAR_8G_SLOT_ID_REG		0xAF

#define LB8G_SFP_I2C_MUX_RESET_ENABLE		0x80
#define LB8G_I2C_PWR_RESET_ENABLE		0x40
/*
 * Common all platforms running Linux
 */

#define CPLD_IMAGE_REVISION	(CPLD_BASE_ADDRESS | 0x1F)
#define CPLD_MAX_REGISTERS	0x100

#define CPLD_REG_VAL_SET	0xFF		/* all bits set in register */
#define CPLD_REG_VAL_CLEAR	0x00		/* all bits cleared in reg */

/* The CPLD_IDE_BASE_ADDR are for newer LC silkworm platforms */
#define CPLD_IDE_BASE_ADDR    0xF8F00000
#define CPLD_IDE_SIZE         1024

#define CPLD_IDE_REGISTER_FILE1	    0x1F0           /* 8 registers */
#define CPLD_IDE_REGISTER_FILE2	    0x3F6           /* 2 registers */

/*
 * Registers for managing the IIC BUS FLUSH feature introduced
 * in the Zentron CP
 */
#define CPLD_IIC_SOURCE_SELECT			0x42
#define CPLD_CPJRPLUS_IIC_SOURCE_SELECT	0x62	/* XXX to be implemented in hw */
#define THOR_CPLD_IIC_SOURCE_SELECT		0x72
#define   CPLD_CPU_SOURCE			0x00 /* Source is the CPU IIC controller */
#define   CPLD_EXTERNAL_SOURCE 		0x01 /* Source is external master device */
#define   CPLD_FPGA_SOURCE			0x02 /* Source is CPLD "bus-flush" HW */

#define CPLD_IIC_FLUSH_CTRL				0x48
#define CPLD_CPJRPLUS_IIC_FLUSH_CTRL	0x68
#define THOR_CPLD_IIC_FLUSH_CTRL		0x21 // Thor
#define	  CPLD_BUS_FLUSH_ENABLE	0x01

#define CPLD_IIC_FLUSH_TIMEOUT			0x49
#define CPLD_CPJRPLUS_IIC_FLUSH_TIMEOUT	0x69

#define CPLD_IIC_FLUSH_STATUS			0x4A
#define CPLD_CPJRPLUS_IIC_FLUSH_STATUS	0x6A
#define THOR_CPLD_IIC_FLUSH_TIMEOUT		0x20 // Thor
#define THOR_CPLD_IIC_FLUSH_STATUS		0x22 // Thor
#define	  CPLD_BUS_FLUSH_ERROR		0x01
#define	  CPLD_BUS_FLUSH_DONE		0x02

/*
 * PCI RAS device definitions. Introduced with the Zentron CP
 */

#define PCI_RAS_PLD_BASE_ADDR		0xF8400000
#define 	PCI_TIMEOUT_ADDR0		0x32
#define 	PCI_TIMEOUT_ADDR1		0x33
#define 	PCI_TIMEOUT_ADDR2		0x34
#define 	PCI_TIMEOUT_ADDR3		0x35
#define 	PCI_TIMEOUT_CBE			0x36
#define PCI_RAS_PLD_SIZE			128

#define CPLD_MISC_INTERRUPT			0x6E
#define		PCI_TIMEOUT_IRQ_BRIDGES	0x38

/*------------------------------------------------------------------------------
 * Starlifter Definitions
 *----------------------------------------------------------------------------*/
#define STARLIFTER4040_BOOT_WDT_CTRL		0x0124
#define STARLIFTER4040_BOOT_WDT_CTR		0x00dc
#define STARLIFTER4040_BOOT_FLASH_WRITE_CTRL	0x002c
#define STARLIFTER4040_BASE_ADDRESS		0xfec000000ull
#define STARLIFTER4040_MAX_REGISTERS		0x10000

#define STARLIFTER_SYS_RESET_ENABLE_REG     	0x78
#define STARLIFTER_SYS_RESET_SHUTDOWN_REG   	0x74

#define STARLIFTER_SYSTEM_SHUTDOWN_ENABLE   	0x01
#define STARLIFTER_SYSTEM_SHUTDOWN          	0x01

/*------------------------------------------------------------------------------
 * Skybolt  Definitions (Sucessive platforms too use these definitions)
 *----------------------------------------------------------------------------*/
#define SKYBOLT_BOOT_FLASH_WRITE_CTRL	0x0B
#define SKYBOLT_BOOT_WDT_CTRL		0x78 // 32 bit reg
#define SKYBOLT_BOOT_WDT_CTR 		0x54 // 32 bit reg
#define SKYBOLT_CPLD_BASE_ADDR		0xfec000000ull
#define SKYBOLT_MAX_REGISTERS		0x10000

#define SKYBOLT_SYS_RESET_ENABLE_REG     0x01C
#define SKYBOLT_SYS_RESET_SHUTDOWN_REG   0x01B

#define SKYBOLT_SYSTEM_SHUTDOWN_ENABLE   0x01
#define SKYBOLT_SYSTEM_SHUTDOWN          0x01

/*
 * Skybolt CPLD I2C reg definitions
 *
 */
#define   SKYBOLT_CPLD_IIC_CTRL		  0x1A
#define   SKYBOLT_CPLD_IIC_MUX_ENABLE     0x01
#define   SKYBOLT_CPLD_IIC_MUX_SEL        0x02
#define   SKYBOLT_CPLD_IIC_FLUSH_CTRL     0x01D
#define   SKYBOLT_CPLD_BUS_FLUSH_ENABLE   0x01
#define   SKYBOLT_CPLD_IIC_FLUSH_TIMEOUT  0x01E
#define   SKYBOLT_FPGA_IIC_FLUSH          0x01
#define   SKYBOLT_CPLD_IIC_FLUSH_STATUS   0x01F
#define   SKYBOLT_CPLD_BUS_FLUSH_ERROR    0x02
#define   SKYBOLT_CPLD_BUS_FLUSH_DONE     0x01

/*------------------------------------------------------------------------------
 * Castor Definitions
 *----------------------------------------------------------------------------*/
#define CASTOR_BOOT_FLASH_WRITE_CTRL	0x0010
#define CASTOR_CPLD_IIC_CTRL			0x0048
#define CASTOR_BOOT_WDT_CTRL			0x0148
#define CASTOR_BOOT_WDT_CTR 			0x0154
#define CASTOR_CPLD_BASE_ADDR			0xfec000000ull
#define CASTOR_MAX_REGISTERS			0x10000

#define CASTOR_SYS_RESET_ENABLE_REG     0x58
#define CASTOR_SYS_RESET_SHUTDOWN_REG   0x54

#define CASTOR_SYSTEM_SHUTDOWN_ENABLE   0x01
#define CASTOR_SYSTEM_SHUTDOWN          0x01

/*
 * Castor CPLD I2C reg definitions
 *
 */

#define   CASTOR_CPLD_IIC_FLUSH_CTRL     0x68
#define   CASTOR_CPLD_BUS_FLUSH_ENABLE   0x01
#define   CASTOR_CPLD_IIC_FLUSH_TIMEOUT  0x6C
#define   CASTOR_FPGA_IIC_FLUSH          0x01
#define   CASTOR_CPLD_IIC_FLUSH_STATUS   0x70
#define   CASTOR_CPLD_BUS_FLUSH_ERROR    0x02
#define   CASTOR_CPLD_BUS_FLUSH_DONE     0x01
#define   CASTOR_CPLD_IIC_MUX_SEL        0x10



/*--------------------------------------------
 * Wedge Definitions
 *--------------------------------------------*/
#define WEDGE_BOOT_FLASH_WRITE_CTRL	0x0B
#define WEDGE_BOOT_WDT_CTRL		0x45 // 32 bit reg
#define WEDGE_BOOT_WDT_CTR 		0x4F

#define	WEDGE_CPLD_IIC_CTRL				0x18
#define WEDGE_CPLD_IIC_MUX_ENABLE		0x01
#define WEDGE_CPLD_IIC_MUX_SEL			0x10
#define WEDGE_CPLD_IIC_FLUSH_CTRL		0x20
#define WEDGE_CPLD_BUS_FLUSH_ENABLE		0x01
#define WEDGE_CPLD_IIC_FLUSH_TIMEOUT	0x21
#define WEDGE_FPGA_IIC_FLUSH			0x01
#define WEDGE_CPLD_IIC_FLUSH_STATUS		0x22
#define WEDGE_CPLD_BUS_FLUSH_ERROR		0x02
#define WEDGE_CPLD_BUS_FLUSH_DONE		0x01


/*------------------------------------------------------------------------------
 * Mercury Definitions
 *----------------------------------------------------------------------------*/

/*
 * These are 32-bit addresses. Later these will change when
 * we switch to 36-bit addresses.
 */

#define MM_BASE_ADDRESS		0xfec000000ull
#define MM_MAX_REGISTERS	0x10000
#define MM_FPGA_VERSION		0x000000
#define MM_BOOT_WDT_CTRL	0x00022C
#define MM_BOOT_WDT_CTR		0x000230

#define LC_BASE_ADDRESS		0xfec000000ull
#define LC_MAX_REGISTERS	0x10000
#define LC48X10G_FPGA_VERSION	0x000200
#define LC48X10G_BOOT_WDT_CTRL	0x000094
#define LC48X10G_BOOT_WDT_CTR	0x00009c

#define MM_ID_REG	0x0218
#define 	MM_SLOTID_MASK	0x03
#define		MM_SLOTID_SHIFT	8
#define 	MM_BPID_MASK	0x0F

#define LC_MISC_SR	0x0078
#define		LC_SLOTID_MASK	0x1F
#define		LC_SLOTID_SHIFT	2
#define		LC_BPID_MASK	0x0F
#define		LC_BPID_SHIFT	7
#define		LC_BPREV_MASK	0x07
#define		LC_BPREV_SHIFT	11

#define MM_MAX_IRQS 12
#define PB_MAX_IRQS 12

/*------------------------------------------------------------------------------
 * DCX (Yoda) Definitions
 *----------------------------------------------------------------------------*/
#define DCX_CPLD_BOOTFLASH_CTRL	(0x25)
#define DCX_BOOT_WDT_CTR		(0xB8)

/*------------------------------------------------------------------------------
 * Carbon Definitions
 *----------------------------------------------------------------------------*/
#define CARBON_CPLD_BOOTFLASH_CTRL        (0x0090)
#define CARBON_CPLD_BOOTFLASH_WRITE_BITS  (0x01) /* 1 = allow, 0 = disallow */
#define CARBON_CPLD_BACKPLANE_SUPPORT     (0x00E0)
#define CARBON_CPLD_SPI_CONTROL_BITS      (0x10) /* VPD owner: 1=CANmic, 0=CP */
#define CARBON_CPLD_SWITCH_BAY_ID         (0x00B5)
#define CARBON_CPLD_BAY_ID_BITS           (0x07) /* 0: TEST IO Card, 1~6: ID */
#define CARBON_CPLD_BAY_ID_VAL_TESTIO     (0)
#define CARBON_CPLD_BOOT_WDT_CTR          (0x0074) /* WDT counter */
#define CARBON_CPLD_BOOT_WDT_CTRL_MSB     (0x0078) /* WDT control (MSB) */
#define CARBON_CPLD_BOOT_WDT_CTRL_LSB     (0x007C) /* WDT control (LSB) */

/*------------------------------------------------------------------------------
 * Redbird Definitions
 *----------------------------------------------------------------------------*/
#define REDBIRD_CPLD_BOOTFLASH_CTRL        (0x0090)
#define REDBIRD_CPLD_BOOTFLASH_WRITE_BITS  (0x01) /* 1 = allow, 0 = disallow */

/*------------------------------------------------------------------------------
 * Shenlong Definitions
 *----------------------------------------------------------------------------*/
#define SHENLONG_CPLD_BOOTFLASH_CTRL        (0x0090)
#define SHENLONG_CPLD_BOOTFLASH_WRITE_BITS  (0x01) /* 1 = allow, 0 = disallow */


extern int cpld_arbitrate_i2c_mux(void);
extern int cpld_release_i2c_mux(int force);
extern int cpld_set_i2c_mux_local(void);
extern int cpld_set_i2c_mux(int use_slot_id, char data);
extern int cpld_get_i2c_mux(char *data);
extern int cpld_read_register(int offset, unsigned char *data);
extern int cpld_write_register(int offset, unsigned char data);
extern int cpld_read_register32(int offset, unsigned int *data);
extern int cpld_write_register32(int offset, unsigned int data);
extern void cpld_spi_mux( char *spi_device_name, int select_cpu );

/* The offset of USB intr register in CPLD */
#define ELARA2X_CPLD_USB_INTR_MASK_REG   0x46
#define CALLISTOX_CPLD_USB_INTR_MASK_REG 0x46
#define CARME_CPLD_USB_INTR_MASK_REG 0x46

/* Switch independent USB intr control routine */
extern int silkworm_mask_usb_intr(void);
extern int silkworm_unmask_usb_intr(void);

#endif /* _BROCADE_CPLD_H */
