/*
 * Brocade platform support for all the Silkworm products
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 or the License, or (at your
 * option) any later version.
 */

#ifdef __KERNEL__
#ifndef __SILKWORM_BHPC_H_
#define __SILKWORM_BHPC_H_

#include <linux/pci.h>
#include <linux/mod_devicetable.h>

/* XXX Remove these once C3 support is merged in */
#define PCI_VENDOR_C3 0x1657U 
#define PCI_DEVICE_ID_C3 0x1d

/* PEX8606 PCIe switch definitions */
#define PEX8606_DEVICE_ID			0x8606
#define PEX8606_DEBUG_CTRL			0x1DC
#define PEX8606_SW_CONFIG_CTRL		0x8000
#define PEX8606_LANE_STATUS			0x1F4
#define PEX8606_EVEN_PORTS_CTRL		0x230
#define PEX8606_NTP_SHIFT			24
#define PEX8606_PORT0_USP_PORT4_NTP 0x04000000
#define PEX8606_PORT4_USP_PORT0_NTP 0x00000400
#define PEX8606_PORT_CONFIG_MASK	0x0F000F00
#define PEX8606_TOTAL_CSR_SIZE		0x00020000	/* 128K */
#define PEX8606_PORT_CSR_SIZE		0x00001000	/* 4k */
#define PEX8606_NT_MODE_ENABLED		0x00040000
#define PEX8606_DISABLE_SEEPROM_LOAD	0x00020000
#define PEX8606_PORT0_OFFSET 		0x0000
#define PEX8606_PORT1_OFFSET 		0x1000
#define PEX8606_PORT4_OFFSET 		0x4000
#define PEX8606_PORT5_OFFSET 		0x5000
#define PEX8606_PORT7_OFFSET 		0x7000
#define PEX8606_PORT9_OFFSET 		0x9000
#define PEX8606_LANE0_STATUS		0x01	/* Lane 0 is port 0 */
#define PEX8606_LANE1_STATUS		0x02	/* Lane 1 is port 4 */
#define PEX8606_PORT0_DISABLE_MASK	0x01
#define PEX8606_PORT4_DISABLE_MASK	0x04

#define BHPC_DISABLE_OMM_SERR		1
#define BHPC_SHOW_SFM_LINKS			2
#define BHPC_RESET_OMM_PORT			3

/* PEX8664 PCIe switch definitions */
#define PEX8664_DEVICE_ID			0x8664
#define PEX8664_DEBUG_CTRL			0x350
#define PEX8664_VS0					0x360
#define PEX8664_SW_CONFIG_CTRL		0x200
#define PEX8664_NTP_SHIFT			8
#define PEX8664_PORT0_USP_PORT1_NTP 0x100
#define PEX8664_PORT1_USP_PORT0_NTP 0x001
#define PEX8664_PORT_CONFIG_MASK 0x303 /* Covers ports 0-8 - all we care about */

/* PEX8713 PCIe switch definitions */
#define PEX8713_DEVICE_ID			0x8713
#define PEX8713_DEVICE_NTP1_ID		0x87A1
#define PEX8713_DEVICE_NTP0_ID      0x87A0
#define PEX8713_VS0_FAILOVER		0xF00
#define PEX8713_PORT_MASK			0x1F
#define PEX8713_NT0_SHIFT			8
#define PEX8713_NT1_SHIFT			16
#define PEX8713_NT0_MASK			0x00001F00
#define PEX8713_NT1_MASK			0x001F0000
#define PEX8713_VS_DEBUG			0xA30
#define PEX8713_DISABLE_SEEPROM_LOAD	0x08
/*
 * Undocumented register. Used to enable SW to change the VSO Failover register
 */
#define PEX8713_SW_CTRL				0x350
#define PEX8713_ALLOW_SW_CTRL			0x200

#define BHPC_SFM_MM0_PORT	0	/* MM0 connects to SFM switch port 0 */
#define BHPC_SFM_MM1_PORT	4	/* MM1 connects to SFM switch port 4 */
#define BHPC_MAX_CHASSIS_SLOTS 29 /* Allow for slot 0 to be NULL */

#define BHPC_FAILURE		-1
#define BHPC_SUCCESS		0
#define BHPC_DSP_LINK_DOWN	1

/*
 * PCIe-PCI bridge definitions. Secondary side has
 * subset of AER registers
 */

#define PCI_SEC_ERR_UNCOR_STATUS	0x2C	/* Uncorrectable Error Status */
#define PCI_SEC_ERR_UNCOR_MASK		0x30	/* Uncorrectable Error Mask */
#define PCI_SEC_ERR_UNCOR_SEVER		0x34	/* Uncorrectable Error Severity */
#define PCI_SEC_ERR_CAP				0x38	/* Advanced Error Capabilities */
#define PCI_SEC_ERR_HEADER_LOG		0x3C	/* Header Log Register (16 bytes) */

/*
 * PCI NTB definitions
 */
#define PCI_TYPE_NTB				0x100
#define PCI_NTB_CFG_ADDR			0x80
#define PCI_NTB_CFG_DATA			0x84
#define PCI_NTB_SECONDARY_STATUS	0x44
#define PCI_NTB_CHIP_CONTROL		0xcc
#define PCI_NTB_CHIP_STATUS			0xd0
 
struct slot;


/*
 * Error Interrupt Handling data structures
 */


#define BHPC_RC_IDX			0
#define BHPC_MM_USP_IDX		1
#define BHPC_MM_DSP_IDX		2
#define BHPC_ENDPOINT_IDX	3
#define BHPC_PCIE_PCI_IDX	4

#define BHPC_ERRDR_IDX		0
#define BHPC_ERRDR_PCT_IDX	1
#define BHPC_ERRDR_PAT_IDX	2
#define BHPC_ERRDR_PCAC_IDX	3
#define BHPC_ERRDR_LDDE_IDX	4
#define BHPC_ERRDR_PNM_IDX	5

#define BHPC_PME_IDX		0
#define BHPC_PME_PTO_IDX	1
#define BHPC_PME_ENL23_IDX	2
#define BHPC_PME_EXL23_IDX	3
#define BHPC_PME_HRD_IDX	4
#define BHPC_PME_LDD_IDX	5
#define BHPC_PME_LUD_IDX	6

#define BHPC_UNDETERMINED_IDX	0
#define BHPC_SPAERR_IDX			1
#define BHPC_STA_IDX			2
#define BHPC_RTA_IDX			3
#define BHPC_RMA_IDX			4
#define BHPC_SSERR_IDX			5
#define BHPC_MPAERR_IDX			6
#define BHPC_SDE_IDX			7
#define BHPC_CTO_IDX			8



#define BHPC_ERR_PCIE_NONE		0
#define BHPC_ERR_PCIE_TA		1
#define BHPC_ERR_PCIE_MA		2
#define BHPC_ERR_PCIE_SD		3
#define BHPC_ERR_PCIE_SERR		4
#define BHPC_ERR_PCIE_UNKNOWN	5



#define BHPC_MAX_HEADER_LOG		4
#define BHPC_INTERRUPT_SOURCE	1
#define BHPC_INIT_SOURCE		2
#define BHPC_PROC_SOURCE		3

#define BHPC_PCI_ERR_MASK		(PCI_STATUS_PARITY | PCI_STATUS_SIG_TARGET_ABORT |\
								 PCI_STATUS_REC_TARGET_ABORT |\
								 PCI_STATUS_REC_MASTER_ABORT |\
								 PCI_STATUS_SIG_SYSTEM_ERROR |\
								 PCI_STATUS_DETECTED_PARITY)

#define BHPC_ERROR_COR		0x0003
#define BHPC_ERROR_NON_COR	0x000C

typedef enum {

	PCIE_ERROR_NONE = 				0,
	PCIE_ERROR_UNKNOWN = 			1,
	PCIE_ERROR_CORR_ERR =			2,
	PCIE_ERROR_UNCORR_ERR =			3,
	PCIE_ERROR_RC_UNCORR_ERR_SRC =	4,
	PCIE_ERROR_RC_UNCORR_ERR_HLOG =	5,
	PCIE_ERROR_RC_CORR_ERR_SRC =	6,
	PCIE_ERROR_RC_CORR_ERR_HLOG =	7,
	PCIE_ERROR_LEGACY_ERR =			8,
	PCIE_ERROR_ERR_DETECT =			9,
	PCIE_ERROR_PME_DETECT =			10,

} bhpc_pcie_error_type;

typedef enum {

	PCIE_EVENT_NONE =		0x00000000,
	PCIE_EVENT_RTA =		0x00000001,
	PCIE_EVENT_STA =		0x00000002,
	PCIE_EVENT_MA =			0x00000004,
	PCIE_EVENT_SD =			0x00000008,
	PCIE_EVENT_RSERR =		0x00000010,
	PCIE_EVENT_SSERR =		0x00000020,
	PCIE_EVENT_CTO =		0x00000040,
	PCIE_EVENT_PPAERR =		0x00000080,
	PCIE_EVENT_SPAERR =		0x00000100,
	PCIE_EVENT_LDDE =		0x00000200,
	PCIE_EVENT_LDD =		0x00000400,
	PCIE_EVENT_LUD =		0x00000800,

	/* New defintions show go above this line */
	PCIE_EVENT_UNKNOWN = 	0x80000000

} bhpc_pcie_event_type;

typedef struct _bhpc_dev_id {

	u16 domain;
	u16 bus;
	u8  dev;
	u8  func;

} bhpc_dev_id;

/*
 * Used to hold the PCIe controller memory-mapped
 * PME registers. These can generate PCIe error interrupts
 */
typedef struct _bhpc_pcie_pme {

	u32 valid;
	u32 config;
	u32 detect;
	u32 enable;
	u32 disable;
	u32 command;

} bhpc_pcie_pme;

/*
 * Used to hold the PCIe controller memory-mapped error
 * control/status registers
 */
typedef struct _bhpc_pcie_controller {

	u32 valid;
	u32 config_addr;
	u32 detect;
	u32 enable;
	u32 disable;
	u32 capture_status;
	u32 capture_data[4];

} bhpc_pcie_controller;

/*
 * Used to hold the PCI/PCIe config registers
 * associated with error control, status, and reporting
 */
typedef struct _bhpc_dev_error {

	u32 valid;		/* True means this instance has valid data */
	u32 dev_type;	/* RC/USP/DSP/ENDPOINT */
	u32 event_detected;	/* True means this device detected an event */

	struct pci_dev *dev;

	bhpc_pcie_event_type event_type; /* Type of events this dev has reported */
	u32 primary_status;
	u32 secondary_status;
	u32 bridge_control;
	u32 device_status;
	u32 link_control;
	u32 slot_control;
	u32 root_control;		/* RC only */
	u32 root_status;		/* RC only */
	u32 uncorrectable_status;
	u32 uncorrectable_mask;
	u32 uncorrectable_sev;
	u32 correctable_status;
	u32 correctable_mask;
	u32 root_err_cmd;		/* RC only */
	u32 root_err_status;	/* RC only */
	u32 corr_err_src;		/* RC only */
	u32 uncorr_err_src;		/* RC only */
	u32 aer_cap;
	u32 header_log[BHPC_MAX_HEADER_LOG];
	u32 mem_window;		/* Mem Limit/Base registers */
	u32 rc_ltssm;		/* Link Training & status state machine */

	u32 sec_uncorrectable_status;
	u32 sec_uncorrectable_mask;
	u32 sec_uncorrectable_sev;
	u32 sec_aer_cap;
	u32 sec_header_log[BHPC_MAX_HEADER_LOG];

} bhpc_dev_error;

#define MAX_DEV_ERROR 32

/*
 * PCIe error data. Contains everything that could be
 * determined from the various error status registers
 * along with the register contents from the RC,
 * MM USP, MM DSP, and (optionally) the endpoint
 * device.
 *
 */
typedef struct _bhpc_error_data {

	u32 message_source; /* interrupt/init/proc */
	int err_irq;

	/* domain:bus:dev ID of the source device generating the event */
	bhpc_dev_id src_dev;

	struct pci_dev *mm_dsp_dev; /* pci_dev object of the parent MM DSP */

	/* domain:bus:dev ID of the parent MM DSP */
	bhpc_dev_id mm_parent_dev;

	/* Logical slot number of the board contaning the source device */
	u32 slot;

	u32 error_idx;

	u32 error_detected;  /* Indicates an error has been detected */
	bhpc_pcie_error_type src_decode; /* Indicates how the err src was identified */

	/* Contains the PCIe controller memory-mapped error detect registers */
	bhpc_pcie_controller pex;

	/* Contains the PCIe controller memory-mapped PME registers */
	bhpc_pcie_pme pme;

	/*
	 * Contains the error control/status config registers of the RC, MM USP,
	 * MM DSP, and (optionally) the Endpoint
	 */
	bhpc_dev_error dev_err[MAX_DEV_ERROR];

	/*
	 * PCI domain. Each PCI controller is a separate PCI domain.
	 * Bus/Device numbering starts from 0 in each domain, so to
	 * fully identify a device requires domain/bus/device numbers.
	 */
	int domain; 

} bhpc_error_data;

typedef struct _bhpc_ops {
	int (*power_on_slot)(int slot);
	int (*power_off_slot)(int slot);
	int (*get_power_status)(int slot, u8 *status);
	int (*get_attention_status)(int slot, u8 *status);
	int (*set_attention_status)(int slot, u8 status);
	int (*get_latch_status)(int slot, u8 *status);
	int (*get_adapter_status)(int slot, u8 *status);
	int (*query_power_fault)(int slot);
} bhpc_ops;

typedef struct _bhpc_slot_map {
    uint8_t slot;
    uint8_t port;
	uint32_t refcount; /* For the blade bridge */
    void  *private;
	int domain;
} bhpc_slot_map;

typedef struct _bhpc_slot_map_table {
    uint8_t max_slots;
    bhpc_slot_map slots[BHPC_MAX_CHASSIS_SLOTS];
} bhpc_slot_map_table;

/*
 * Each DSP provides a range of addresses (memory or bus) for all of
 * its downstream devices. If the DSP connects to chassis PCI slots via
 * a PCIe-PCI bridge, then the range is subdivided into a subrange per
 * chassis slot. This is done so that the slot causing a PCI error can
 * be identified base on the address that was being accessed.
 */

#define BHPC_MAX_DSP_SUBRANGES 4 /* Max of 4 chassis PCI slots per DSP */
typedef struct _subrange {

	u8 slot;	/* Chassis slot number */
	u8 devid;	/* Blade bridge device ID */
	u32 idsel;	/* Blade bridge Idsel */
	u32 base;	/* Address (mem or bus) base value */
	u32 limit;  /* Address (mem or bus) limit value */

} subrange;

typedef struct _dsp_subranges {
	u8 number;
	subrange subranges[BHPC_MAX_DSP_SUBRANGES];
} dsp_subranges;

/*
 * Store MM RC and PCIe switch port information for
 * use during PCIe error interrupt handling:
 *  - domain/bus/dev 	 - Port Device ID
 *  - first_bus/last_bus - defines range of busses downstream of this port
 *  - mem_base/mem_limit - defines the memory window provided by this
 *                         switch port. Used to determine error source
 *                         device based on captured memory address
 *  - slot  			 - Logical slot number this port connects to.
 *  - idsel              - IDSEL number of the device. PCI devices only
 *  - pci_dev            - object of the port.
 *  - subranges          - Table of mem_base/mem_limit subranges per slot
 *                         to be used for PCI slots.
 */
#define BHPC_MAX_DSP 32
typedef struct _dsp_dev_map {
	u32 domain;
	u32 bus;
	u32 dev;
	u32 first_bus;
	u32 last_bus;
	u32 mem_base;
	u32 mem_limit;
	u32 slot;
	struct pci_dev *pdev;
	u32 idsel;
	dsp_subranges mem_subranges;
	dsp_subranges bus_subranges;

} dsp_dev_map;

typedef struct _bhpc_dsp_dev_map_table {
	uint8_t base_bus;
	uint8_t max_entries;
	dsp_dev_map dsp_map[BHPC_MAX_DSP];
} bhpc_dsp_map_table;

typedef struct _bhpc_platform {
	int enable_irqs;
    bhpc_slot_map_table *slot_map;
    void (*log_fn)(int log_id, int slot, int errlevel, const char *format, ...);
    void (*trace_fn)(int log_id, int slot, int narg, ...);
	void (*error_fn)(bhpc_error_data *ddata);
    bhpc_ops *plat_ops;
    struct pci_device_id *aer_devices;
	bhpc_dsp_map_table *dsp_map;
} bhpc_platform;

typedef struct _bhpc_svc_ops {

   int (*initialize)(void);
   int (*remove)(void);
   int (*blade_insert)(uint8_t slot, struct pci_dev **dev);
   int (*blade_remove)(uint8_t slot);
   int (*restore_config)(uint8_t slot);
   int (*upstream_enable_set)(uint8_t slot, uint8_t state);
   int (*upstream_enable_get)(uint8_t slot, uint8_t *state);
   int (*error_control)(uint8_t slot, int enable);
   int (*command)(uint8_t slot, int opcode);

} bhpc_svc_ops;

bhpc_svc_ops *bhpc_driver_register(bhpc_platform *platform);
void bhpc_driver_unregister(bhpc_platform *platform);

int bhpc_irq_init(void (*error_fn)(bhpc_error_data *ddata),
				  bhpc_dsp_map_table *dsp_table);

extern void bhpc_irq_cleanup(void);
extern void bhpc_disable_scan_error_reports(struct pci_dev *dev);
extern void bhpc_enable_scan_error_reports(struct pci_dev *dev);
extern void bhpc_setup_configs(struct pci_dev *dev, int domain, int enable_legacy);

#endif /* __SILKWORM_BHPC_H_ */
#endif /* __KERNEL__ */
