/*
 *
 *    Copyright (c) 1996-2007 Brocade Communications Systems, Inc.
 *    All rights reserved.
 *
 *    Description:
 *      System Inventory (SIN) interfaces and definitions.
 *
 */

#ifndef __SIN_H__
#define	__SIN_H__


#ifdef __cplusplus
extern "C" {
#endif

/* Preprocessor Defines */
                                 
#define SIN_VERSION             1
                                     
#define	SIN_DATA_MAX		64

typedef struct sin_rec {
	unsigned int	 sr_id;
	unsigned int	 sr_pid;
	unsigned int	 sr_flags;
	unsigned int	 sr_class;
	unsigned int	 sr_subclass;
	unsigned char	 sr_data[SIN_DATA_MAX];
} sin_rec_t;

typedef struct sin_stats {
	unsigned int	 ss_version;	/* Implementation version */
	unsigned int	 ss_size;	/* Record size */
	unsigned int	 ss_used;	/* Records in use */
	unsigned int	 ss_total;	/* Total records allowed */
} sin_stats_t;

#define	SIN_AGGREGATE	0x00000001	/* Record is an aggregate object */

/*
 * Many of these values are largely borrowed from PCI. Any other assignment
 * would have been equally as arbitrary.
 */

#define	SIN_CLASS_OTHER				0
#define	SIN_SUBCLASS_OTHER		0

/*
 * Mass storage devices
 */
#define	SIN_CLASS_STORAGE      			1
#define SIN_CLASS_STORAGE_SCS1		1
#define SIN_CLASS_STORAGE_IDE		2
#define SIN_CLASS_STORAGE_FLOPPY	3
#define SIN_CLASS_STORAGE_IPI		4
#define SIN_CLASS_STORAGE_RAID		5

/*
 * Network interfaces
 */
#define SIN_CLASS_NETWORK			2
#define SIN_CLASS_NETWORK_ETHERNET	1
#define SIN_CLASS_NETWORK_TOKEN_RING	2
#define SIN_CLASS_NETWORK_FDDI		3
#define SIN_CLASS_NETWORK_ATM		4

/*
 * Human Interface Devices (HIDs)
 */
#define SIN_CLASS_HID				3
#define	SIN_CLASS_HID_CRT		1
#define	SIN_CLASS_HID_LCD		2
#define	SIN_CLASS_HID_KEYBOARD		3
#define	SIN_CLASS_HID_MOUSE		4
#define	SIN_CLASS_HID_KEYPAD		5

/*
 * Solid-state discrete memory devices
 */
#define SIN_CLASS_MEMORY			5
#define SIN_CLASS_MEMORY_DRAM		1
#define SIN_CLASS_MEMORY_FLASH		2
#define	SIN_CLASS_MEMORY_ROM		3
#define	SIN_CLASS_MEMORY_CACHE		4

/*
 * Cache types
 */
#define SIN_CLASS_MEM_CACHE_DATA	1
#define	SIN_CLASS_MEM_CACHE_INSTR	2
#define	SIN_CLASS_MEM_CACHE_UNIFIED	3

/*
 * Processor bus bridges
 */
#define SIN_CLASS_BRIDGE			6
#define SIN_CLASS_BRIDGE_HOST		1
#define SIN_CLASS_BRIDGE_ISA		2
#define SIN_CLASS_BRIDGE_EISA		3
#define SIN_CLASS_BRIDGE_MC		4
#define SIN_CLASS_BRIDGE_PCI		5
#define SIN_CLASS_BRIDGE_PCMCIA		6
#define SIN_CLASS_BRIDGE_NUBUS		7
#define SIN_CLASS_BRIDGE_CARDBUS	8
#define SIN_CLASS_BRIDGE_RACEWAY	9
#define	SIN_CLASS_BRIDGE_I2C		10

/*
 * Communications channels
 */
#define SIN_CLASS_COMM				7
#define SIN_CLASS_COMM_SERIAL		1
#define SIN_CLASS_COMM_PARALLEL		2
#define SIN_CLASS_COMM_MULTISERIAL	3
#define SIN_CLASS_COMM_MODEM		4

/*
 * System components
 */
#define SIN_CLASS_SYSTEM			8
#define SIN_CLASS_SYSTEM_PIC		1
#define SIN_CLASS_SYSTEM_DMA		2
#define SIN_CLASS_SYSTEM_TIMER		3
#define SIN_CLASS_SYSTEM_RTC		4
#define SIN_CLASS_SYSTEM_SYSCTLR	5
#define SIN_CLASS_SYSTEM_PCI_BASE	6

/*
 * Processor(s)
 */
#define SIN_CLASS_PROCESSOR			9
#define	SIN_CLASS_PROCESSOR_CPU		1
#define	SIN_CLASS_PROCESSOR_FPU		2
#define	SIN_CLASS_PROCESSOR_DSP		3

/*
 * CPU types
 */
#define	SIN_CLASS_PROCESSOR_CPU_ALPHA	1
#define	SIN_CLASS_PROCESSOR_CPU_ARM	2
#define	SIN_CLASS_PROCESSOR_CPU_I960	3
#define	SIN_CLASS_PROCESSOR_CPU_IA64	4
#define	SIN_CLASS_PROCESSOR_CPU_M68K	5
#define	SIN_CLASS_PROCESSOR_CPU_MIPS	6
#define	SIN_CLASS_PROCESSOR_CPU_PARISC	7
#define	SIN_CLASS_PROCESSOR_CPU_POWERPC	8
#define	SIN_CLASS_PROCESSOR_CPU_SPARC	9
#define	SIN_CLASS_PROCESSOR_CPU_SUPERH	10
#define	SIN_CLASS_PROCESSOR_CPU_X86	11

/*
 * Bus devices
 */
#define	SIN_CLASS_BUS				10
#define	SIN_CLASS_BUS_PCI		1
#define	SIN_CLASS_BUS_PLB		2
#define	SIN_CLASS_BUS_I2C		3
#define	SIN_CLASS_BUS_SMB		SIN_CLASS_BUS_I2C

/*
 * System printed circuit boards
 */
#define	SIN_CLASS_BOARD				11
#define	SIN_CLASS_BOARD_PROCESSOR	1
#define	SIN_CLASS_BOARD_IO		2
#define	SIN_CLASS_BOARD_BACKPLANE	3

#define SIN_CLASS_SERIAL			12		
#define SIN_CLASS_SERIAL_FIREWIRE	1
#define SIN_CLASS_SERIAL_ACCESS	        2
#define SIN_CLASS_SERIAL_SSA		3
#define SIN_CLASS_SERIAL_USB		4
#define SIN_CLASS_SERIAL_FIBER		5
#define SIN_CLASS_SERIAL_SMBUS		6

#define	SIN_CLASS_PROM				13


typedef struct sin_mem_data {
	unsigned long	md_base;
	unsigned long	md_size;
} sin_mem_data_t;


sin_rec_t * sin_get(sin_rec_t *srp, int *statep);
int sin_add(sin_rec_t *srp);
int sin_find_sr(int sr_class, int sr_subclass, sin_rec_t* sinrec, int* state);

#ifdef __cplusplus
}
#endif

#endif /* __SIN_H__ */
