/*
 *
 *    Copyright (c) 1996-2010 Brocade Communications Systems, Inc.
 *    All rights reserved.
 *
 *    Description:
 *	Header for Intel/DEC 21554/21555 non-transparent PCI-to-PCI bridge
 *	(aka Drawbridge) device driver.
 *
 */

#ifndef __SYS_DB_DB_H__
#define	__SYS_DB_DB_H__

#ifdef __cplusplus
extern "C" {
#endif

/* Error codes returned by db module */

#define DB_OPERATION_SUCCESSFUL		0
#define DB_INVALID_DEVICE_ID		1 /* Not in 0-31 */
#define DB_DRAWBRIDGE_UNINITIALIZED	2 /* data structs not init'd */
#define DB_BUS_UNAVAILABLE		3 /* Requested bus already in use */
#define DB_BUS_CREATE_FAIL		4 /* Could not allocate mem for bus */
#define DB_DEVICE_EXISTS		5 /* Already in PCI list */
#define DB_ISOLATED_BRIDGE		6 /* Scan found only a bridge */
#define DB_NONBRIDGE_DEVICE		7 /* Did not find a bridge device */
#define DB_EMPTY_BLADE_SLOT		8 /* Scan found nothing */
#define DB_INVALID_PARAMETER		9 /* invalid func. input parameters */

/* Errors we care about during PCI scan */
#define DB_FATAL_ERROR_MASK		(PCI_STATUS_SIG_SYSTEM_ERROR | \
					PCI_STATUS_DETECTED_PARITY)



 /* 36-bit physical base addr of PCI mem */

#define OLYMPUS_PCI_PHYS_BASE	0x380000000ll
#define METEOR_PCI_PHYS_BASE	0x340000000ll
#define SATURN_PCI_PHYS_BASE	0x340000000ll

	/* 32-bit physical base addr of PCI mem for Freescale */

#define NEPTUNE_PCI_PHYS_BASE	0x80000000

  /* 32-bit physical address of 1st BAR of 1st 21555 bridge */

#define DB_OLYMPUS_BASE		0x80000000
#define DB_METEOR_BASE		0x40000000
#define DB_SATURN_BASE		0x40000000


    /* Meteor, Ulysses, Saturn allows 128MB of space per blade slot */

#define DB_ULYSSES_BLADE_ALLOTMENT	0x08000000
#define DB_METEOR_BLADE_ALLOTMENT	0x08000000
#define DB_SATURN_BLADE_ALLOTMENT	0x08000000
#define DB_NEPTUNE_BLADE_ALLOTMENT	0x08000000
#define DB_YODA_BLADE_ALLOTMENT      0x08000000
#define DB_BLADE_ALLOTMENT		DB_ULYSSES_BLADE_ALLOTMENT

    /* Athena allows 64MB of space per blade slot */

#define DB_ATHENA_BLADE_ALLOTMENT	0x04000000

    /* Legacy (Ulysses) systems use fixed 4MB of space per device */

#define DB_SLOT_ALLOTMENT	0x00400000

    /* Legacy (Ulysses) systems allow no more than 32 devices per blade */

#define DB_MAX_DEVICES_PER_BLADE	32

    /* Blade busses start at 2. Busses 0 & 1 are on the CP */

#define DB_BLADE_BUS_OFFSET		2	// Blade busses start at 0x02,
						// 0x03, 0x82, 0x83, etc

	/* Reserve top 1 MB of blade memory range for small BAR allocations */

#define DB_SMALL_BAR_ALLOC_SZ		0x100000

	/* Threshold for small BAR allocations (128K) */

#define DB_SMALL_BAR_ALLOC_THRESH	0x20000


    /*
     * PCI device number of the first PCI device (transparent bridge)
     * on the first physical blade slot in the switch
     */
#define DB_FIRST_BLADE_SLOT		14

#define DB_DRAWBRIDGE1_DEVID		1
#define DB_DRAWBRIDGE2_DEVID		2
#define DB_DRAWBRIDGE3_DEVID		3
#define DB_DRAWBRIDGE4_DEVID		4

#define DB_PCI_SCOMMAND			0x44    // Secondary side command reg
#define DB_PCI_SSTATUS			0x46    // Secondary side status reg
#define DB_PCI_SCD_CACHE_LINE_SIZE	0x4C	// Secondary cache line size
#define DB_PCI_SLATENCY_TIMER		0x4D   // Secondary side latency time

#define DB_PCI_SBASE_CSR_MEM		0x50    // Secondary side CSR Memory Bar
#define DB_PCI_SBASE_CSR_IO		0x54    // Secondary side CSR/IO BAR
#define DB_PCI_SBASE_ADDRESS_0		0x58    // Secondary side Upstream
						// Memory BAR 0
#define DB_PCI_SBASE_ADDRESS_1		0x5C    // Secondary side Upstream
						// Memory BAR 1 */
#define DB_PCI_SBASE_ADDRESS_2		0x60    // Secondary side Upstream
						// Memory BAR 2

#define	DB_PCI_DSTR_CFG_AR		0x80	// Downstream Config. Addr.
#define	DB_PCI_DSTR_CFG_DR		0x84	// Downstream Config. Data
#define	DB_PCI_USTR_CFG_AR		0x88	// Upstream Config. Address
#define	DB_PCI_USTR_CFG_DR		0x8C	// Upstream Config. Data

#define	DB_PCI_CFG_OWN			0x90	// Config. Ownership Bits
#define	  DB_CFG_DSTR_OS		0x0001	// Downstream Ownership
#define	  DB_CFG_USTR_OS		0x0100	// Upstream Ownership
#define	DB_PCI_CFG_CSR			0x92	// Config. Ctrl. and Status
#define	  DB_CFG_DSTR_OC		0x0002	// Downstream Own. Ctrl.
#define	  DB_CFG_USTR_OC		0x0200	// Upstream Own. Ctrl.

#define	DB_PCI_DSTR_XBAR_0		0x94	// Downstream Translated Base 0
#define	DB_PCI_DSTR_XBAR_1		0x98	// Downstream Translated Base 1
#define	DB_PCI_DSTR_XBAR_2		0x9C	// Downstream Translated Base 2
#define	DB_PCI_DSTR_XBAR_3		0xA0	// Downstream Translated Base 3

#define	DB_PCI_USTR_XBAR_0		0xA4	// Upstream Translated Base 0
#define	DB_PCI_USTR_XBAR_1		0xA8	// Upstream Translated Base 1

#define	DB_PCI_DSTR_SETUP_0		0xAC	// Downstream Setup 0
#define	DB_PCI_DSTR_SETUP_1		0xB0	// Downstream Setup 1
#define	DB_PCI_DSTR_SETUP_2		0xB4	// Downstream Setup 2
#define	DB_PCI_DSTR_SETUP_3_LO		0xB8	// Downstream Setup 3 Lower
#define	DB_PCI_DSTR_SETUP_3_HI		0xBC	// Downstream Setup 3 Upper

#define	DB_PCI_XROMSETUP		0xC0	// Primary Expansion ROM Setup

#define	DB_PCI_USTR_SETUP_0		0xC4	// Upstream Setup 0
#define	DB_PCI_USTR_SETUP_1		0xC8	// Upstream Setup 1

#define DB_PCI_CCR0			0xCC	// Chip Control 0
#define	  DB_CCR0_MASTER_ABORT		0x0001	// Master Abort Mode
#define   DB_CCR0_MEMWR_DISC_CTL	0x0002	// Memory Write Disconnect
						// Control
#define   DB_CCR0_PMASTER_TO		0x0004	// Primary Master Timeout
#define   DB_CCR0_SMASTER_TO		0x0008	// Secondary Master Timeout
#define   DB_CCR0_PMASTER_TO_DISABLE   	0x0010	// Primary Master Timeout
						// Disable
#define   DB_CCR0_SMASTER_TO_DISABLE   	0x0020	// Secondary Master Timeout
						// Disable
#define   DB_CCR0_DLYD_XACT_ORDER_CTL	0x0040	// Delayed Transaction
						// Order Control
#define   DB_CCR0_SERR_FWD		0x0080	// SERR# Forward Enable
#define   DB_CCR0_USTR_DAC_PREFETCH    	0x0100	// Upstream DAC Prefetch Dis.
#define   DB_CCR0_MULTI_DEVICE		0x0200	// Multiple Device Enable
#define   DB_CCR0_PRI_ACCESS_DISABLE   	0x0400	// Primary Access Lockout
#define   DB_CCR0_SCLK_DISABLE		0x0800	// Secondary Clock Disable
#define   DB_CCR0_VGA_MODE_MASK		0xC000	// VGA Mode mask
#define   DB_CCR0_VGA_MODE_NONE		0x0000	// Ignore VGA transactions
#define   DB_CCR0_VGA_MODE_PRI_FWD	0x4000	// Forward transactions from
						// primary to secondary
#define   DB_CCR0_VGA_MODE_SEC_FWD	0x8000	// Forward transactions from
						// secondary to primary

#define	DB_PCI_CCR1			0xCE	// Chip Control 1
#define   DB_CCR1_PRI_PSTD_WR_THRESH 0x0001 // Primary Posted Write Threshold
#define   DB_CCR1_SEC_PSTD_WR_THRESH 0x0002 // Secondary Posted Write Threshold
#define   DB_CCR1_PRI_DLYD_RD_THRESH 0x000C // Primary Delayed Read Threshold
#define   DB_CCR1_SEC_DLYD_RD_THRESH 0x0030 // Secondary Delayed WriteThreshold
#define   DB_CCR1_SUB_DECODE	0x00C0	// Subtractive Decode Enable mask
#define   DB_CCR1_SUB_DECODE_NONE 0x0000	// No subtractive decoding
#define   DB_CCR1_SUB_DECODE_PRI 0x0040	// Subtractive decoding on pri. bus
#define   DB_CCR1_SUB_DECODE_SEC 0x0080	// Subtractive decoding on sec. bus
#define   DB_CCR1_PAGE_SIZE		0x0F00	/* Page Size mask */
#define	    DB_CCR1_PAGE_SIZE_NONE	0x0000	/* Disable Upstream Mem BAR2 */
#define	    DB_CCR1_PAGE_SIZE_256B	0x0100	/* 256 bytes */
#define	    DB_CCR1_PAGE_SIZE_512B	0x0200	/* 512 bytes */
#define	    DB_CCR1_PAGE_SIZE_1KB	0x0300	/* 1 KB */
#define	    DB_CCR1_PAGE_SIZE_2KB	0x0400	/* 2 KB */
#define	    DB_CCR1_PAGE_SIZE_4KB	0x0500	/* 4 KB */
#define	    DB_CCR1_PAGE_SIZE_8KB	0x0600	/* 8 KB */
#define	    DB_CCR1_PAGE_SIZE_16KB	0x0700	/* 16 KB */
#define	    DB_CCR1_PAGE_SIZE_32KB	0x0800	/* 32 KB */
#define	    DB_CCR1_PAGE_SIZE_64KB	0x0900	/* 64 KB */
#define	    DB_CCR1_PAGE_SIZE_128KB	0x0A00	/* 128 KB */
#define	    DB_CCR1_PAGE_SIZE_256KB	0x0B00	/* 256 KB */
#define	    DB_CCR1_PAGE_SIZE_512KB	0x0C00	/* 512 KB */
#define	    DB_CCR1_PAGE_SIZE_1MB	0x0D00	/* 1 MB */
#define	    DB_CCR1_PAGE_SIZE_2MB	0x0E00	/* 2 MB */
#define	    DB_CCR1_PAGE_SIZE_4MB	0x0F00	/* 4 MB */

#define   DB_CCR1_I2O_ENABLE   		0x1000	/* I2O Message Unit Enable */
#define   DB_CCR1_I2O_SIZE		0xE000	/* I2O FIFO Size */
#define	    DB_CCR_I20_SIZE_256		0x0000	/* 256 entries */
#define	    DB_CCR_I20_SIZE_512		0x2000	/* 512 entries */
#define	    DB_CCR_I20_SIZE_1K		0x4000	/* 1K entries */
#define	    DB_CCR_I20_SIZE_2K		0x6000	/* 2K entries */
#define	    DB_CCR_I20_SIZE_4K		0x8000	/* 4K entries */
#define	    DB_CCR_I20_SIZE_8K		0xA000	/* 8K entries */
#define	    DB_CCR_I20_SIZE_16K		0xC000	/* 16K entries */
#define	    DB_CCR_I20_SIZE_32K		0xE000	/* 32K entries */

#define DB_PCI_CSR		0xD0	/* Chip Status */

#define DB_PCI_ARB_CR		0xD2	/* Arbiter Control */

#define DB_PCI_PSERR		0xD4	/* Primary SERR# Disables */
#define DB_PCI_SSERR		0xD6	/* Secondary SERR# Disables */

#define DB_PCI_RESET_CTL	0xD8	/* Reset Control */
#define   DB_RESET_CTL_SEC	0x01	/* Secondary bus reset */
#define   DB_RESET_CTL_CHIP	0x02	/* Chip reset */
#define   DB_RESET_CTL_SUB_STAT	0x04	/* Subsystem status via s_pme_l pin */
#define   DB_RESET_CTL_L_STAT	0x08	/* Status of the l_stat pin */


/* Memory-mapped Control and Status Registers Offsets */

#define	DB_CSR_DSTR_CFG_AR	0x00	/* Downstream Configuration Address */
#define	DB_CSR_DSTR_CFG_DR	0x04	/* Downstream Configuration Data */
#define	DB_CSR_USTR_CFG_AR	0x08	/* Upstream Configuration Address */
#define	DB_CSR_USTR_CFG_DR	0x0C	/* Upstream Configuration Data */

#define	DB_CSR_CFG_OWN		0x10	/* Configuration Ownership Bits */
#define	DB_CSR_CFG_CTRL		0x12	/* Configuration Control and Status */

#define	DB_CSR_DSTR_IO_AR	0x14	/* Downstream I/O Address */
#define	DB_CSR_DSTR_IO_DR	0x18	/* Downstream I/O Data */
#define	DB_CSR_USTR_IO_AR	0x1C	/* Upstream I/O Address */
#define	DB_CSR_USTR_IO_DR	0x20	/* Upstream I/O Data */

#define	DB_CSR_IO_OWN		0x24	/* I/O Ownership Bits */
#define	DB_CSR_IO_CSR		0x26	/* I/O Control and Status */

#define	DB_CSR_LUTOFF		0x28	/* Lookup Table Offset */
#define	DB_CSR_LUTDATA		0x2C	/* Lookup Table Data */

#define DB_CSR_PRI_INT_CLR	0x98    /* Primary Clear IRQ - 16 bits */
#define DB_CSR_SEC_INT_CLR	0x9A    /* Secondary Clear IRQ - 16 bits */

#define DB_CSR_PRI_INT_SET	0x9C    /* Primary Set IRQ - 16 bits */
#define DB_CSR_SEC_INT_SET	0x9E    /* Secondary Set IRQ - 16 bits */

#define DB_CSR_PRI_INT_CLR_MASK 0xA0    /* Primary Clear IRQ mask - 16 bits */
#define DB_CSR_SEC_INT_CLR_MASK 0xA2    /* Secondary Clear IRQ mask - 16 bits */

#define DB_CSR_PRI_INT_SET_MASK 0xA4    /* Primary Set IRQ mask - 16 bits */
#define DB_CSR_SEC_INT_SET_MASK 0xA6    /* Secondary Set IRQ mask - 16 bits */


/* Type Definitions */

typedef	uint8_t	byte;
typedef uint16_t word;
typedef uint32_t dword;


/*
 * 21554 "Drawbridge" Memory-mapped Configuration and Status Registers
 */
typedef struct db_csr_s {
	uint32_t	db_dstr_cfg_ar;	/* Downstream Config. Addr. */
	uint32_t	db_dstr_cfg_dr;	/* Downstream Config. Data */
	uint32_t	db_ustr_cfg_ar;	/* Upstream Config. Addr. */
	uint32_t	db_ustr_cfg_dr;	/* Upstream Config. Data */
	uint16_t	db_cfg_own;		/* Configuration Ownership */
	uint16_t	db_cfg_ctrl;		/* Config. Control & Status */
	uint32_t	db_dstr_io_ar;		/* Downstream I/O Addr. */
	uint32_t	db_dstr_io_dr;		/* Downstream I/O Data */
	uint32_t	db_ustr_io_ar;		/* Upstream I/O Addr. */
	uint32_t	db_ustr_io_dr;		/* Upstream I/O Data */
	uint16_t	db_io_own;		/* I/O Ownership */
	uint16_t	db_io_ctrl;		/* I/O Control & Status */
	uint8_t		db_lut_off;		/* Lookup Table Offset */
	uint8_t		db_reserved0[3];
	uint32_t	db_lut_data;		/* Lookup Table Data */
	struct {
		uint32_t outbd_plist_status;	/* I20 Out Post_List Status */
		uint32_t outbd_plist_intmask;	/* I20 Out Post_List Int Msk */
		uint32_t inbd_plist_status;	/* I20 In Post_List Status */
		uint32_t inbd_plist_intmask;	/* I20 In Post_List Int Mask */
		uint32_t inbd_queue;		/* I2O Inbound Queue */
		uint32_t outbd_queue;		/* I20 Outbound Queue */
		uint32_t inbd_flist_head;	/* I20 In Free_List Head */
		uint32_t inbd_plist_tail;	/* I20 In Post_List Tail */
		uint32_t outbd_flist_tail;	/* I20 Out Free_List Tail */
		uint32_t outbd_plist_head;	/* I20 Out Post_List Head */
		uint32_t inbd_plist_cntr;	/* I20 In Post_List Counter */
		uint32_t inbd_flist_cntr;	/* I20 In Free_List Counter */
		uint32_t outbd_plist_cntr;	/* I20 Out Post_List Counter */
		uint32_t outbd_flist_cntr;	/* I20 Out Free_List Counter */
	} db_i2o;
	uint32_t	db_dstr_xbar[4];	/* Dnstr. Memory Xlated BARs */
	uint32_t	db_ustr_xbar[2];	/* Upstr. Memory Xlated BARs */
	uint16_t	db_reserved1;
	uint16_t	db_chip_status;	/* Chip Status */
	uint16_t	db_chip_intmask_set;	/* Chip Set Interrupt Mask */
	uint16_t	db_chip_intmask_clr;	/* Chip Clear Interrupt Mask */
	uint32_t	db_ustr_pgbounds_int[2];
	uint32_t	db_ustr_pgbounds_mask[2];
	uint16_t	db_pri_int_clr;	/* Pri. Interrupt Clear */
	uint16_t	db_sec_int_clr;	/* Sec. Interrupt Clear */
	uint16_t	db_pri_int_set;	/* Pri. Interrupt Set */
	uint16_t	db_sec_int_set;	/* Sec. Interrupt Set */
	uint16_t	db_pri_int_clr_mask;	/* Pri. Interrupt Clear Mask */
	uint16_t	db_sec_int_clr_mask;	/* Sec. Interrupt Clear Mask */
	uint16_t	db_pri_int_set_mask;	/* Pri. Interrupt Set Mask */
	uint16_t	db_sec_int_set_mask;	/* Sec. Interrupt Set Mask */
	uint32_t	db_scratchpad[8];	/* Scratchpad Registers */
	uint16_t	db_rom_setup;		/* ROM Setup */
	uint8_t		db_rom_data;		/* ROM Data */
	uint8_t		db_reserved2;
	uint32_t	db_rom_addr_ctl;	/* ROM Address & Control */
	uint32_t	db_reserved3[12];
	uint32_t	db_ustr_lut[64];	/* Upstream Lookup Table */
	uint32_t	db_reserved4[896];
} db_csr_t;

struct pci_dev;
struct resource;

/*
 * The callback to find db.
 */
typedef void(*db_find_t)(int, int, struct pci_dev **);

/*
 * Client interface to perform deployment-specific functions
 *    e.g. obtain BAR size
 */

typedef enum db_usr_cmd {
	DB_USR_CMD_GET_DEV_ATTR = 0,
	DB_USR_CMD_MAXP1
} db_usr_cmd_t;

/* data type for DB_USR_CMD_GET_DEV_ATTR */
typedef struct db_usr_cmd_get_dev_attr {
	unsigned int	bus;					/* out */
	unsigned int	devfn;					/* out */
	int				num_bars;				/* out and in */
	unsigned short	vendor;					/* in */
	unsigned short	device;					/* in */
#define DB_USR_MAX_BARS	6					/* must equal DB_MAX_BARS */
	unsigned long	bar_sz[DB_USR_MAX_BARS];	/* in */
	unsigned long	bar_attr[DB_USR_MAX_BARS];	/* in */
} db_usr_cmd_get_dev_attr_t;

typedef int(*db_usr_ctrl_func_t)(void *usr, db_usr_cmd_t cmd, void *data);
typedef	struct	db_usr_if_s {
	db_usr_ctrl_func_t	ctrl_func;
	void				*data;
} db_usr_if_t;

void db_dump_csr(struct pci_dev *dev, db_csr_t *buf);
int db_reset_drawbridge(void);
int db_scan_drawbridge(struct pci_dev *dev, int recursive);
int db_remove_tree(struct pci_dev *dp);
db_csr_t* db_get_csrbase(int bridge, int local);
uint32_t db_get_irq(int bridge);
int db_pci_primary_reset(void);
int db_drawbridge_secondary_reset(int bridge);
void db_bridge_secondary_reset(struct pci_dev *bridge);
int db_secondary_reset_control(int bridge, int assert);
int db_pci_chip_reset(struct pci_dev *dp);
void db_remove_child_devices(struct pci_dev *dp);
void get_memory_limits(unsigned long *installed_ram,
			 unsigned long *avail_ram);
int db_scan_blade(
		struct pci_dev *drawbridge, struct pci_dev **dev, int devid);
int db_scan_blade_noreconfig(
		struct pci_dev *drawbridge, struct pci_dev **dev, int devid,
		db_usr_if_t *usr_if);
struct pci_bus *db_bus_exists(struct pci_bus *bus, int nr);
int db_set_cache_line_size(int bridge, u8 size);
int db_get_drawbridge_status(struct pci_dev *bridge, int *primary_status,
			     int *secondary_status);
int db_restore_drawbridge(int drawbridge);
int db_timeout_restore_drawbridge(struct pci_dev *drawbridge);
int db_get_forwarding_window(int bridge, struct resource *rsrc);
int db_add_drawbridge_subordinate(struct pci_dev *drawbridge);
int db_secondary_mem_enable(int enable);
int db_is_secondary_mem_enabled(struct pci_dev *dp);
void db_register_find_db(db_find_t);

int db_devnum_to_idx(int devnum);
int db_idx_to_devnum(int index);

    /*
     * Functions no longer supported. API is still
     * present, otherwise some modules will not load.
     * All of these now return -ENOSYS.
     */
uint32_t db_get_dma_membase(int devnum, int virtual);
uint32_t db_get_dma_memsize(int devnum);
uint32_t db_get_dma_pcibase(int devnum);
int db_set_doorbell(u_int door, int bridge, int local, int state);
int db_get_doorbell(int bridge, int local, u16 *irq);
int db_mailbox_get(int bridge, int local, int mailbox, int *data);
int db_mailbox_send(int bridge, int local, int mailbox, int data);

#ifdef __cplusplus
}
#endif

#endif /* __SYS_DB_DB_H__ */
