/* This is the header files for the 'virtual i2c' adapter driver.
 * See Documentation/i2c/virtual_i2c for usage information.
 */
#ifndef __I2C_VIRTUAL_H
#define __I2C_VIRTUAL_H

#include <linux/i2c.h>

#define I2C_CAP_FSM_400KHZ 0x1

struct i2c_mux_ctrl {
	unsigned long	addr;		
	unsigned long	value;

	unsigned int write_enable; /* Indicate if write enable needs to be managed */

	int	 id; /* For error messages. ID's the hardware having the problem */

	/* fn which enables the mux */
	int (*select)(struct i2c_adapter *adap, struct i2c_mux_ctrl *mux);

	/* fn which disables the mux */
	int (*deselect)(struct i2c_adapter *adap, struct i2c_mux_ctrl *mux);
};

/* This has to be exposed, since the code which assigns the callbacks must use it. */
struct i2c_virt_priv {
	struct i2c_adapter	*real_adap;	/* 'real' adapter */
	struct i2c_adapter	*phys_adap;	/* 'root'- hw adapter */
	struct i2c_mux_ctrl	mux;		/* MUX settings */
	struct i2c_statistics	stats;		/* virt adap Statistics */
	unsigned int		i2c_cap;
};


static inline struct i2c_adapter *i2c_get_real_adap(struct i2c_adapter *adap)
{
	struct i2c_virt_priv *priv;

	if((adap->id & 0xff) != I2C_HW_VIRT)
		return NULL;
	
	priv = i2c_get_adapdata(adap);
	return(priv->real_adap);
}

static inline struct i2c_adapter *i2c_get_phys_adap(struct i2c_adapter *adap)
{
	struct i2c_virt_priv *priv;

	if((adap->id & 0xff) != I2C_HW_VIRT)
		return NULL;
	
	priv = i2c_get_adapdata(adap);
	if (priv)
		return(priv->phys_adap);
	else
		return NULL;
}

static inline int i2c_set_fsm(struct i2c_adapter *adap, int fsm)
{
	struct i2c_virt_priv *priv;

	if((adap->id & 0xff) != I2C_HW_VIRT)
		return -1;
	
	priv = i2c_get_adapdata(adap);
	if (priv) {
                if (fsm)
                        priv->i2c_cap |= I2C_CAP_FSM_400KHZ;
                else
                        priv->i2c_cap &= ~I2C_CAP_FSM_400KHZ;
	}
	else
		return -1;

	return (0);
}

int unregister_virtual_i2c_adap(struct i2c_adapter *adap);
struct i2c_adapter
*register_virtual_i2c_adap(struct i2c_adapter *real_adap, 
						   unsigned long mux_addr, 
						   unsigned long mux_val, 
						   void *select_cb,
						   void *deselect_cb,
						   int id);

#endif /* __I2C_VIRTUAL_H */
